@@ -54,6 +54,7 @@
 #include "brasero-jacket-edit.h"
 
 #include "burn-basics.h"
+#include "burn-debug.h"
 #include "burn-session.h"
 #include "burn-medium.h"
 #include "brasero-drive-selection.h"
@@ -92,7 +93,13 @@ struct BraseroBurnDialogPrivate {
 	GtkWidget *image;
 	BraseroTrayIcon *tray;
 
+	/* for our final statistics */
+	GTimer *total_time;
+	gint64 total_size;
+	GSList *rates;
+
 	guint is_writing:1;
+	guint is_creating_image:1;
 };
 
 #define TIMEOUT	10000
@@ -144,7 +151,7 @@ brasero_burn_dialog_class_init (BraseroBurnDialogClass * klass)
 
 static void
 brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
-				 BraseroTrackDataType input,
+				 BraseroTrackType *input,
 				 BraseroMedia media)
 {
 	gchar *title = NULL;
@@ -162,7 +169,21 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 		title = g_strdup (_("Brasero - Creating Image"));
 	}
 	else if (media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL)) {
-		if (input == BRASERO_TRACK_TYPE_DATA) {
+		if (BRASERO_TRACK_TYPE_HAS_VIDEO (input)) {
+			if (flags & BRASERO_BURN_FLAG_DUMMY) {
+				title = g_strdup (_("Brasero - Burning DVD (Simulation)"));
+				header = g_strdup (_("<big><b>Simulation of video DVD burning</b></big>"));
+			}
+			else {
+				title = g_strdup (_("Brasero - Burning DVD"));
+				header = g_strdup (_("<big><b>Burning video DVD</b></big>"));
+			}
+
+			gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+						      "media-optical-video-new",
+						      GTK_ICON_SIZE_DIALOG);
+		}
+		else if (input->type == BRASERO_TRACK_TYPE_DATA) {
 			if (flags & BRASERO_BURN_FLAG_DUMMY) {
 				title = g_strdup (_("Brasero - Burning DVD (Simulation)"));
 				header = g_strdup (_("<big><b>Simulation of data DVD burning</b></big>"));
@@ -176,7 +197,7 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 						      "media-optical-data-new",
 						      GTK_ICON_SIZE_DIALOG);
 		}
-		else if (input == BRASERO_TRACK_TYPE_IMAGE) {
+		else if (input->type == BRASERO_TRACK_TYPE_IMAGE) {
 			if (flags & BRASERO_BURN_FLAG_DUMMY) {
 				title = g_strdup (_("Burning DVD (Simulation)"));
 				header = g_strdup (_("<big><b>Simulation of image to DVD burning</b></big>"));
@@ -190,7 +211,7 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 						      "media-optical",
 						      GTK_ICON_SIZE_DIALOG);
 		}
-		else if (input == BRASERO_TRACK_TYPE_DISC) {
+		else if (input->type == BRASERO_TRACK_TYPE_DISC) {
 			if (flags & BRASERO_BURN_FLAG_DUMMY) {
 				title = g_strdup (_("Brasero - Copying DVD (Simulation)"));
 				header = g_strdup (_("<big><b>Simulation of data DVD copying</b></big>"));
@@ -201,12 +222,26 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 			}
 
 			gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
-						"media-optical-copy",
-						GTK_ICON_SIZE_DIALOG);
+						      "media-optical-copy",
+						      GTK_ICON_SIZE_DIALOG);
 		}
 	}
 	else if (media & BRASERO_MEDIUM_CD) {
-		if (input == BRASERO_TRACK_TYPE_AUDIO) {
+		if (BRASERO_TRACK_TYPE_HAS_VIDEO (input)) {
+			if (flags & BRASERO_BURN_FLAG_DUMMY) {
+				title = g_strdup (_("Brasero - Burning CD (Simulation)"));
+				header = g_strdup (_("<big><b>Simulation of (S)VCD burning</b></big>"));
+			}
+			else {
+				title = g_strdup (_("Brasero - Burning CD"));
+				header = g_strdup (_("<big><b>Burning (S)VCD</b></big>"));
+			}
+
+			gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+						      "drive-removable-media",
+						      GTK_ICON_SIZE_DIALOG);
+		}
+		else if (input->type == BRASERO_TRACK_TYPE_AUDIO) {
 			if (flags & BRASERO_BURN_FLAG_DUMMY) {
 				title = g_strdup (_("Brasero - Burning CD (Simulation)"));
 				header = g_strdup (_("<big><b>Simulation of audio CD burning</b></big>"));
@@ -220,7 +255,7 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 						      "media-optical-audio-new",
 						      GTK_ICON_SIZE_DIALOG);
 		}
-		else if (input == BRASERO_TRACK_TYPE_DATA) {
+		else if (input->type == BRASERO_TRACK_TYPE_DATA) {
 			if (flags & BRASERO_BURN_FLAG_DUMMY) {
 				title = g_strdup (_("Brasero - Burning CD (Simulation)"));
 				header = g_strdup (_("<big><b>Simulation of data CD burning</b></big>"));
@@ -234,7 +269,7 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 						      "media-optical-data-new",
 						      GTK_ICON_SIZE_DIALOG);
 		}
-		else if (input == BRASERO_TRACK_TYPE_DISC) {
+		else if (input->type == BRASERO_TRACK_TYPE_DISC) {
 			if (flags & BRASERO_BURN_FLAG_DUMMY) {
 				title = g_strdup (_("Brasero - Copying CD (Simulation)"));
 				header = g_strdup(_("<big><b>Simulation of CD copying</b></big>"));
@@ -248,7 +283,7 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 						      "media-optical-copy",
 						      GTK_ICON_SIZE_DIALOG);
 		}
-		else if (input == BRASERO_TRACK_TYPE_IMAGE) {
+		else if (input->type == BRASERO_TRACK_TYPE_IMAGE) {
 			if (flags & BRASERO_BURN_FLAG_DUMMY) {
 				title = g_strdup (_("Brasero - Burning CD (Simulation)"));
 				header = g_strdup (_("<big><b>Simulation of image to CD burning</b></big>"));
@@ -263,7 +298,21 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 						      GTK_ICON_SIZE_DIALOG);
 		}
 	}
-	else if (input == BRASERO_TRACK_TYPE_AUDIO) {
+	else if (BRASERO_TRACK_TYPE_HAS_VIDEO (input)) {
+		if (flags & BRASERO_BURN_FLAG_DUMMY) {
+			title = g_strdup (_("Brasero - Burning disc (Simulation)"));
+			header = g_strdup (_("<big><b>Simulation of video disc burning</b></big>"));
+		}
+		else {
+			title = g_strdup (_("Brasero - Burning disc"));
+			header = g_strdup (_("<big><b>Burning video disc</b></big>"));
+		}
+
+		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+					      "drive-removable-media",
+					      GTK_ICON_SIZE_DIALOG);
+	}
+	else if (input->type == BRASERO_TRACK_TYPE_AUDIO) {
 		if (flags & BRASERO_BURN_FLAG_DUMMY) {
 			title = g_strdup (_("Brasero - Burning CD (Simulation)"));
 			header = g_strdup (_("<big><b>Simulation of audio CD burning</b></big>"));
@@ -274,10 +323,10 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 		}
 
 		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
-					      "gnome-dev-removable",
+					      "drive-removable-media",
 					      GTK_ICON_SIZE_DIALOG);
 	}
-	else if (input == BRASERO_TRACK_TYPE_DATA) {
+	else if (input->type == BRASERO_TRACK_TYPE_DATA) {
 		if (flags & BRASERO_BURN_FLAG_DUMMY) {
 			title = g_strdup (_("Brasero - Burning Disc (Simulation)"));
 			header = g_strdup (_("<big><b>Simulation of data disc burning</b></big>"));
@@ -288,10 +337,10 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 		}
 
 		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
-					      "gnome-dev-removable",
+					      "drive-removable-media",
 					      GTK_ICON_SIZE_DIALOG);
 	}
-	else if (input == BRASERO_TRACK_TYPE_DISC) {
+	else if (input->type == BRASERO_TRACK_TYPE_DISC) {
 		if (flags & BRASERO_BURN_FLAG_DUMMY) {
 			title = g_strdup (_("Brasero - Copying Disc (Simulation)"));
 			header = g_strdup(_("<big><b>Simulation of disc copying</b></big>"));
@@ -301,10 +350,10 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 			header = g_strdup(_("<big><b>Copying disc</b></big>"));
 		}
 		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
-					      "gnome-dev-removable",
+					      "drive-removable-media",
 					      GTK_ICON_SIZE_DIALOG);
 	}
-	else if (input == BRASERO_TRACK_TYPE_IMAGE) {
+	else if (input->type == BRASERO_TRACK_TYPE_IMAGE) {
 		if (flags & BRASERO_BURN_FLAG_DUMMY) {
 			title = g_strdup (_("Brasero - Burning Disc (Simulation)"));
 			header = g_strdup (_("<big><b>Simulation of image to disc burning</b></big>"));
@@ -315,7 +364,7 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog,
 		}
 
 		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
-					      "gnome-dev-removable",
+					      "drive-removable-media",
 					      GTK_ICON_SIZE_DIALOG);
 	}
 
@@ -343,15 +392,15 @@ brasero_burn_dialog_get_media_type_string (BraseroBurn *burn,
 	if (type & BRASERO_MEDIUM_HAS_DATA) {
 		if (!insert) {
 			if (type & BRASERO_MEDIUM_REWRITABLE)
-				message = g_strdup (_("replace the disc with a rewritable disc holding data."));
+				message = g_strdup (_("Please, replace the disc with a rewritable disc holding data."));
 			else
-				message = g_strdup (_("replace the disc with a disc holding data."));
+				message = g_strdup (_("Please, replace the disc with a disc holding data."));
 		}
 		else {
 			if (type & BRASERO_MEDIUM_REWRITABLE)
-				message = g_strdup (_("insert a rewritable disc holding data."));
+				message = g_strdup (_("Please, insert a rewritable disc holding data."));
 			else
-				message = g_strdup (_("insert a disc holding data."));
+				message = g_strdup (_("Please, insert a disc holding data."));
 		}
 	}
 	else if (type & BRASERO_MEDIUM_WRITABLE) {
@@ -366,48 +415,48 @@ brasero_burn_dialog_get_media_type_string (BraseroBurn *burn,
 		if ((type & BRASERO_MEDIUM_CD) && !(type & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))) {
 			if (!insert) {
 				if (isosize)
-					message = g_strdup_printf (_("replace the disc with a recordable CD with a least %i MiB free."), 
+					message = g_strdup_printf (_("Please, replace the disc with a recordable CD with at least %i MiB of free space."), 
 								   (int) (isosize / 1048576));
 				else
-					message = g_strdup (_("replace the disc with a recordable CD."));
+					message = g_strdup (_("Please, replace the disc with a recordable CD."));
 			}
 			else {
 				if (isosize)
-					message = g_strdup_printf (_("insert a recordable CD with a least %i MiB free."), 
+					message = g_strdup_printf (_("Please, insert a recordable CD with at least %i MiB of free space."), 
 								   (int) (isosize / 1048576));
 				else
-					message = g_strdup (_("insert a recordable CD."));
+					message = g_strdup (_("Please, insert a recordable CD."));
 			}
 		}
 		else if (!(type & BRASERO_MEDIUM_CD) && (type & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))) {
 			if (!insert) {
 				if (isosize)
-					message = g_strdup_printf (_("replace the disc with a recordable DVD with a least %i MiB free."), 
+					message = g_strdup_printf (_("Please, replace the disc with a recordable DVD with at least %i MiB of free space."), 
 								   (int) (isosize / 1048576));
 				else
-					message = g_strdup (_("replace the disc with a recordable DVD."));
+					message = g_strdup (_("Please, replace the disc with a recordable DVD."));
 			}
 			else {
 				if (isosize)
-					message = g_strdup_printf (_("insert a recordable DVD with a least %i MiB free."), 
+					message = g_strdup_printf (_("Please, insert a recordable DVD with at least %i MiB of free space."), 
 								   (int) (isosize / 1048576));
 				else
-					message = g_strdup (_("insert a recordable DVD."));
+					message = g_strdup (_("Please, insert a recordable DVD."));
 			}
 		}
 		else if (!insert) {
 			if (isosize)
-				message = g_strdup_printf (_("replace the disc with a recordable CD or DVD with a least %i MiB free."), 
+				message = g_strdup_printf (_("Please, replace the disc with a recordable CD or DVD with at least %i MiB of free space."), 
 							   (int) (isosize / 1048576));
 			else
-				message = g_strdup (_("replace the disc with a recordable CD or DVD."));
+				message = g_strdup (_("Please, replace the disc with a recordable CD or DVD."));
 		}
 		else {
 			if (isosize)
-				message = g_strdup_printf (_("insert a recordable CD or DVD with a least %i MiB free."), 
+				message = g_strdup_printf (_("Please, insert a recordable CD or DVD with at least %i MiB of free space."), 
 							   (int) (isosize / 1048576));
 			else
-				message = g_strdup (_("insert a recordable CD or DVD."));
+				message = g_strdup (_("Please, insert a recordable CD or DVD."));
 		}
 	}
 
@@ -445,68 +494,62 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 		hide = TRUE;
 	}
 
+	g_timer_stop (dialog->priv->total_time);
+
 	if (drive)
 		drive_name = brasero_drive_get_display_name (drive);
 	else
 		drive_name = NULL;
 
 	if (error == BRASERO_BURN_WARNING_INSERT_AFTER_COPY) {
-		main_message = g_strdup (_("An image of the disc has been created on your hard drive. Burning is about to begin:"));
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
+		secondary_message = g_strdup (_("An image of the disc has been created on your hard drive."
+						"\nBurning will begin as soon as a recordable medium is inserted."));
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
 	else if (error == BRASERO_BURN_WARNING_CHECKSUM) {
-		main_message = g_strdup (_("A data integrity test is about to begin:"));
-		secondary_message = g_strdup (_("Please, re-insert the disc in the CD/DVD burner."));
+		secondary_message = g_strdup (_("A data integrity test will begin as soon as the medium is inserted."));
+		main_message = g_strdup (_("Please, re-insert the disc in the CD/DVD burner."));
 	}
 	else if (error == BRASERO_BURN_WARNING_NEXT_COPY) {
-		main_message = g_strdup (_("A recording was successfully completed."
-					   "\nThe next recording is about to begin:"));
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
+		secondary_message = g_strdup (_("A recording was successfully completed."
+						"\nThe next recording will begin as soon as a recordable medium is inserted."));
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
 	}
 	else if (error == BRASERO_BURN_ERROR_MEDIA_BUSY) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" is busy:"),
-						drive_name);
+		main_message = g_strdup_printf (_("The disc in \"%s\" is busy."), drive_name);
 		secondary_message = g_strdup (_("make sure another application is not using it."));
 	} 
 	else if (error == BRASERO_BURN_ERROR_MEDIA_NONE) {
-		main_message = g_strdup_printf (_("There is no disc in \"%s\":"),
-						drive_name);
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
+		secondary_message = g_strdup_printf (_("There is no disc in \"%s\"."), drive_name);
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
 	}
 	else if (error == BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" is not supported:"),
-						drive_name);
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
+		secondary_message = g_strdup_printf (_("The disc in \"%s\" is not supported."), drive_name);
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
 	}
 	else if (error == BRASERO_BURN_ERROR_MEDIA_NOT_REWRITABLE) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" is not rewritable:"),
-						drive_name);
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
+		secondary_message = g_strdup_printf (_("The disc in \"%s\" is not rewritable."), drive_name);
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
 	else if (error == BRASERO_BURN_ERROR_MEDIA_BLANK) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" is empty:"),
-						drive_name);
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
+		secondary_message = g_strdup_printf (_("The disc in \"%s\" is empty."), drive_name);
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
 	else if (error == BRASERO_BURN_ERROR_MEDIA_NOT_WRITABLE) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" is not writable:"),
-						drive_name);
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
+		secondary_message = g_strdup_printf (_("The disc in \"%s\" is not writable."), drive_name);
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
 	else if (error == BRASERO_BURN_ERROR_DVD_NOT_SUPPORTED) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" is a DVD:"),
-						drive_name);
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
+		secondary_message = g_strdup_printf (_("The disc in \"%s\" is a DVD."), drive_name);
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
 	else if (error == BRASERO_BURN_ERROR_CD_NOT_SUPPORTED) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" is a CD:"),
-						drive_name);
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
+		secondary_message = g_strdup_printf (_("The disc in \"%s\" is a CD."), drive_name);
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
 	else if (error == BRASERO_BURN_ERROR_MEDIA_SPACE) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" is not big enough:"),
-						drive_name);
-		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
+		secondary_message = g_strdup_printf (_("The disc in \"%s\" is not big enough."), drive_name);
+		main_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
 	}
 	else if (error == BRASERO_BURN_ERROR_NONE) {
 		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, FALSE);
@@ -515,9 +558,8 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 		secondary_message = NULL;
 	}
 	else if (error == BRASERO_BURN_ERROR_RELOAD_MEDIA) {
-		main_message = g_strdup_printf (_("The disc in \"%s\" needs to be reloaded:"),
-						drive_name);
-		secondary_message = g_strdup (_("eject the disc and reload it."));
+		secondary_message = g_strdup_printf (_("The disc in \"%s\" needs to be reloaded."), drive_name);
+		main_message = g_strdup (_("Please, eject the disc and reload it."));
 	}
 
 	g_free (drive_name);
@@ -568,17 +610,19 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 		src = brasero_burn_session_get_src_drive (dialog->priv->session);
 		if (drive == src)
 			brasero_burn_dialog_update_info (dialog,
-							 dialog->priv->input.type, 
+							 &dialog->priv->input, 
 							 brasero_medium_get_status (medium));
 	}
 	else
 		brasero_burn_dialog_update_info (dialog,
-						 dialog->priv->input.type, 
+						 &dialog->priv->input, 
 						 brasero_medium_get_status (medium));
 
 	if (hide)
 		gtk_widget_hide (GTK_WIDGET (dialog));
 
+	g_timer_start (dialog->priv->total_time);
+
 	if (result != GTK_RESPONSE_OK)
 		return BRASERO_BURN_CANCEL;
 
@@ -604,6 +648,8 @@ brasero_burn_dialog_loss_warnings_cb (GtkDialog *dialog,
 		hide = TRUE;
 	}
 
+	g_timer_stop (BRASERO_BURN_DIALOG (dialog)->priv->total_time);
+
 	window = GTK_WINDOW (dialog);
 	message = gtk_message_dialog_new (window,
 					  GTK_DIALOG_DESTROY_WITH_PARENT|
@@ -643,6 +689,8 @@ brasero_burn_dialog_loss_warnings_cb (GtkDialog *dialog,
 	if (hide)
 		gtk_widget_hide (GTK_WIDGET (dialog));
 
+	g_timer_start (BRASERO_BURN_DIALOG (dialog)->priv->total_time);
+
 	if (result == GTK_RESPONSE_ACCEPT)
 		return BRASERO_BURN_NEED_RELOAD;
 
@@ -715,6 +763,8 @@ brasero_burn_dialog_disable_joliet_cb (BraseroBurn *burn,
 		hide = TRUE;
 	}
 
+	g_timer_stop (BRASERO_BURN_DIALOG (dialog)->priv->total_time);
+
 	window = GTK_WINDOW (dialog);
 	message = gtk_message_dialog_new (window,
 					  GTK_DIALOG_DESTROY_WITH_PARENT|
@@ -745,6 +795,8 @@ brasero_burn_dialog_disable_joliet_cb (BraseroBurn *burn,
 	if (hide)
 		gtk_widget_hide (GTK_WIDGET (dialog));
 
+	g_timer_start (BRASERO_BURN_DIALOG (dialog)->priv->total_time);
+
 	if (result != GTK_RESPONSE_OK)
 		return BRASERO_BURN_CANCEL;
 
@@ -753,7 +805,7 @@ brasero_burn_dialog_disable_joliet_cb (BraseroBurn *burn,
 
 static void
 brasero_burn_dialog_update_title_writing_progress (BraseroBurnDialog *dialog,
-						   BraseroTrackDataType input,
+						   BraseroTrackType *input,
 						   BraseroMedia media,
 						   guint percent)
 {
@@ -768,77 +820,45 @@ brasero_burn_dialog_update_title_writing_progress (BraseroBurnDialog *dialog,
 	if (media == BRASERO_MEDIUM_FILE)
 		title = g_strdup_printf (_("Brasero - Creating Image (%i%% Done)"), percent);
 	else if (media & BRASERO_MEDIUM_DVD) {
-		if (input == BRASERO_TRACK_TYPE_DATA) {
-			if (flags & BRASERO_BURN_FLAG_DUMMY)
-				title = g_strdup (_("Brasero - Burning DVD (Simulation)"));
-			else
-				title = g_strdup_printf (_("Brasero - Burning DVD (%i%% Done)"), percent);
-		}
-		else if (input == BRASERO_TRACK_TYPE_IMAGE) {
-			if (flags & BRASERO_BURN_FLAG_DUMMY)
-				title = g_strdup (_("Brasero - Burning DVD (Simulation)"));
-			else
-				title = g_strdup_printf (_("Brasero - Burning DVD (%i%% Done)"), percent);
-		}
-		else if (input == BRASERO_TRACK_TYPE_DISC) {
+		if (input->type == BRASERO_TRACK_TYPE_DISC) {
 			if (flags & BRASERO_BURN_FLAG_DUMMY)
 				title = g_strdup (_("Brasero - Copying DVD (Simulation)"));
 			else
 				title = g_strdup_printf (_("Brasero - Copying DVD (%i%% Done)"), percent);
 		}
+		else {
+			if (flags & BRASERO_BURN_FLAG_DUMMY)
+				title = g_strdup (_("Brasero - Burning DVD (Simulation)"));
+			else
+				title = g_strdup_printf (_("Brasero - Burning DVD (%i%% Done)"), percent);
+		}
 	}
 	else if (media & BRASERO_MEDIUM_CD) {
-		if (input == BRASERO_TRACK_TYPE_AUDIO) {
-			if (flags & BRASERO_BURN_FLAG_DUMMY)
-				title = g_strdup (_("Brasero - Burning CD (Simulation)"));
-			else
-				title = g_strdup_printf (_("Brasero - Burning CD (%i%% Done)"), percent);
-		}
-		else if (input == BRASERO_TRACK_TYPE_DATA) {
-			if (flags & BRASERO_BURN_FLAG_DUMMY)
-				title = g_strdup (_("Brasero - Burning CD (Simulation)"));
-			else
-				title = g_strdup_printf (_("Brasero - Burning CD (%i%% Done)"), percent);
-		}
-		else if (input == BRASERO_TRACK_TYPE_DISC) {
+		if (input->type == BRASERO_TRACK_TYPE_DISC) {
 			if (flags & BRASERO_BURN_FLAG_DUMMY)
 				title = g_strdup (_("Brasero - Copying CD (Simulation)"));
 			else
 				title = g_strdup_printf (_("Brasero - Copying CD (%i%% Done)"), percent);
 		}
-		else if (input == BRASERO_TRACK_TYPE_IMAGE) {
+		else {
 			if (flags & BRASERO_BURN_FLAG_DUMMY)
 				title = g_strdup (_("Brasero - Burning CD (simulation)"));
 			else
 				title = g_strdup_printf (_("Brasero - Burning CD (%i%% Done)"), percent);
 		}
 	}
-	else if (input == BRASERO_TRACK_TYPE_AUDIO) {
-		if (flags & BRASERO_BURN_FLAG_DUMMY)
-			title = g_strdup (_("Brasero - Burning CD (Simulation)"));
-		else
-			title = g_strdup_printf (_("Brasero - Burning CD (%i%% Done)"), percent);
-	}
-	else if (input == BRASERO_TRACK_TYPE_DATA) {
-		if (flags & BRASERO_BURN_FLAG_DUMMY)
-			title = g_strdup (_("Brasero - Burning Disc (Simulation)"));
-		else
-			title = g_strdup_printf (_("Brasero - Burning Disc (%i%% Done)"), percent);
-	}
-	else if (input == BRASERO_TRACK_TYPE_DISC) {
+	else if (input->type == BRASERO_TRACK_TYPE_DISC) {
 		if (flags & BRASERO_BURN_FLAG_DUMMY)
 			title = g_strdup (_("Brasero - Copying Disc (Simulation)"));
 		else
 			title = g_strdup_printf (_("Brasero - Copying Disc (%i%% Done)"), percent);
 	}
-	else if (input == BRASERO_TRACK_TYPE_IMAGE) {
+	else {
 		if (flags & BRASERO_BURN_FLAG_DUMMY)
 			title = g_strdup (_("Brasero - Burning Disc (Simulation)"));
 		else
 			title = g_strdup_printf (_("Brasero - Burning Disc (%i%% Done)"), percent);
 	}
-	else
-		return;
 
 	gtk_window_set_title (GTK_WINDOW (dialog), title);
 	g_free (title);
@@ -879,7 +899,7 @@ brasero_burn_dialog_progress_changed_real (BraseroBurnDialog *dialog,
 
 	if (task_progress >= 0.0 && dialog->priv->is_writing)
 		brasero_burn_dialog_update_title_writing_progress (dialog,
-								   dialog->priv->input.type,
+								   &dialog->priv->input,
 								   media,
 								   (guint) (task_progress * 100.0));
 
@@ -895,6 +915,10 @@ brasero_burn_dialog_progress_changed_real (BraseroBurnDialog *dialog,
 	brasero_tray_icon_set_progress (BRASERO_TRAYICON (dialog->priv->tray),
 					task_progress,
 					remaining);
+
+	if (rate > 0 && dialog->priv->is_writing)
+		dialog->priv->rates = g_slist_prepend (dialog->priv->rates,
+						       GINT_TO_POINTER ((gint) rate));
 }
 
 static void
@@ -923,6 +947,8 @@ brasero_burn_dialog_progress_changed_cb (BraseroBurn *burn,
 						   task_progress,
 						   remaining,
 						   media);
+	if ((dialog->priv->is_writing || dialog->priv->is_creating_image) && isosize > 0)
+		dialog->priv->total_size = isosize;
 }
 
 static void
@@ -958,10 +984,11 @@ brasero_burn_dialog_action_changed_cb (BraseroBurn *burn,
 
 		brasero_burn_status (burn, &media, NULL, NULL, NULL);
 		brasero_burn_dialog_update_info (dialog,
-						 dialog->priv->input.type,
+						 &dialog->priv->input,
 						 media);
 	}
 
+	dialog->priv->is_creating_image = (action == BRASERO_BURN_ACTION_CREATING_IMAGE);
 	dialog->priv->is_writing = is_writing;
 
 	brasero_burn_get_action_string (dialog->priv->burn, action, &string);
@@ -985,8 +1012,16 @@ brasero_burn_dialog_dummy_success_cb (BraseroBurn *burn,
 	GtkWidget *message;
 	GtkWindow *window;
 	GtkWidget *button;
+	gboolean hide;
 	gint id;
 
+	if (!GTK_WIDGET_VISIBLE (dialog)) {
+		gtk_widget_show (GTK_WIDGET (dialog));
+		hide = TRUE;
+	}
+
+	g_timer_stop (dialog->priv->total_time);
+
 	window = GTK_WINDOW (dialog);
 	message = gtk_message_dialog_new (window,
 					  GTK_DIALOG_DESTROY_WITH_PARENT|
@@ -1014,6 +1049,11 @@ brasero_burn_dialog_dummy_success_cb (BraseroBurn *burn,
 	answer = gtk_dialog_run (GTK_DIALOG (message));
 	gtk_widget_destroy (message);
 
+	if (hide)
+		gtk_widget_hide (GTK_WIDGET (dialog));
+
+	g_timer_start (dialog->priv->total_time);
+
 	if (answer == GTK_RESPONSE_OK) {
 		if (id)
 			g_source_remove (id);
@@ -1135,6 +1175,16 @@ brasero_burn_dialog_finalize (GObject * object)
 		cobj->priv->session = NULL;
 	}
 
+	if (cobj->priv->total_time) {
+		g_timer_destroy (cobj->priv->total_time);
+		cobj->priv->total_time = NULL;
+	}
+
+	if (cobj->priv->rates) {
+		g_slist_free (cobj->priv->rates);
+		cobj->priv->rates = NULL;
+	}
+
 	g_free (cobj->priv);
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -1273,6 +1323,13 @@ brasero_burn_dialog_setup_session (BraseroBurnDialog *dialog,
 
 	brasero_tray_icon_set_action (BRASERO_TRAYICON (dialog->priv->tray),
 				      BRASERO_BURN_ACTION_NONE);
+
+	if (dialog->priv->total_time)
+		g_timer_destroy (dialog->priv->total_time);
+
+	dialog->priv->total_time = g_timer_new ();
+	g_timer_start (dialog->priv->total_time);
+
 	return BRASERO_BURN_OK;
 }
 
@@ -1573,6 +1630,7 @@ brasero_burn_dialog_success_run (BraseroBurnDialog *dialog)
 static void
 brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 {
+	gint64 rate;
 	BraseroMedia media;
 	BraseroDrive *drive;
 	gchar *primary = NULL;
@@ -1623,6 +1681,26 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 
 	brasero_burn_dialog_activity_stop (dialog, primary);
 
+	/* show total required time and average speed */
+	rate = 0;
+	if (dialog->priv->rates) {
+		int num = 0;
+		GSList *iter;
+
+		for (iter = dialog->priv->rates; iter; iter = iter->next) {
+			rate += GPOINTER_TO_INT (iter->data);
+			num ++;
+		}
+		rate /= num;
+	}
+
+	brasero_burn_progress_display_session_info (BRASERO_BURN_PROGRESS (dialog->priv->progress),
+						    g_timer_elapsed (dialog->priv->total_time, NULL),
+						    rate,
+						    (media & BRASERO_MEDIUM_DVD),
+						    dialog->priv->total_size);
+
+
 	if (brasero_burn_session_get_input_type (dialog->priv->session, NULL) == BRASERO_TRACK_TYPE_AUDIO) {
 		GtkWidget *button;
 
@@ -1707,10 +1785,8 @@ brasero_burn_dialog_end_session (BraseroBurnDialog *dialog,
 				 BraseroBurnResult result,
 				 GError *error)
 {
-	if (dialog->priv->burn) {
-		g_object_unref (dialog->priv->burn);
-		dialog->priv->burn = NULL;
-	}
+	if (dialog->priv->total_time)
+		g_timer_stop (dialog->priv->total_time);
 
 	brasero_burn_session_stop (dialog->priv->session);
 
@@ -1737,6 +1813,21 @@ brasero_burn_dialog_end_session (BraseroBurnDialog *dialog,
 		brasero_burn_dialog_notify_success (dialog);
 	}
 
+	if (dialog->priv->burn) {
+		g_object_unref (dialog->priv->burn);
+		dialog->priv->burn = NULL;
+	}
+
+	if (dialog->priv->rates) {
+		g_slist_free (dialog->priv->rates);
+		dialog->priv->rates = NULL;
+	}
+
+	if (dialog->priv->total_time) {
+		g_timer_destroy (dialog->priv->total_time);
+		dialog->priv->total_time = NULL;
+	}
+
 	return FALSE;
 }
 
@@ -1766,7 +1857,7 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 		media = brasero_medium_get_status (medium);
 	}
 
-	brasero_burn_dialog_update_info (dialog, dialog->priv->input.type, media);
+	brasero_burn_dialog_update_info (dialog, &dialog->priv->input, media);
 
 	/* start the recording session */
 	brasero_burn_dialog_activity_start (dialog);
