@@ -460,6 +460,8 @@ brasero_data_disc_paste_activated_cb (GtkAction *action,
 	data = g_new0 (BraseroClipData, 1);
 	data->disc = disc;
 
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 1);
+
 	parent = brasero_data_disc_get_parent (disc);
 	if (parent)
 		data->reference = brasero_data_project_reference_new (priv->project, parent);
@@ -598,9 +600,9 @@ brasero_data_disc_project_loaded_cb (BraseroDataProject *project,
 		brasero_disc_message_remove_buttons (BRASERO_DISC_MESSAGE (message));
 
 		brasero_disc_message_set_primary (BRASERO_DISC_MESSAGE (message),
-						  _("The contents of the project changed since it was saved:"));
+						  _("The contents of the project changed since it was saved."));
 		brasero_disc_message_set_secondary (BRASERO_DISC_MESSAGE (message),
-						    _("Do you want to continue or discard it?"));
+						    _("Do you want to continue or discard the old project layout?"));
 
 		brasero_disc_message_set_image (BRASERO_DISC_MESSAGE (message),GTK_STOCK_DIALOG_WARNING);
 		brasero_disc_message_set_progress_active (BRASERO_DISC_MESSAGE (message), FALSE);
@@ -903,15 +905,15 @@ brasero_data_disc_2G_file_cb (BraseroDataProject *project,
 					 GTK_DIALOG_MODAL,
 					 GTK_MESSAGE_WARNING,
 					 GTK_BUTTONS_NONE,
-					 _("Do you really want to add \"%s\" to the selection and use ISO9660 level 3 standard to support it?"),
+					 _("Do you really want to add \"%s\" to the selection and use the third version of ISO9660 standard to support it?"),
 					 name);
 
-	gtk_window_set_title (GTK_WINDOW (dialog), _("File Over 2 Gio"));
+	gtk_window_set_title (GTK_WINDOW (dialog), _("File Over 2 GiB"));
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("The size of the file is over 2 Gio. This isn't supported by ISO9660 standard in his first and second versions (the most widespread ones)."
-						    "\nISO9660 level 3 standard is supported by most of the operating systems including linux and all versions of Windows ©. Therefore the burnt medium can be read by these operating systems."
-						    "\nA known exception is MacOS X that can't read images created with ISO9660 level 3 standard."));
+						  _("The size of the file is over 2 GiB. This isn't supported by ISO9660 standard in his first and second versions (the most widespread ones)."
+						    "\nIt is recommended to use the third version of ISO9660 standard which is supported by most of the operating systems including Linux and all versions of Windows ©."
+						    "\nA known exception is MacOS X that can't read images created with version 3 of ISO9660 standard."));
 
 	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Don't add"), GTK_RESPONSE_NO);
 	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Add"), GTK_RESPONSE_YES);
@@ -1063,11 +1065,17 @@ static void
 brasero_data_disc_clear (BraseroDisc *disc)
 {
 	BraseroDataDiscPrivate *priv;
+	GtkAction *action;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (disc);
+
 	if (priv->loading)
 		return;
 
+	action = gtk_action_group_get_action (priv->disc_group, "ImportSession");
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+
 	if (priv->load_errors) {
 		g_slist_foreach (priv->load_errors, (GFunc) g_free , NULL);
 		g_slist_free (priv->load_errors);
@@ -1077,12 +1085,15 @@ brasero_data_disc_clear (BraseroDisc *disc)
 	priv->G2_files = FALSE;
 	priv->deep_directory = FALSE;
 
+ 	brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_SIZE);
 	brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_LOADING);
 	brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_MULTISESSION);
 
 	brasero_data_project_reset (priv->project);
 	brasero_file_filtered_clear (BRASERO_FILE_FILTERED (priv->filter));
 	brasero_disc_size_changed (disc, 0);
+
+	gdk_window_set_cursor (GTK_WIDGET (disc)->window, NULL);
 }
 
 static void
@@ -1094,11 +1105,8 @@ brasero_data_disc_reset (BraseroDisc *disc)
 	priv = BRASERO_DATA_DISC_PRIVATE (disc);
 
 	action = gtk_action_group_get_action (priv->disc_group, "ImportSession");
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action))) {
-		g_signal_handlers_block_by_func (action, brasero_data_disc_import_session_cb, disc);
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
-		g_signal_handlers_unblock_by_func (action, brasero_data_disc_import_session_cb, disc);
-	}
 
 	if (priv->load_errors) {
 		g_slist_foreach (priv->load_errors, (GFunc) g_free , NULL);
@@ -1112,9 +1120,14 @@ brasero_data_disc_reset (BraseroDisc *disc)
 	priv->G2_files = FALSE;
 	priv->deep_directory = FALSE;
 
-	brasero_file_filtered_clear (BRASERO_FILE_FILTERED (priv->filter));
+ 	brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_SIZE);
+	brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_LOADING);
+	brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_MULTISESSION);
 
+	brasero_file_filtered_clear (BRASERO_FILE_FILTERED (priv->filter));
 	brasero_disc_size_changed (disc, 0);
+
+	gdk_window_set_cursor (GTK_WIDGET (disc)->window, NULL);
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 0);
 }
 
@@ -1244,7 +1257,7 @@ brasero_data_disc_set_session_param (BraseroDisc *self,
 	if (brasero_data_project_is_joliet_compliant (priv->project))
 		fs_type |= BRASERO_IMAGE_FS_JOLIET;
 
-	if (stats->num_2Gio != 0)
+	if (stats->num_2GiB != 0)
 		fs_type |= BRASERO_IMAGE_ISO_FS_LEVEL_3|BRASERO_IMAGE_FS_UDF;
 
 	if (stats->num_deep != 0)
@@ -1333,8 +1346,8 @@ brasero_data_disc_message_response_cb (BraseroDiscMessage *message,
 	if (response != GTK_RESPONSE_CANCEL)
 		return;
 
-	brasero_data_disc_reset (BRASERO_DISC (self));
 	priv->loading = FALSE;
+	brasero_data_disc_clear (BRASERO_DISC (self));
 }
 
 static BraseroDiscResult
@@ -1979,14 +1992,14 @@ brasero_data_disc_init (BraseroDataDisc *object)
 
 	priv = BRASERO_DATA_DISC_PRIVATE (object);
 
-	gtk_box_set_spacing (GTK_BOX (object), 0);
+	gtk_box_set_spacing (GTK_BOX (object), 8);
 
 	/* the information displayed about how to use this tree */
 	priv->notebook = brasero_disc_get_use_info_notebook ();
 	gtk_widget_show (priv->notebook);
 	gtk_box_pack_start (GTK_BOX (object), priv->notebook, TRUE, TRUE, 0);
 
-	mainbox = gtk_vbox_new (FALSE, 12);
+	mainbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (mainbox);
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), mainbox, NULL);
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 0);
@@ -2210,7 +2223,7 @@ brasero_data_disc_init (BraseroDataDisc *object)
 			  G_CALLBACK (brasero_data_disc_filter_expanded_cb),
 			  object);
 	gtk_widget_show (priv->filter);
-	gtk_box_pack_end (GTK_BOX (mainbox), priv->filter, FALSE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (object), priv->filter, FALSE, TRUE, 0);
 
 	g_signal_connect (priv->filter,
 			  "filtered",
