@@ -440,7 +440,9 @@ brasero_data_project_node_to_uri (BraseroDataProject *self,
 			break;
 
 		/* the + 1 is for the separator */
-		escaped_name = g_uri_escape_string (BRASERO_FILE_NODE_NAME (node), NULL, TRUE);
+		escaped_name = g_uri_escape_string (BRASERO_FILE_NODE_NAME (node),
+						    G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
+						    FALSE);
 		uri_len += strlen (escaped_name) + 1;
 		list = g_slist_prepend (list, escaped_name);
 	}
@@ -839,12 +841,35 @@ brasero_data_project_uri_remove_graft (BraseroDataProject *self,
 		brasero_utils_unregister_string (key);
 
 	if (graft) {
-		/* NOTE: no need to free graft->uri since that's key */
+		/* NOTE: no need to free graft->uri since that's the key */
 		g_slist_free (graft->nodes);
 		g_free (graft);
 	}
 }
 
+static gboolean
+brasero_data_project_graft_is_needed (BraseroDataProject *self,
+				      BraseroGraft *graft)
+{
+	BraseroURINode *uri_node;
+
+	uri_node = graft->node;
+	if (uri_node->nodes)
+		return TRUE;
+
+	/* there aren't any node grafted for this URI. See if we should keep the
+	 * URI in the hash; if so, the URI must have parents in the hash */
+	if (brasero_data_project_uri_has_parent (self, uri_node->uri)) {
+		/* here that means that this URI is nowhere in the tree but has
+		 * parent URIs which are grafted. So keep it in the hash to
+		 * signal that URI is not in the tree. */
+		return TRUE;
+	}
+
+	brasero_data_project_uri_remove_graft (self, uri_node->uri);
+	return FALSE;
+}
+
 static BraseroURINode *
 brasero_data_project_uri_add_graft (BraseroDataProject *self,
 				    const gchar *uri)
@@ -959,7 +984,7 @@ brasero_data_project_remove_node_children_graft_cb (const gchar *key,
 			continue;
 
 		if (brasero_file_node_is_ancestor (data->node, iter_node))
-			graft->nodes = g_slist_remove (graft->nodes, iter_node);
+			brasero_file_node_ungraft (iter_node);
 	}
 
 	if (graft->nodes)
@@ -1049,24 +1074,18 @@ brasero_data_project_node_removed (BraseroDataProject *self,
 		graft = BRASERO_FILE_NODE_GRAFT (node);
 		uri_node = graft->node;
 
-		if (g_slist_length (uri_node->nodes) == 1) {
+		/* NOTE: after this function the graft is invalid */
+		brasero_file_node_ungraft (node);
+
+		if (!uri_node->nodes) {
 			/* that's the last node grafted for this URI.
 			 * There are no more nodes for this URI after.
 			 * See if we should keep the URI in the hash;
 			 * if so, the URI must have parents in the hash
 			 */
-			if (brasero_data_project_uri_has_parent (self, uri_node->uri)) {
-				/* here that means that this URI is 
-				 * nowhere in the tree but has parent
-				 * URIs which are grafted. */
-				g_slist_free (uri_node->nodes);
-				uri_node->nodes = NULL;
-			}
-			else
+			if (!brasero_data_project_uri_has_parent (self, uri_node->uri))
 				brasero_data_project_uri_remove_graft (self, uri_node->uri);
 		}
-		else
-			uri_node->nodes = g_slist_remove (uri_node->nodes, node);
 	}
 	else if (!node->is_imported) {
 		gchar *uri;
@@ -1582,6 +1601,38 @@ brasero_data_project_add_imported_session_file (BraseroDataProject *self,
 
 	node = brasero_file_node_check_name_existence (parent, BRASERO_VOLUME_FILE_NAME (file));
 	if (node) {
+		/* The node exists but it may be that we've loaded the project
+		 * before. Then the necessary directories to hold the grafted
+		 * files will have been created as fake directories. We need to
+		 * replace those whenever we run into one but not lose their 
+		 * children. */
+		if (node->is_fake && node->is_tmp_parent) {
+			BraseroGraft *graft;
+			BraseroURINode *uri_node;
+
+			graft = BRASERO_FILE_NODE_GRAFT (node);
+			uri_node = graft->node;
+
+			/* NOTE after this function graft is invalid */
+			brasero_file_node_ungraft (node);
+
+			/* see if uri_node is still needed */
+			if (!uri_node->nodes
+			&&  !brasero_data_project_uri_has_parent (self, uri_node->uri))
+				brasero_data_project_uri_remove_graft (self, uri_node->uri);
+
+			node->is_fake = FALSE;
+			node->is_imported = TRUE;
+			node->is_tmp_parent = FALSE;
+
+			/* Something has changed, tell the tree */
+			klass = BRASERO_DATA_PROJECT_GET_CLASS (self);
+			if (klass->node_changed)
+				klass->node_changed (self, node);
+
+			return node;
+		}
+
 		if (brasero_data_project_file_signal (self, NAME_COLLISION_SIGNAL, BRASERO_FILE_NODE_NAME (node)))
 			return NULL;
 
@@ -1646,6 +1697,54 @@ brasero_data_project_add_empty_directory (BraseroDataProject *self,
 	return node;
 }
 
+static void
+brasero_data_project_update_uri (BraseroDataProject *self,
+				 BraseroFileNode *node,
+				 const gchar *uri)
+{
+	gchar *parent_uri;
+	BraseroGraft *graft;
+	BraseroURINode *uri_node;
+
+	graft = BRASERO_FILE_NODE_GRAFT (node);
+	if (!strcmp (graft->node->uri, uri)) {
+		/* Nothing needs update */
+		return;
+	}
+
+	/* different URIS; make sure the node still needs a graft:
+	 * - if so, update it
+	 * - if not, remove it*/
+	parent_uri = brasero_data_project_node_to_uri (self, node->parent);
+	if (parent_uri) {
+		guint parent_len;
+
+		parent_len = strlen (parent_uri);
+
+		if (strncmp (parent_uri, uri, parent_len)
+		&&  uri [parent_len] != G_DIR_SEPARATOR) {
+			/* The node hasn't been put under its rightful parent
+			 * from the original file system. That means we must add
+			 * a graft or update the current one. */
+			uri_node = brasero_data_project_uri_add_graft (self, uri);
+			brasero_file_node_graft (node, uri_node);
+		}
+		else {
+			/* rightful parent: ungraft it */
+			brasero_file_node_ungraft (node);
+		}
+
+		g_free (parent_uri);
+	}
+	else {
+		uri_node = brasero_data_project_uri_add_graft (self, uri);
+		brasero_file_node_graft (node, uri_node);
+	}
+
+	/* the node was ungrafted, check if the old graft is still needed */
+	brasero_data_project_graft_is_needed (self, graft);
+}
+
 void
 brasero_data_project_node_loaded (BraseroDataProject *self,
 				  BraseroFileNode *node,
@@ -1714,6 +1813,19 @@ brasero_data_project_node_loaded (BraseroDataProject *self,
 			       priv->loading);
 	}
 
+	/* If the node is not grafted because it was put under its original 
+	 * parent on the file system it comes from, then its parent URI can't
+	 * have changed (the parent it was put under had already its URI cleaned
+	 * of any symlink). Its URI may be different though if it's a symlink
+	 * but that case is treated somewhere else. */
+	if (node->is_grafted) {
+		/* The URI of the node could be different from the one we gave
+		 * earlier as brasero-io looks for parent symlinks and replace
+		 * them with their target. So since it's a graft, we need to 
+		 * update the graft URI just to make sure. */
+		brasero_data_project_update_uri (self, node, uri);
+	}
+
 	size = g_file_info_get_size (info);
 	if (type != G_FILE_TYPE_DIRECTORY) {
 		if (BRASERO_SIZE_TO_SECTORS (size, 2048) > BRASERO_FILE_2G_LIMIT
@@ -1774,7 +1886,6 @@ brasero_data_project_node_loaded (BraseroDataProject *self,
 
 	/* signal the changes */
 	brasero_data_project_node_changed (self, node);
-
 	if (size_changed)
 		g_signal_emit (self,
 			       brasero_data_project_signals [SIZE_CHANGED_SIGNAL],
@@ -1859,7 +1970,9 @@ brasero_data_project_add_loading_node (BraseroDataProject *self,
 	/* make sure that name doesn't exist */
 	node = brasero_file_node_check_name_existence (parent, name);
 	if (node) {
-		if (brasero_data_project_file_signal (self, NAME_COLLISION_SIGNAL, BRASERO_FILE_NODE_NAME (node))) {
+		if (brasero_data_project_file_signal (self,
+						      NAME_COLLISION_SIGNAL,
+						      BRASERO_FILE_NODE_NAME (node))) {
 			g_free (name);
 			return NULL;
 		}
@@ -1978,7 +2091,7 @@ brasero_data_project_add_node_from_info (BraseroDataProject *self,
 			if (brasero_data_project_file_signal (self, G2_FILE_SIGNAL, name))
 				return NULL;
 	}
-	else if (!brasero_file_node_get_depth (parent) == 5) {
+	else if (brasero_file_node_get_depth (parent) == 5) {
 		if (brasero_data_project_file_signal (self, DEEP_DIRECTORY_SIGNAL, name))
 			return NULL;
 	}
@@ -2752,6 +2865,28 @@ brasero_data_project_load_contents (BraseroDataProject *self,
 		/* get the URI for this node. There should be one now that all
 		 * graft nodes are in the tree. */
 		uri = brasero_data_project_node_to_uri (self, tmp);
+		if (!uri) {
+			/* This node has been grafted under a node that was
+			 * imported or was itself an imported node. Since there
+			 * is no imported nodes any more, then it has to become
+			 * fake.
+			 * NOTE: it has to be a directory */
+			tmp->is_fake = TRUE;
+			tmp->is_loading = FALSE;
+			tmp->is_reloading = FALSE;
+			//tmp->is_tmp_parent = FALSE;
+
+			graft = brasero_data_project_uri_ensure_graft (self, NEW_FOLDER);
+			brasero_file_node_graft (tmp, graft);
+
+			priv->loading -= 2;
+
+			/* Signal that something has changed in the tree */
+			if (klass->node_added)
+				klass->node_added (self, tmp, uri);
+
+			continue;
+		}
 
 		/* graft it ? */
 		graft = brasero_data_project_uri_ensure_graft (self, uri);
@@ -3025,7 +3160,9 @@ brasero_data_project_file_added (BraseroFileMonitor *monitor,
 
 	/* get the new URI */
 	parent_uri = brasero_data_project_node_to_uri (BRASERO_DATA_PROJECT (monitor), parent);
-	escaped_name = g_uri_escape_string (name, NULL, TRUE);
+	escaped_name = g_uri_escape_string (name,
+					    G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
+					    FALSE);
 	uri = g_strconcat (parent_uri, G_DIR_SEPARATOR_S, escaped_name, NULL);
 	g_free (escaped_name);
 	g_free (parent_uri);
@@ -3063,7 +3200,9 @@ brasero_data_project_file_update_URI (BraseroDataProject *self,
 	uri_node = graft->node;
 
 	/* get the new uri */
-	escaped_name = g_uri_escape_string (name, NULL, TRUE);
+	escaped_name = g_uri_escape_string (name,
+					    G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
+					    FALSE);
 	uri = g_build_path (G_DIR_SEPARATOR_S, parent_uri, escaped_name, NULL);
 	g_free (escaped_name);
 
@@ -3139,7 +3278,9 @@ brasero_data_project_file_graft (BraseroDataProject *self,
 	parent = g_path_get_dirname (uri);
 	g_free (uri);
 
-	escaped_name = g_uri_escape_string (real_name, NULL, TRUE);
+	escaped_name = g_uri_escape_string (real_name,
+					    G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
+					    FALSE);
 	uri = g_strconcat (parent, G_DIR_SEPARATOR_S, escaped_name, NULL);
 	g_free (escaped_name);
 	g_free (parent);
