@@ -184,7 +184,12 @@ brasero_data_vfs_check_uri_result (BraseroDataVFS *self,
 
 	/* Only signal errors if the node was specifically added by the user
 	 * that is if it is loading. So check the loading GHashTable to know 
-	 * that. Otherwise this URI comes from directory exploration. */
+	 * that. Otherwise this URI comes from directory exploration.
+	 * The problem is the URI returned by brasero-io could be different
+	 * from the one set in the loading hash if there are parent symlinks.
+	 * That's one of the readon why we passed the orignal URI as a
+	 * registered string in the callback. Of course that's not true when
+	 * we're loading directory contents. */
 
 	if (error) {
 		if (error->domain == G_IO_ERROR && error->code == G_IO_ERROR_NOT_FOUND) {
@@ -324,6 +329,10 @@ brasero_data_vfs_directory_check_symlink_loop (BraseroDataVFS *self,
 
 	priv = BRASERO_DATA_VFS_PRIVATE (self);
 
+	/* Of course for a loop to exist, it must be a directory */
+	if (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY)
+		return FALSE;
+
 	target_uri = g_file_info_get_symlink_target (info);
 	if (!target_uri)
 		return FALSE;
@@ -469,9 +478,7 @@ brasero_data_vfs_directory_load_result (GObject *owner,
 
 		/* See if we are supposed to keep them */
 		if (status != BRASERO_DATA_VFS_RESTORED && priv->filter_hidden) {
-			brasero_data_project_exclude_uri (BRASERO_DATA_PROJECT (self),
-							  uri);
-
+			brasero_data_project_exclude_uri (BRASERO_DATA_PROJECT (self), uri);
 			if (status == BRASERO_DATA_VFS_NONE) {
 				/* Advertise only once this filtered URI */
 				g_signal_emit (self,
@@ -610,8 +617,9 @@ brasero_data_vfs_loading_node_result (GObject *owner,
 	BraseroDataVFSPrivate *priv = BRASERO_DATA_VFS_PRIVATE (self);
 
 	nodes = g_hash_table_lookup (priv->loading, registered);
+
 	/* check the status of the operation */
-	if (!brasero_data_vfs_check_uri_result (self, uri, error, info)) {
+	if (!brasero_data_vfs_check_uri_result (self, registered, error, info)) {
 		/* we need to remove the loading node that is waiting */
 		for (iter = nodes; iter; iter = iter->next) {
 			BraseroFileNode *node;
@@ -619,7 +627,10 @@ brasero_data_vfs_loading_node_result (GObject *owner,
 
 			reference = GPOINTER_TO_INT (iter->data);
 			node = brasero_data_project_reference_get (BRASERO_DATA_PROJECT (self), reference);
-			brasero_data_project_remove_node (BRASERO_DATA_PROJECT (self), node);
+
+			/* the node could have been removed in the mean time */
+			if (node)
+				brasero_data_project_remove_node (BRASERO_DATA_PROJECT (self), node);
 		}
 
 		return;
@@ -653,12 +664,18 @@ brasero_data_vfs_loading_node_result (GObject *owner,
 		 * reloading. */
 
 		if (!node->is_loading) {
-			brasero_data_project_node_reloaded (BRASERO_DATA_PROJECT (self), node, uri, info);
+			brasero_data_project_node_reloaded (BRASERO_DATA_PROJECT (self),
+							    node,
+							    uri,
+							    info);
 			continue;
 		}
 
 		/* update node */
-		brasero_data_project_node_loaded (BRASERO_DATA_PROJECT (self), node, uri, info);
+		brasero_data_project_node_loaded (BRASERO_DATA_PROJECT (self),
+						  node,
+						  uri,
+						  info);
 
 		/* See what type of file it is. If that's a directory then 
 		 * explore it right away */
@@ -720,14 +737,13 @@ brasero_data_vfs_loading_node (BraseroDataVFS *self,
 	GSList *nodes;
 
 	/* NOTE: this function receives URIs only from utf8 origins (not from
-	 * gnome-vfs for example) so we can assume that this is safe */
+	 * GIO for example) so we can assume that this is safe */
 
 	priv = BRASERO_DATA_VFS_PRIVATE (self);
 
 	if (!node->is_reloading) {
 		gchar *name;
 		GFile *vfs_uri;
-		gchar *unescaped_name;
 
 		/* g_path_get_basename is not comfortable with uri related
 		 * to the root directory so check that before */
@@ -735,9 +751,10 @@ brasero_data_vfs_loading_node (BraseroDataVFS *self,
 		name = g_file_get_basename (vfs_uri);
 		g_object_unref (vfs_uri);
 
-		unescaped_name = g_uri_unescape_string (name, NULL);
-		g_free (name);
-		name = unescaped_name;
+
+		/* NOTE and reminder names are already unescaped; the following
+		 * is not needed: unescaped_name = g_uri_unescape_string (name,
+		 * NULL); */
 
 		if (!name)
 			return TRUE;
@@ -1063,6 +1080,42 @@ brasero_data_vfs_reset (BraseroDataProject *project,
 		BRASERO_DATA_PROJECT_CLASS (brasero_data_vfs_parent_class)->reset (project, num_nodes);
 }
 
+static void
+brasero_data_vfs_filter_hidden_changed (GConfClient *client,
+					guint cxn,
+					GConfEntry *entry,
+					gpointer data)
+{
+	BraseroDataVFSPrivate *priv;
+	GConfValue *value;
+
+	priv = BRASERO_DATA_VFS_PRIVATE (data);
+
+	value = gconf_entry_get_value (entry);
+	if (value->type != GCONF_VALUE_BOOL)
+		return;
+
+	priv->filter_hidden = gconf_value_get_bool (value);
+}
+
+static void
+brasero_data_vfs_filter_broken_sym_changed (GConfClient *client,
+					    guint cxn,
+					    GConfEntry *entry,
+					    gpointer data)
+{
+	BraseroDataVFSPrivate *priv;
+	GConfValue *value;
+
+	priv = BRASERO_DATA_VFS_PRIVATE (data);
+
+	value = gconf_entry_get_value (entry);
+	if (value->type != GCONF_VALUE_BOOL)
+		return;
+
+	priv->filter_broken_sym = gconf_value_get_bool (value);
+}
+
 static void
 brasero_data_vfs_init (BraseroDataVFS *object)
 {
@@ -1079,6 +1132,18 @@ brasero_data_vfs_init (BraseroDataVFS *object)
 	priv->filter_broken_sym = gconf_client_get_bool (client,
 							 BRASERO_FILTER_BROKEN_SYM_KEY,
 							 NULL);
+	gconf_client_notify_add (client,
+				 BRASERO_FILTER_HIDDEN_KEY,
+				 brasero_data_vfs_filter_hidden_changed,
+				 object,
+				 NULL,
+				 NULL);
+	gconf_client_notify_add (client,
+				 BRASERO_FILTER_BROKEN_SYM_KEY,
+				 brasero_data_vfs_filter_broken_sym_changed,
+				 object,
+				 NULL,
+				 NULL);
 	g_object_unref (client);
 
 	/* create the hash tables */
