@@ -51,6 +51,7 @@
 #include "burn-drive.h"
 #include "brasero-drive-selection.h"
 #include "brasero-drive-properties.h"
+#include "brasero-drive-info.h"
 #include "brasero-image-properties.h"
 #include "brasero-dest-selection.h"
 
@@ -61,6 +62,8 @@ struct _BraseroDestSelectionPrivate
 	BraseroBurnCaps *caps;
 	BraseroBurnSession *session;
 
+	GtkWidget *info;
+
 	GtkWidget *drive_prop;
 	GtkWidget *button;
 
@@ -208,17 +211,10 @@ brasero_dest_selection_drive_properties (BraseroDestSelection *self)
 					    rate);
 
 	flags = brasero_burn_session_get_flags (priv->session);
-	if (!brasero_dest_selection_check_same_src_dest (self)) {
-		brasero_burn_caps_get_flags (priv->caps,
-					     priv->session,
-					     &supported,
-					     &compulsory);
-	}
-	else {
-		supported = BRASERO_DRIVE_PROPERTIES_FLAGS;
-		supported &= ~BRASERO_BURN_FLAG_NO_TMP_FILES;
-		compulsory = BRASERO_BURN_FLAG_NONE;
-	}
+	brasero_burn_caps_get_flags (priv->caps,
+				     priv->session,
+				     &supported,
+				     &compulsory);
 
 	brasero_drive_properties_set_flags (BRASERO_DRIVE_PROPERTIES (priv->drive_prop),
 					    flags,
@@ -732,7 +728,7 @@ brasero_dest_selection_image_properties (BraseroDestSelection *self)
 	gtk_widget_destroy (priv->drive_prop);
 	priv->drive_prop = NULL;
 
-	brasero_drive_selection_set_image_path (BRASERO_DRIVE_SELECTION (self), image_path);
+	brasero_drive_info_set_image_path (BRASERO_DRIVE_INFO (priv->info), image_path);
 	brasero_dest_selection_set_output_path (self,
 						format,
 						image_path);
@@ -764,72 +760,47 @@ brasero_dest_selection_add_drive_properties_flags (BraseroDestSelection *self,
 						   BraseroBurnFlag *supported_retval,
 						   BraseroBurnFlag *compulsory_retval)
 {
+	BraseroBurnFlag flag;
 	BraseroDestSelectionPrivate *priv;
 	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
 	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
 
 	priv = BRASERO_DEST_SELECTION_PRIVATE (self);
 
-	/* wipe out previous flags */
-	brasero_burn_session_remove_flag (priv->session,
-					  BRASERO_DRIVE_PROPERTIES_FLAGS|
-					  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE|
-					  BRASERO_BURN_FLAG_FAST_BLANK|
-					  BRASERO_BURN_FLAG_DAO);
+	/* add flags then wipe out flags from session to check them one by one */
+	flags |= brasero_burn_session_get_flags (priv->session);
+	brasero_burn_session_remove_flag (priv->session, flags);
 
-	/* check each flag before re-adding it */
 	brasero_burn_caps_get_flags (priv->caps,
 				     priv->session,
 				     &supported,
 				     &compulsory);
 
-	if ((flags & BRASERO_BURN_FLAG_EJECT)
-	&&  (supported & BRASERO_BURN_FLAG_EJECT)) {
-		brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_EJECT);
-		brasero_burn_caps_get_flags (priv->caps,
-					     priv->session,
-					     &supported,
-					     &compulsory);
-	}
+	for (flag = 1; flag < BRASERO_BURN_FLAG_LAST; flag <<= 1) {
+		/* see if this flag was originally set */
+		if (!(flags & flag))
+			continue;
 
-	if ((flags & BRASERO_BURN_FLAG_BURNPROOF)
-	&&  (supported & BRASERO_BURN_FLAG_BURNPROOF)) {
-		brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_BURNPROOF);
-		brasero_burn_caps_get_flags (priv->caps,
-					     priv->session,
-					     &supported,
-					     &compulsory);
-	}
+		/* Don't set write modes now in this case */
+		if (brasero_burn_session_same_src_dest_drive (priv->session)
+		&& (flag & (BRASERO_BURN_FLAG_DAO|BRASERO_BURN_FLAG_RAW)))
+			continue;
 
-	if ((flags & BRASERO_BURN_FLAG_NO_TMP_FILES)
-	&&  (supported & BRASERO_BURN_FLAG_NO_TMP_FILES)) {
-		brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_NO_TMP_FILES);
-		brasero_burn_caps_get_flags (priv->caps,
-					     priv->session,
-					     &supported,
-					     &compulsory);
-	}
+		if (compulsory
+		&& (compulsory & brasero_burn_session_get_flags (priv->session)) != compulsory) {
+			brasero_burn_session_add_flag (priv->session, compulsory);
+			supported = BRASERO_BURN_FLAG_NONE;
+			compulsory = BRASERO_BURN_FLAG_NONE;
+			brasero_burn_caps_get_flags (priv->caps,
+						     priv->session,
+						     &supported,
+						     &compulsory);
+		}
 
-	if ((flags & BRASERO_BURN_FLAG_DUMMY)
-	&&  (supported & BRASERO_BURN_FLAG_DUMMY)) {
-		brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_DUMMY);
-		brasero_burn_caps_get_flags (priv->caps,
-					     priv->session,
-					     &supported,
-					     &compulsory);
-	}
-
-	/* check additional flags */
-	if (supported & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) {
-		/* clean up the disc and have more space when possible */
-		brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE);
-		brasero_burn_caps_get_flags (priv->caps,
-					     priv->session,
-					     &supported,
-					     &compulsory);
-
-		if (supported & BRASERO_BURN_FLAG_FAST_BLANK) {
-			brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_FAST_BLANK);
+		if (supported & flag) {
+			brasero_burn_session_add_flag (priv->session, flag);
+			supported = BRASERO_BURN_FLAG_NONE;
+			compulsory = BRASERO_BURN_FLAG_NONE;
 			brasero_burn_caps_get_flags (priv->caps,
 						     priv->session,
 						     &supported,
@@ -837,15 +808,24 @@ brasero_dest_selection_add_drive_properties_flags (BraseroDestSelection *self,
 		}
 	}
 
-	/* use DAO whenever it's possible */
-	if (supported & BRASERO_BURN_FLAG_DAO) {
-		brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_DAO);
-		brasero_burn_caps_get_flags (priv->caps,
-					     priv->session,
-					     &supported,
-					     &compulsory);
+	/* Always set this flag whenever possible */
+	if (supported & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) {
+		brasero_burn_session_add_flag (priv->session,
+					       BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE|
+					       BRASERO_BURN_FLAG_FAST_BLANK);
+	}
+
+	/* When copying with same drive don't set write mode, it'll be set later */
+	if (!brasero_burn_session_same_src_dest_drive (priv->session)) {
+		/* use DAO whenever it's possible */
+		if (supported & BRASERO_BURN_FLAG_DAO) {
+			brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_DAO);
+			brasero_burn_caps_get_flags (priv->caps,
+						     priv->session,
+						     &supported,
+						     &compulsory);
+		}
 	}
-	brasero_burn_session_add_flag (priv->session, compulsory);
 
 	if (supported_retval)
 		*supported_retval = supported;
@@ -951,6 +931,9 @@ brasero_dest_selection_set_drive_properties (BraseroDestSelection *self)
 	g_free (key);
 
 	if (brasero_dest_selection_check_same_src_dest (self)) {
+		BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
+		BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
+
 		/* Special case */
 
 		/* wipe out previous flags */
@@ -966,17 +949,18 @@ brasero_dest_selection_set_drive_properties (BraseroDestSelection *self)
 			flags = BRASERO_BURN_FLAG_EJECT|
 				BRASERO_BURN_FLAG_BURNPROOF;
 
-		brasero_burn_session_add_flag (priv->session, flags);
+		brasero_dest_selection_add_drive_properties_flags (self,
+								   flags|
+								   BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE|
+								   BRASERO_BURN_FLAG_FAST_BLANK,
+								   &supported,
+								   &compulsory);
 
 		/* NOTE: of course NO_TMP is not possible; DAO and BLANK_BEFORE
 		 * could be yet. The problem here is that we cannot test all
 		 * this since we don't know yet what the disc type is going to 
 		 * be. So we set DAO and BLANK_BEFORE_WRITE just in case.
 		 * Hopefully burn.c will be able to handle that later. */
-		brasero_burn_session_add_flag (priv->session,
-					       BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE|
-					       BRASERO_BURN_FLAG_FAST_BLANK|
-					       BRASERO_BURN_FLAG_DAO);
 	}
 	else if (!value) {
 		BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
@@ -1032,6 +1016,7 @@ static void
 brasero_dest_selection_set_image_properties (BraseroDestSelection *self)
 {
 	BraseroDestSelectionPrivate *priv;
+	BraseroBurnResult result;
 	BraseroTrackType output;
 	gchar *path;
 
@@ -1068,13 +1053,19 @@ brasero_dest_selection_set_image_properties (BraseroDestSelection *self)
 	brasero_dest_selection_set_output_path (self,
 						output.subtype.img_format,
 						path);
-	brasero_drive_selection_set_image_path (BRASERO_DRIVE_SELECTION (self),
-						path);
+	brasero_drive_info_set_image_path (BRASERO_DRIVE_INFO (priv->info), path);
 	g_free (path);
 
 	brasero_burn_session_remove_flag (priv->session,
 					  BRASERO_BURN_FLAG_DUMMY|
 					  BRASERO_BURN_FLAG_NO_TMP_FILES);
+
+	result = brasero_burn_caps_is_session_supported (priv->caps, priv->session);
+	g_signal_emit (self,
+		       brasero_dest_selection_signals [VALID_MEDIA_SIGNAL],
+		       0,
+		       (result == BRASERO_BURN_OK));
+	gtk_widget_set_sensitive (priv->button, (result == BRASERO_BURN_OK));
 }
 
 static void
@@ -1142,7 +1133,7 @@ brasero_dest_selection_check_image_settings (BraseroDestSelection *self)
 		brasero_dest_selection_set_output_path (self,
 							format,
 							path);
-		brasero_drive_selection_set_image_path (BRASERO_DRIVE_SELECTION (self), path);
+		brasero_drive_info_set_image_path (BRASERO_DRIVE_INFO (priv->info), path);
 		g_free (path);
 	}
 	else
@@ -1160,6 +1151,13 @@ brasero_dest_selection_check_image_settings (BraseroDestSelection *self)
 	}
 
 	brasero_burn_session_remove_flag (priv->session, BRASERO_BURN_FLAG_DUMMY);
+
+	result = brasero_burn_caps_is_session_supported (priv->caps, priv->session);
+	g_signal_emit (self,
+		       brasero_dest_selection_signals [VALID_MEDIA_SIGNAL],
+		       0,
+		       (result == BRASERO_BURN_OK));
+	gtk_widget_set_sensitive (priv->button, (result == BRASERO_BURN_OK));
 }
 
 static void
@@ -1177,10 +1175,11 @@ brasero_dest_selection_check_drive_settings (BraseroDestSelection *self)
 	if (brasero_dest_selection_check_same_src_dest (self)) {
 		/* These are always set in any case and there is no way to check
 		 * the current flags */
-		brasero_burn_session_add_flag (priv->session,
-					       BRASERO_BURN_FLAG_DAO|
-					       BRASERO_BURN_FLAG_FAST_BLANK|
-					       BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE);
+		brasero_dest_selection_add_drive_properties_flags (self,
+								   BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE|
+								   BRASERO_BURN_FLAG_FAST_BLANK,
+								   &supported,
+								   &compulsory);
 	}
 	else {
 		/* Try to properly update the flags for the current drive */
@@ -1230,18 +1229,20 @@ brasero_dest_selection_source_changed (BraseroBurnSession *session,
 
 	priv = BRASERO_DEST_SELECTION_PRIVATE (self);
 
+	brasero_drive_info_set_same_src_dest (BRASERO_DRIVE_INFO (priv->info),
+					      brasero_burn_session_same_src_dest_drive (priv->session));
+
 	if (brasero_burn_session_is_dest_file (priv->session)) {
 		/* check that if a path was set there may be none if there was
 		 * no disc inserted when the dialog was created. */
 		if (brasero_burn_session_get_output (priv->session, NULL, NULL, NULL) != BRASERO_BURN_OK)
 			brasero_dest_selection_set_image_properties (self);
+		else
+			brasero_dest_selection_check_image_settings (self);
 
 		return;
 	}
 
-	/* NOTE: that can't happen if we are going to write to an image since
-	 * that would mean we are changing the image format (something we don't
-	 * do). So it has to be when we write to a drive */
 	brasero_dest_selection_set_drive_properties (self);
 }
 
@@ -1325,13 +1326,17 @@ brasero_dest_selection_drive_changed (BraseroDriveSelection *selection,
 
 	priv = BRASERO_DEST_SELECTION_PRIVATE (selection);
 
+	brasero_drive_info_set_medium (BRASERO_DRIVE_INFO (priv->info),
+ 				       brasero_drive_get_medium (drive));
+ 
+ 
 	if (!priv->session)
 		return;
 
 	brasero_burn_session_set_burner (priv->session, drive);
 
 	if (brasero_burn_session_same_src_dest_drive (priv->session))
-		brasero_drive_selection_set_same_src_dest (selection);
+ 		brasero_drive_info_set_same_src_dest (BRASERO_DRIVE_INFO (priv->info), TRUE);
 }
 
 static void
@@ -1355,6 +1360,14 @@ brasero_dest_selection_init (BraseroDestSelection *object)
 
 	priv = BRASERO_DEST_SELECTION_PRIVATE (object);
 
+	priv->info = brasero_drive_info_new ();
+	gtk_widget_show (priv->info);
+	gtk_box_pack_start (GTK_BOX (object),
+			    priv->info,
+			    FALSE,
+			    FALSE,
+			    0);
+
 	priv->caps = brasero_burn_caps_get_default ();
 	manager = brasero_plugin_manager_get_default ();
 	priv->caps_sig = g_signal_connect (manager,
@@ -1472,12 +1485,16 @@ brasero_dest_selection_set_property (GObject *object,
 		if (drive)
 			g_object_unref (drive);
 
-		if (brasero_burn_session_same_src_dest_drive (priv->session))
-			brasero_drive_selection_set_same_src_dest (BRASERO_DRIVE_SELECTION (object));
-		else if (brasero_burn_session_is_dest_file (session))
+ 		brasero_drive_info_set_same_src_dest (BRASERO_DRIVE_INFO (priv->info),
+						      brasero_burn_session_same_src_dest_drive (priv->session));
+
+		/* NOTE: there is no need for the following as we don't know the
+		 * track type yet. Better wait for the track to be set */
+		/* if (brasero_burn_session_is_dest_file (session))
 			brasero_dest_selection_set_image_properties (BRASERO_DEST_SELECTION (object));
 		else
 			brasero_dest_selection_set_drive_properties (BRASERO_DEST_SELECTION (object));
+		*/
 
 		priv->input_sig = g_signal_connect (session,
 						    "input-changed",
