@@ -80,6 +80,7 @@ struct _BraseroDiscMessagePrivate
 	guint timeout;
 
 	guint changing_style:1;
+	guint prevent_destruction:1;
 };
 
 #define BRASERO_DISC_MESSAGE_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_DISC_MESSAGE, BraseroDiscMessagePrivate))
@@ -87,7 +88,6 @@ struct _BraseroDiscMessagePrivate
 enum
 {
 	RESPONSE,
-
 	LAST_SIGNAL
 };
 
@@ -111,10 +111,12 @@ brasero_disc_message_timeout (gpointer data)
 	priv = BRASERO_DISC_MESSAGE_PRIVATE (data);
 	priv->timeout = 0;
 
+	priv->prevent_destruction = TRUE;
 	g_signal_emit (data,
 		       disc_message_signals [RESPONSE],
 		       0,
 		       GTK_RESPONSE_DELETE_EVENT);
+	priv->prevent_destruction = FALSE;
 
 	gtk_widget_destroy (GTK_WIDGET (data));
 	return FALSE;
@@ -163,9 +165,9 @@ brasero_disc_message_expander_activated_cb (GtkExpander *expander,
 					    BraseroDiscMessage *self)
 {
 	if (!gtk_expander_get_expanded (expander))
-		gtk_expander_set_label (expander, _("_Hide errors"));
+		gtk_expander_set_label (expander, _("_Hide changes"));
 	else
-		gtk_expander_set_label (expander, _("_Show errors"));
+		gtk_expander_set_label (expander, _("_Show changes"));
 }
 
 void
@@ -247,10 +249,29 @@ static void
 brasero_disc_message_button_clicked_cb (GtkButton *button,
 					BraseroDiscMessage *self)
 {
+	BraseroDiscMessagePrivate *priv;
+
+	priv = BRASERO_DISC_MESSAGE_PRIVATE (self);
+
+	priv->prevent_destruction = TRUE;
 	g_signal_emit (self,
 		       disc_message_signals [RESPONSE],
-		       0, 
+		       0,
 		       GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button), RESPONSE_TYPE)));
+	priv->prevent_destruction = FALSE;
+
+	gtk_widget_destroy (GTK_WIDGET (self));
+}
+
+void
+brasero_disc_message_destroy (BraseroDiscMessage *self)
+{
+	BraseroDiscMessagePrivate *priv;
+
+	priv = BRASERO_DISC_MESSAGE_PRIVATE (self);
+
+	if (priv->prevent_destruction)
+		return;
 
 	gtk_widget_destroy (GTK_WIDGET (self));
 }
@@ -263,13 +284,19 @@ brasero_disc_message_add_button (BraseroDiscMessage *self,
 				 GtkResponseType response)
 {
 	GtkWidget *button;
+	PangoLayout *layout;
 	BraseroDiscMessagePrivate *priv;
 
 	priv = BRASERO_DISC_MESSAGE_PRIVATE (self);
 
 	button = gtk_button_new_with_mnemonic (text);
-	gtk_size_group_add_widget (group, button);
-	gtk_size_group_add_widget (priv->group, button);
+
+	/* only add buttons to group if the text is not wrapped. Otherwise
+	 * buttons would be too big. */
+	layout = gtk_label_get_layout (priv->primary);
+	if (!pango_layout_is_wrapped (layout))
+		gtk_size_group_add_widget (priv->group, button);
+
 	gtk_widget_set_tooltip_text (button, tooltip);
 	gtk_widget_show (button);
 	g_signal_connect (button,
@@ -291,13 +318,19 @@ void
 brasero_disc_message_add_close_button (BraseroDiscMessage *self)
 {
 	GtkWidget *button;
+	PangoLayout *layout;
 	GtkWidget *alignment;
 	BraseroDiscMessagePrivate *priv;
 
 	priv = BRASERO_DISC_MESSAGE_PRIVATE (self);
 
 	button = gtk_button_new ();
-	gtk_size_group_add_widget (priv->group, button);
+
+	/* only add buttons to group if the text is not wrapped. Otherwise
+	 * buttons would be too big. */
+	layout = gtk_label_get_layout (priv->primary);
+	if (pango_layout_is_wrapped (layout))
+		gtk_size_group_add_widget (priv->group, button);
 
 	alignment = gtk_alignment_new (1.0, 0.0, 0.0, 0.0);
 	gtk_widget_show (alignment);
@@ -445,6 +478,8 @@ brasero_disc_message_set_secondary (BraseroDiscMessage *self,
 
 	if (!priv->secondary) {
 		priv->secondary = gtk_label_new (NULL);
+		gtk_label_set_line_wrap_mode (GTK_LABEL (priv->secondary), GTK_WRAP_WORD);
+		gtk_label_set_line_wrap (GTK_LABEL (priv->secondary), TRUE);
 		gtk_misc_set_alignment (GTK_MISC (priv->secondary), 0.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (priv->text_box), priv->secondary, FALSE, TRUE, 0);
 	}
@@ -522,13 +557,19 @@ brasero_disc_message_init (BraseroDiscMessage *object)
 	gtk_box_pack_start (GTK_BOX (main_box), priv->text_box, TRUE, TRUE, 0);
 
 	priv->primary = gtk_label_new (NULL);
+	gtk_label_set_line_wrap_mode (GTK_LABEL (priv->primary), GTK_WRAP_WORD);
+	gtk_label_set_line_wrap (GTK_LABEL (priv->primary), TRUE);
 	gtk_size_group_add_widget (priv->group, priv->primary);
 	gtk_misc_set_alignment (GTK_MISC (priv->primary), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (priv->text_box), priv->primary, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (priv->text_box), priv->primary, TRUE, TRUE, 0);
 
 	priv->button_box = gtk_vbox_new (FALSE, 8);
 	gtk_widget_show (priv->button_box);
-	gtk_box_pack_start (GTK_BOX (main_box), priv->button_box, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (main_box),
+			    priv->button_box,
+			    FALSE,
+			    FALSE,
+			    0);
 }
 
 static void
