@@ -268,13 +268,14 @@ brasero_disc_option_dialog_update_joliet (BraseroDiscOptionDialog *dialog)
 	if (!priv->joliet_toggle)
 		return FALSE;
 
-	/* what we want to check is Joliet support */
+	/* what we want to check Joliet support */
 	brasero_burn_session_get_input_type (priv->session, &source);
 
 	source.subtype.fs_type |= BRASERO_IMAGE_FS_JOLIET;
 	result = brasero_burn_caps_is_input_supported (priv->caps,
 						       priv->session,
-						       &source);
+						       &source,
+						       FALSE);
 	if (result == BRASERO_BURN_OK) {
 		if (GTK_WIDGET_IS_SENSITIVE (priv->joliet_toggle))
 			return FALSE;
@@ -1241,9 +1242,8 @@ brasero_disc_option_dialog_set_disc (BraseroDiscOptionDialog *dialog,
 
 	brasero_disc_set_session_param (disc, priv->session);
 
-	/* see if we should lock the drive */
-	lock_drive = (brasero_burn_session_get_flags (priv->session) & (BRASERO_BURN_FLAG_APPEND|
-									BRASERO_BURN_FLAG_MERGE)) != 0;
+	/* see if we should lock the drive only with MERGE */
+	lock_drive = (brasero_burn_session_get_flags (priv->session) & BRASERO_BURN_FLAG_MERGE) != 0;
 	brasero_drive_selection_lock (BRASERO_DRIVE_SELECTION (priv->selection), lock_drive);
 
 	priv->output_sig = g_signal_connect (priv->session,
