@@ -64,7 +64,7 @@ void
 brasero_drive_selection_lock (BraseroDriveSelection *selection,
 			      gboolean locked);
 
-void
+gboolean
 brasero_drive_selection_set_drive (BraseroDriveSelection *selection,
 				   BraseroDrive *drive);
 
@@ -78,13 +78,6 @@ brasero_drive_selection_get_drive (BraseroDriveSelection *selection);
 BraseroMedium *
 brasero_drive_selection_get_medium (BraseroDriveSelection *selection);
 
-void
-brasero_drive_selection_set_image_path (BraseroDriveSelection *selection,
-					const gchar *path);
-
-void
-brasero_drive_selection_set_same_src_dest (BraseroDriveSelection *selection);
-
 void
 brasero_drive_selection_set_button (BraseroDriveSelection *selection,
 				    GtkWidget *button);
