@@ -55,7 +55,6 @@
 enum  {
 	STOCK_ID_COL,
 	UNESCAPED_URI_COL,
-	URI_COL,
 	TYPE_COL,
 	STATUS_COL,
 	ACTIVABLE_COL,
@@ -68,6 +67,14 @@ struct _BraseroFileFilteredPrivate
 	GtkWidget *tree;
 	GtkWidget *restore;
 	GtkWidget *options;
+
+	GSList *broken;
+	GSList *hidden;
+	GSList *recursive;
+	GSList *unreadable;
+
+	guint idle_id;
+
 	guint num;
 };
 
@@ -121,7 +128,8 @@ brasero_file_filtered_update (BraseroFileFiltered *self)
 
 	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
 
-	markup = brasero_file_filtered_get_label_text (priv->num, gtk_expander_get_expanded (GTK_EXPANDER (self)));
+	markup = brasero_file_filtered_get_label_text (priv->num,
+						       gtk_expander_get_expanded (GTK_EXPANDER (self)));
 	widget = gtk_expander_get_label_widget (GTK_EXPANDER (self));
 	gtk_label_set_markup_with_mnemonic (GTK_LABEL (widget), markup);
 	g_free (markup);
@@ -188,10 +196,10 @@ brasero_file_filtered_remove (BraseroFileFiltered *self,
 	brasero_file_filtered_update (self);
 }
 
-void
-brasero_file_filtered_add (BraseroFileFiltered *self,
-			   const gchar *uri,
-			   BraseroFilterStatus status)
+static void
+brasero_file_filtered_add_real (BraseroFileFiltered *self,
+				const gchar *unescaped_uri,
+				BraseroFilterStatus status)
 {
 	gchar *labels [] = { N_("hidden file"),
 			     N_("unreadable file"),
@@ -200,7 +208,6 @@ brasero_file_filtered_add (BraseroFileFiltered *self,
 			     NULL };
 	BraseroFileFilteredPrivate *priv;
 	const gchar *stock_id;
-	gchar *unescaped_uri;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	gchar *type;
@@ -216,20 +223,120 @@ brasero_file_filtered_add (BraseroFileFiltered *self,
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 
-	unescaped_uri = g_uri_unescape_string (uri, NULL);
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 			    STOCK_ID_COL, stock_id,
 			    UNESCAPED_URI_COL, unescaped_uri,
-			    URI_COL, uri,
 			    TYPE_COL, _(type),
 			    STATUS_COL, status,
 			    ACTIVABLE_COL, (status != BRASERO_FILTER_UNREADABLE && status != BRASERO_FILTER_RECURSIVE_SYM),
 			    -1);
-	g_free (unescaped_uri);
+
+	priv->num ++;
+}
+
+static gboolean
+brasero_file_filtered_add_loop (gpointer data)
+{
+	GSList *iter;
+	BraseroFileFilteredPrivate *priv;
+	BraseroFileFiltered *self = BRASERO_FILE_FILTERED (data);
+
+	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
+
+	for (iter = priv->hidden; iter; iter = iter->next) {
+		gchar *unescaped_uri;
+
+		unescaped_uri = iter->data;
+		brasero_file_filtered_add_real (self,
+						unescaped_uri,
+						BRASERO_FILTER_HIDDEN);
+		g_free (unescaped_uri);
+	}
+	g_slist_free (priv->hidden);
+	priv->hidden = NULL;
+
+	for (iter = priv->broken; iter; iter = iter->next) {
+		gchar *unescaped_uri;
+
+		unescaped_uri = iter->data;
+		brasero_file_filtered_add_real (self,
+						unescaped_uri,
+						BRASERO_FILTER_BROKEN_SYM);
+		g_free (unescaped_uri);
+	}
+	g_slist_free (priv->broken);
+	priv->broken = NULL;
+
+	for (iter = priv->recursive; iter; iter = iter->next) {
+		gchar *unescaped_uri;
+
+		unescaped_uri = iter->data;
+		brasero_file_filtered_add_real (self,
+						unescaped_uri,
+						BRASERO_FILTER_RECURSIVE_SYM);
+		g_free (unescaped_uri);
+	}
+	g_slist_free (priv->recursive);
+	priv->recursive = NULL;
+
+	for (iter = priv->unreadable; iter; iter = iter->next) {
+		gchar *unescaped_uri;
+
+		unescaped_uri = iter->data;
+		brasero_file_filtered_add_real (self,
+						unescaped_uri,
+						BRASERO_FILTER_UNREADABLE);
+		g_free (unescaped_uri);
+	}
+	g_slist_free (priv->unreadable);
+	priv->unreadable = NULL;
 
 	/* update label */
-	priv->num ++;
 	brasero_file_filtered_update (self);
+
+	priv->idle_id = 0;
+	return FALSE;
+}
+
+void
+brasero_file_filtered_add (BraseroFileFiltered *self,
+			   const gchar *uri,
+			   BraseroFilterStatus status)
+{
+	BraseroFileFilteredPrivate *priv;
+
+	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
+
+	/* The idea here is to delay the introduction of each file in the tree
+	 * and the label update so as not slow down brasero too much */
+	switch (status) {
+	case BRASERO_FILTER_HIDDEN:
+		priv->hidden = g_slist_prepend (priv->hidden,
+						g_uri_unescape_string (uri, NULL));
+		break;
+
+	case BRASERO_FILTER_BROKEN_SYM:
+		priv->broken = g_slist_prepend (priv->broken,
+						g_uri_unescape_string (uri, NULL));
+		break;
+	case BRASERO_FILTER_RECURSIVE_SYM:
+		priv->recursive = g_slist_prepend (priv->recursive,
+						   g_uri_unescape_string (uri, NULL));
+		break;
+	case BRASERO_FILTER_UNREADABLE:
+		priv->unreadable = g_slist_prepend (priv->unreadable,
+						    g_uri_unescape_string (uri, NULL));
+		break;
+	case BRASERO_FILTER_NONE:
+	case BRASERO_FILTER_UNKNOWN:
+		default:
+		break;
+	}
+
+	if (!priv->idle_id)
+		priv->idle_id = g_timeout_add (1000,
+					       brasero_file_filtered_add_loop,
+					       self);
 }
 
 static void
@@ -275,6 +382,7 @@ brasero_file_filtered_restore_pressed_cb (GtkButton *button,
 	for (iter = selected; iter; iter = iter->next) {
 		GtkTreePath *treepath;
 		GtkTreeIter treeiter;
+		gchar *escaped_uri;
 		gchar *uri;
 
 		treepath = iter->data;
@@ -282,9 +390,18 @@ brasero_file_filtered_restore_pressed_cb (GtkButton *button,
 		gtk_tree_path_free (treepath);
 
 		gtk_tree_model_get (model, &treeiter,
-				    URI_COL, &uri, 
+				    UNESCAPED_URI_COL, &uri, 
 				    -1);
-		g_signal_emit (self, file_filtered_signals [RESTORED_SIGNAL], 0, uri);
+
+		escaped_uri = g_uri_escape_string (uri,
+						   G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
+						   FALSE);
+		g_signal_emit (self,
+			       file_filtered_signals [RESTORED_SIGNAL],
+			       0,
+			       escaped_uri);
+		g_free (escaped_uri);
+
 		gtk_list_store_remove (GTK_LIST_STORE (model), &treeiter);
 		priv->num --;
 	}
@@ -316,6 +433,35 @@ brasero_file_filtered_clear (BraseroFileFiltered *self)
 
 	priv = BRASERO_FILE_FILTERED_PRIVATE (self);
 
+	if (priv->idle_id) {
+		g_source_remove (priv->idle_id);
+		priv->idle_id = 0;
+	}
+
+	if (priv->hidden) {
+		g_slist_foreach (priv->hidden, (GFunc) g_free, NULL);
+		g_slist_free (priv->hidden);
+		priv->hidden = NULL;
+	}
+
+	if (priv->broken) {
+		g_slist_foreach (priv->broken, (GFunc) g_free, NULL);
+		g_slist_free (priv->broken);
+		priv->broken = NULL;
+	}
+
+	if (priv->recursive) {
+		g_slist_foreach (priv->recursive, (GFunc) g_free, NULL);
+		g_slist_free (priv->recursive);
+		priv->recursive = NULL;
+	}
+
+	if (priv->unreadable) {
+		g_slist_foreach (priv->unreadable, (GFunc) g_free, NULL);
+		g_slist_free (priv->unreadable);
+		priv->unreadable = NULL;
+	}
+
 	priv->num = 0;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
@@ -363,7 +509,6 @@ brasero_file_filtered_init (BraseroFileFiltered *object)
 				    G_TYPE_STRING,
 				    G_TYPE_STRING,
 				    G_TYPE_STRING,
-				    G_TYPE_STRING,
 				    G_TYPE_INT,
 				    G_TYPE_BOOLEAN);
 
@@ -390,10 +535,10 @@ brasero_file_filtered_init (BraseroFileFiltered *object)
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_end (column, renderer, TRUE);
 	gtk_tree_view_column_add_attribute (column, renderer,
-					    "text", URI_COL);
+					    "text", UNESCAPED_URI_COL);
 	gtk_tree_view_column_set_title (column, _("Files"));
 	gtk_tree_view_append_column (GTK_TREE_VIEW (priv->tree), column);
-	gtk_tree_view_column_set_sort_column_id (column, URI_COL);
+	gtk_tree_view_column_set_sort_column_id (column, UNESCAPED_URI_COL);
 	gtk_tree_view_column_set_clickable (column, TRUE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
@@ -457,6 +602,39 @@ brasero_file_filtered_init (BraseroFileFiltered *object)
 static void
 brasero_file_filtered_finalize (GObject *object)
 {
+	BraseroFileFilteredPrivate *priv;
+
+	priv = BRASERO_FILE_FILTERED_PRIVATE (object);
+
+	if (priv->idle_id) {
+		g_source_remove (priv->idle_id);
+		priv->idle_id = 0;
+	}
+
+	if (priv->hidden) {
+		g_slist_foreach (priv->hidden, (GFunc) g_free, NULL);
+		g_slist_free (priv->hidden);
+		priv->hidden = NULL;
+	}
+
+	if (priv->broken) {
+		g_slist_foreach (priv->broken, (GFunc) g_free, NULL);
+		g_slist_free (priv->broken);
+		priv->broken = NULL;
+	}
+
+	if (priv->recursive) {
+		g_slist_foreach (priv->recursive, (GFunc) g_free, NULL);
+		g_slist_free (priv->recursive);
+		priv->recursive = NULL;
+	}
+
+	if (priv->unreadable) {
+		g_slist_foreach (priv->unreadable, (GFunc) g_free, NULL);
+		g_slist_free (priv->unreadable);
+		priv->unreadable = NULL;
+	}
+
 	G_OBJECT_CLASS (brasero_file_filtered_parent_class)->finalize (object);
 }
 
