@@ -651,7 +651,7 @@ brasero_file_node_ungraft (BraseroFileNode *node)
 	/* Remove it from the URINode list of grafts */
 	graft->node->nodes = g_slist_remove (graft->node->nodes, node);
 
-	/* The name must be exactly the one of the URI*/
+	/* The name must be exactly the one of the URI */
 	node->is_grafted = FALSE;
 	node->union1.name = graft->name;
 
@@ -754,9 +754,9 @@ brasero_file_node_set_from_info (BraseroFileNode *node,
 		sectors = BRASERO_SIZE_TO_SECTORS (g_file_info_get_size (info), 2048);
 
 		if (sectors > BRASERO_FILE_2G_LIMIT && BRASERO_FILE_NODE_SECTORS (node) <= BRASERO_FILE_2G_LIMIT)
-			stats->num_2Gio ++;
+			stats->num_2GiB ++;
 		else if (sectors <= BRASERO_FILE_2G_LIMIT && BRASERO_FILE_NODE_SECTORS (node) > BRASERO_FILE_2G_LIMIT)
-			stats->num_2Gio --;
+			stats->num_2GiB --;
 
 		/* The node isn't grafted and it's a file. So we must propagate
 		 * its size up to the parent graft node. */
@@ -777,7 +777,7 @@ brasero_file_node_set_from_info (BraseroFileNode *node,
 gchar *
 brasero_file_node_get_uri_name (const gchar *uri)
 {
-	gchar *unescaped_name;
+	gchar *utf8_name;
 	GFile *vfs_uri;
 	gchar *name;
 
@@ -787,18 +787,18 @@ brasero_file_node_get_uri_name (const gchar *uri)
 	name = g_file_get_basename (vfs_uri);
 	g_object_unref (vfs_uri);
 
-	unescaped_name = g_uri_unescape_string (name, NULL);
-	g_free (name);
+	/* NOTE and reminder names are already unescaped; the following is not
+	 * needed: unescaped_name = g_uri_unescape_string (name, NULL); */
 
 	/* NOTE: a graft should be added for non utf8 name since we
 	 * modify them; in fact we use this function only in the next
 	 * one which creates only grafted nodes. */
-	name = brasero_file_node_validate_utf8_name (unescaped_name);
-	if (name) {
-		g_free (unescaped_name);
-		return name;
+	utf8_name = brasero_file_node_validate_utf8_name (name);
+	if (utf8_name) {
+		g_free (name);
+		return utf8_name;
 	}
-	return unescaped_name;
+	return name;
 }
 
 BraseroFileNode *
@@ -1015,9 +1015,9 @@ brasero_file_node_destroy_with_children (BraseroFileNode *node,
 
 	/* update all statistics on tree if any */
 	if (stats) {
-		/* check if that's a 2 Gio file */
-		if (node->is_2Gio)
-			stats->num_2Gio --;
+		/* check if that's a 2 GiB file */
+		if (node->is_2GiB)
+			stats->num_2GiB --;
 
 		/* check if that's a deep directory file */
 		if (node->is_deep)
