@@ -59,9 +59,6 @@ struct _BraseroImageOptionDialogPrivate {
 
 	BraseroBurnCaps *caps;
 
-	gulong caps_sig;
-	gulong session_sig;
-
 	BraseroIO *io;
 	BraseroIOJobBase *info_type;
 
@@ -91,15 +88,42 @@ brasero_image_option_dialog_set_track (BraseroImageOptionDialog *dialog,
 
 	priv = BRASERO_IMAGE_OPTION_DIALOG_PRIVATE (dialog);
 
-	if (!priv->track) {
-		priv->track = brasero_track_new (BRASERO_TRACK_TYPE_IMAGE);
-		brasero_burn_session_add_track (priv->session, priv->track);
+	/* add a track every time to send a signal */
+	if (priv->track) {
+		gchar *uri = NULL;
+		BraseroTrackType type = { 0, };
+
+		/* make sure something actually changed */
+		brasero_track_get_type (priv->track, &type);
+
+		if (image)
+			uri = brasero_track_get_image_source (priv->track, TRUE);
+		else if (toc)
+			uri = brasero_track_get_toc_source (priv->track, TRUE);
+
+		if (!toc && !image && !uri)
+			return;
+
+		if((format == type.subtype.img_format)
+		&&  uri && (image || toc)
+		&& !strcmp (uri, image?image:toc)) {
+			g_free (uri);
+			return;
+		}
+
+		g_free (uri);
+
+		brasero_burn_session_clear_current_track (priv->session);
+		brasero_track_unref (priv->track);
 	}
 
+	/* set image type before adding so that signal has the right type */
+	priv->track = brasero_track_new (BRASERO_TRACK_TYPE_IMAGE);
 	brasero_track_set_image_source (priv->track,
 					image,
 					toc,
 					format);
+	brasero_burn_session_add_track (priv->session, priv->track);
 }
 
 static void
@@ -122,12 +146,32 @@ brasero_image_option_dialog_image_info_cb (GObject *object,
 		return;
 	}
 
-    	/* Add it to recent file manager */
-	if (!strcmp (g_file_info_get_content_type (info), "application/x-toc"))
-		brasero_image_option_dialog_set_track (dialog,
-						       BRASERO_IMAGE_FORMAT_CLONE,
-						       NULL,
-						       uri);
+	if (!strcmp (g_file_info_get_content_type (info), "application/x-toc")
+	||  !strcmp (g_file_info_get_content_type (info), "application/x-cdrdao-toc")
+	||  !strcmp (g_file_info_get_content_type (info), "application/x-cue")) {
+		BraseroImageFormat format;
+		gchar *path;
+
+		path = g_filename_from_uri (uri, NULL, NULL);
+		format = brasero_image_format_identify_cuesheet (path);
+		g_free (path);
+
+		if (format != BRASERO_IMAGE_FORMAT_NONE)
+			brasero_image_option_dialog_set_track (dialog,
+							       format,
+							       NULL,
+							       uri);
+		else if (g_str_has_suffix (path, ".toc"))
+			brasero_image_option_dialog_set_track (dialog,
+							       BRASERO_IMAGE_FORMAT_CLONE,
+							       NULL,
+							       uri);
+		else
+			brasero_image_option_dialog_set_track (dialog,
+							       BRASERO_IMAGE_FORMAT_NONE,
+							       NULL,
+							       uri);
+	}
 	else if (!strcmp (g_file_info_get_content_type (info), "application/octet-stream")) {
 		/* that could be an image, so here is the deal:
 		 * if we can find the type through the extension, fine.
@@ -153,25 +197,6 @@ brasero_image_option_dialog_image_info_cb (GObject *object,
 						       BRASERO_IMAGE_FORMAT_BIN,
 						       uri,
 						       NULL);
-	else if (!strcmp (g_file_info_get_content_type (info), "application/x-cdrdao-toc"))
-		/* This image type is installed by brasero */
-		brasero_image_option_dialog_set_track (dialog,
-						       BRASERO_IMAGE_FORMAT_CDRDAO,
-						       NULL,
-						       uri);
-	else if (!strcmp (g_file_info_get_content_type (info), "application/x-cue")) {
-		BraseroImageFormat format;
-		gchar *path;
-
-		path = g_filename_from_uri (uri, NULL, NULL);
-		format = brasero_image_format_identify_cuesheet (path);
-		g_free (path);
-
-		brasero_image_option_dialog_set_track (dialog,
-						       BRASERO_IMAGE_FORMAT_CUE,
-						       NULL,
-						       uri);
-	}
 	else
 		brasero_image_option_dialog_set_track (dialog,
 						       BRASERO_IMAGE_FORMAT_NONE,
@@ -221,8 +246,7 @@ brasero_image_option_dialog_changed (BraseroImageOptionDialog *dialog)
 	priv = BRASERO_IMAGE_OPTION_DIALOG_PRIVATE (dialog);
 
 	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (priv->file));
-	brasero_image_type_chooser_get_format (BRASERO_IMAGE_TYPE_CHOOSER (priv->format),
-					       &format);
+	brasero_image_type_chooser_get_format (BRASERO_IMAGE_TYPE_CHOOSER (priv->format), &format);
 
 	switch (format) {
 	case BRASERO_IMAGE_FORMAT_NONE:
@@ -258,6 +282,20 @@ brasero_image_option_dialog_changed (BraseroImageOptionDialog *dialog)
 	g_free (uri);	
 }
 
+static void
+brasero_image_option_dialog_format_changed (BraseroImageTypeChooser *format,
+					    BraseroImageOptionDialog *dialog)
+{
+	brasero_image_option_dialog_changed (dialog);
+}
+
+static void
+brasero_image_option_dialog_file_changed (GtkFileChooser *chooser,
+					  BraseroImageOptionDialog *dialog)
+{
+	brasero_image_option_dialog_changed (dialog);
+}
+
 static void
 brasero_image_option_dialog_set_formats (BraseroImageOptionDialog *dialog)
 {
@@ -290,42 +328,16 @@ brasero_image_option_dialog_set_formats (BraseroImageOptionDialog *dialog)
 		input.subtype.img_format = format;
 		result = brasero_burn_caps_is_input_supported (priv->caps,
 							       priv->session,
-							       &input);
+							       &input,
+							       FALSE);
 		if (result == BRASERO_BURN_OK)
 			formats |= format;
 	}
+
 	brasero_image_type_chooser_set_formats (BRASERO_IMAGE_TYPE_CHOOSER (priv->format),
 					        formats);
 }
 
-static void
-brasero_image_option_dialog_format_changed (BraseroImageTypeChooser *format,
-					    BraseroImageOptionDialog *dialog)
-{
-	brasero_image_option_dialog_changed (dialog);
-}
-
-static void
-brasero_image_option_dialog_file_changed (GtkFileChooser *chooser,
-					  BraseroImageOptionDialog *dialog)
-{
-	brasero_image_option_dialog_changed (dialog);
-}
-
-static void
-brasero_image_option_dialog_output_changed_cb (BraseroBurnSession *session,
-					       BraseroImageOptionDialog *dialog)
-{
-	brasero_image_option_dialog_set_formats (dialog);
-}
-
-static void
-brasero_image_option_dialog_caps_changed (BraseroPluginManager *manager,
-					  BraseroImageOptionDialog *dialog)
-{
-	brasero_image_option_dialog_set_formats (dialog);
-}
-
 void
 brasero_image_option_dialog_set_image_uri (BraseroImageOptionDialog *dialog,
 					   const gchar *uri)
@@ -472,6 +484,8 @@ brasero_image_option_dialog_valid_media_cb (BraseroDestSelection *selection,
 	BraseroImageOptionDialogPrivate *priv;
 
 	priv = BRASERO_IMAGE_OPTION_DIALOG_PRIVATE (self);
+
+	brasero_image_option_dialog_set_formats (self);
 	gtk_widget_set_sensitive (priv->button, valid);
 }
 
@@ -504,7 +518,6 @@ brasero_image_option_dialog_init (BraseroImageOptionDialog *obj)
 	GConfClient *client;
 	GtkWidget *box, *box1;
 	GtkFileFilter *filter;
-	BraseroPluginManager *manager;
 	BraseroImageOptionDialogPrivate *priv;
 
 	priv = BRASERO_IMAGE_OPTION_DIALOG_PRIVATE (obj);
@@ -527,11 +540,6 @@ brasero_image_option_dialog_init (BraseroImageOptionDialog *obj)
 				      GTK_RESPONSE_OK);
 
 	priv->caps = brasero_burn_caps_get_default ();
-	manager = brasero_plugin_manager_get_default ();
-	priv->caps_sig = g_signal_connect (manager,
-					   "caps-changed",
-					   G_CALLBACK (brasero_image_option_dialog_caps_changed),
-					   obj);
 
 	priv->session = brasero_burn_session_new ();
 	brasero_burn_session_add_flag (priv->session,
@@ -541,10 +549,6 @@ brasero_image_option_dialog_init (BraseroImageOptionDialog *obj)
 				       BRASERO_BURN_FLAG_CHECK_SIZE|
 				       BRASERO_BURN_FLAG_DONT_CLEAN_OUTPUT|
 				       BRASERO_BURN_FLAG_FAST_BLANK);
-	priv->session_sig = g_signal_connect (priv->session,
-					      "output-changed",
-					      G_CALLBACK (brasero_image_option_dialog_output_changed_cb),
-					      obj);
 
 	/* first box */
 	priv->selection = brasero_dest_selection_new (priv->session);
@@ -701,19 +705,6 @@ brasero_image_option_dialog_finalize (GObject *object)
 		priv->track = NULL;
 	}
 
-	if (priv->caps_sig) {
-		BraseroPluginManager *manager;
-
-		manager = brasero_plugin_manager_get_default ();
-		g_signal_handler_disconnect (manager, priv->caps_sig);
-		priv->caps_sig = 0;
-	}
-
-	if (priv->session_sig) {
-		g_signal_handler_disconnect (priv->session, priv->session_sig);
-		priv->session_sig = 0;
-	}
-
 	if (priv->session) {
 		g_object_unref (priv->session);
 		priv->session = NULL;
