@@ -57,6 +57,7 @@ struct _BraseroIOPrivate
 
 	/* used for metadata */
 	GSList *metadatas;
+	GSList *metadata_running;
 
 	/* used to "buffer" some results returned by metadata.
 	 * It takes time to return metadata and it's not unusual
@@ -76,7 +77,7 @@ struct _BraseroIOPrivate
 
 struct _BraseroIOResultCallbackData {
 	gpointer callback_data;
-	guint ref;
+	gint ref;
 };
 typedef struct _BraseroIOResultCallbackData BraseroIOResultCallbackData;
 
@@ -250,8 +251,8 @@ brasero_io_unref_result_callback_data (BraseroIOResultCallbackData *data,
 	if (!data)
 		return;
 
-	data->ref --;
-	if (data->ref > 0)
+	/* see atomically if we are the last to hold a lock */
+	if (!g_atomic_int_dec_and_test (&data->ref))
 		return;
 
 	if (destroy)
@@ -276,33 +277,6 @@ brasero_io_job_result_free (BraseroIOJobResult *result)
 	g_free (result);
 }
 
-static void
-brasero_io_job_free (BraseroIOJob *job)
-{
-	if (job->callback_data)
-		job->callback_data->ref --;
-
-	g_free (job->uri);
-	g_free (job);
-}
-
-static void
-brasero_io_job_destroy (BraseroAsyncTaskManager *self,
-			gpointer callback_data)
-{
-	BraseroIOJob *job = callback_data;
-
-	/* If a job is destroyed we don't call the destroy callback since it
-	 * otherwise it would be called in a different thread. All object that
-	 * cancel io ops are doing it either in destroy () and therefore handle
-	 * all destruction for callback_data or if they don't they usually don't
-	 * pass any callback data anyway. */
-	/* NOTE: usually threads are cancelled from the main thread/loop and
-	 * block until the active task is removed which means that if we called
-	 * the destroy () then the destruction would be done in the main loop */
-	brasero_io_job_free (job);
-}
-
 /**
  * Used to return the results
  */
@@ -314,36 +288,41 @@ brasero_io_return_result_idle (gpointer callback_data)
 	BraseroIOResultCallbackData *data;
 	BraseroIOJobResult *result;
 	BraseroIOPrivate *priv;
+	int i;
 
 	priv = BRASERO_IO_PRIVATE (self);
 
-	g_mutex_lock (priv->lock);
+	/* Return several results at a time that can be a huge speed gain.
+	 * What should be the value that provides speed and responsiveness. */
+	for (i = 0; i < 25; i ++) {
+		g_mutex_lock (priv->lock);
+
+		if (!priv->results) {
+			priv->results_id = 0;
+			g_mutex_unlock (priv->lock);
+			return FALSE;
+		}
+
+		result = priv->results->data;
+		priv->results = g_slist_remove (priv->results, result);
 
-	if (!priv->results) {
-		priv->results_id = 0;
 		g_mutex_unlock (priv->lock);
-		return FALSE;
+
+		data = result->callback_data;
+		if (result->uri || result->info || result->error)
+			result->base->callback (result->base->object,
+						result->error,
+						result->uri,
+						result->info,
+						data? data->callback_data:NULL);
+
+		/* Else this is just to call destroy () for callback data */
+		brasero_io_unref_result_callback_data (data,
+						       result->base->object,
+						       result->base->destroy,
+						       FALSE);
+		brasero_io_job_result_free (result);
 	}
-
-	result = priv->results->data;
-	priv->results = g_slist_remove (priv->results, result);
-
-	g_mutex_unlock (priv->lock);
-
-	data = result->callback_data;
-	if (result->uri || result->info || result->error)
-		result->base->callback (result->base->object,
-					result->error,
-					result->uri,
-					result->info,
-					data? data->callback_data:NULL);
-
-	/* Else this is just to call destroy () for callback data */
-	brasero_io_unref_result_callback_data (data,
-					       result->base->object,
-					       result->base->destroy,
-					       FALSE);
-	brasero_io_job_result_free (result);
 	return TRUE;
 }
 
@@ -383,8 +362,8 @@ brasero_io_return_result (BraseroIO *self,
 	result->uri = g_strdup (uri);
 
 	if (callback_data) {
+		g_atomic_int_inc (&callback_data->ref);
 		result->callback_data = callback_data;
-		callback_data->ref ++;
 	}
 
 	brasero_io_queue_result (self, result);
@@ -404,13 +383,10 @@ brasero_io_set_job (BraseroIOJob *job,
 	job->base = base;
 	job->uri = g_strdup (uri);
 	job->options = options;
+	job->callback_data = callback_data;
 
-	if (callback_data) {
-		job->callback_data = callback_data;
-		job->callback_data->ref ++;
-	}
-	else
-		job->callback_data = NULL;
+	if (callback_data)
+		g_atomic_int_inc (&job->callback_data->ref);
 }
 
 static void
@@ -435,6 +411,72 @@ brasero_io_push_job (BraseroIO *self,
 						  job);
 }
 
+/**
+ * Job destruction
+ */
+
+static void
+brasero_io_job_free (BraseroIO *self,
+		     gboolean cancelled,
+		     BraseroIOJob *job)
+{
+	/* NOTE: the callback_data member is never destroyed here since it would
+	 * be destroyed in a thread different from the main loop.
+	 * Either it's destroyed in the thread that called brasero_io_cancel ()
+	 * or after all results are returned (and therefore in main loop).
+	 * As a special case, some jobs like read directory contents have to
+	 * return a dummy result to destroy the callback_data if the directory
+	 * is empty.
+	 * If the job happens to be the last to carry a reference then destroy
+	 * it but do it in the main loop by sending a dummy result. */
+
+	/* NOTE2: that's also used for directory loading when there aren't any
+	 * result to notify the caller that the operation has finished but that
+	 * there weren't any result to report. */
+	if (job->callback_data) {
+		/* see atomically if we are the last to hold a lock:
+		 * If so, and if cancelled is TRUE destroy it now since we are
+		 * always cancelled (and destroyed in the main loop). Otherwise
+		 * add a dummy result to destroy callback_data. */
+		if (g_atomic_int_dec_and_test (&job->callback_data->ref)) {
+			if (cancelled) {
+				job->base->destroy (job->base->object,
+						    TRUE,
+						    job->callback_data);
+				g_free (job->callback_data);
+			}
+			else
+				brasero_io_return_result (self,
+							  job->base,
+							  NULL,
+							  NULL,
+							  NULL,
+							  job->callback_data);
+		}
+	}
+
+	g_free (job->uri);
+	g_free (job);
+}
+
+static void
+brasero_io_job_destroy (BraseroAsyncTaskManager *manager,
+			gboolean cancelled,
+			gpointer callback_data)
+{
+	BraseroIOJob *job = callback_data;
+
+	/* If a job is destroyed we don't call the destroy callback since it
+	 * otherwise it would be called in a different thread. All object that
+	 * cancel io ops are doing it either in destroy () and therefore handle
+	 * all destruction for callback_data or if they don't they usually don't
+	 * pass any callback data anyway. */
+	/* NOTE: usually threads are cancelled from the main thread/loop and
+	 * block until the active task is removed which means that if we called
+	 * the destroy () then the destruction would be done in the main loop */
+	brasero_io_job_free (BRASERO_IO (manager), cancelled, job);
+}
+
 /**
  * This part deals with symlinks, that allows to get unique filenames by
  * replacing any parent symlink by its target and check for recursive
@@ -446,10 +488,14 @@ brasero_io_check_for_parent_symlink (const gchar *escaped_uri,
 				     GCancellable *cancel)
 {
 	GFile *parent;
+	GFile *file;
     	gchar *uri;
 
-	parent = g_file_new_for_uri (escaped_uri);
-    	uri = g_file_get_uri (parent);
+	/* don't check if the node itself is a symlink since that'll be done */
+	file = g_file_new_for_uri (escaped_uri);
+    	uri = g_file_get_uri (file);
+	parent = g_file_get_parent (file);
+	g_object_unref (file);
 
 	while (parent) {
 	    	GFile *tmp;
@@ -490,10 +536,10 @@ brasero_io_check_for_parent_symlink (const gchar *escaped_uri,
 			else
 				new_root = g_filename_to_uri (target_path, NULL, NULL);
 
-			newuri = g_strconcat (new_root,
-					      uri + strlen (parent_uri),
-					      NULL);
-
+			newuri = g_build_path (G_DIR_SEPARATOR_S,
+					       new_root,
+					       uri + strlen (parent_uri),
+					       NULL);
 		    	g_free (uri);
 		    	uri = newuri;	
 
@@ -644,8 +690,12 @@ brasero_io_get_metadata_info (BraseroIO *self,
 	BraseroIOPrivate *priv;
 	const gchar *mime;
 	gboolean result;
+	GSList *iter;
 	GList *node;
 
+	if (g_cancellable_is_cancelled (cancel))
+		return FALSE;
+
 	priv = BRASERO_IO_PRIVATE (self);
 
 	mime = g_file_info_get_content_type (info);
@@ -657,6 +707,40 @@ brasero_io_get_metadata_info (BraseroIO *self,
 	||  !strcmp (mime, "application/octet-stream")))
 		return FALSE;
 
+	g_mutex_lock (priv->lock);
+
+	/* First see if a metadata is running with the same uri and the same
+	 * flags as us. In this case, acquire the lock and wait for the lock
+	 * to become available which will mean that it has finished
+	 * NOTE: since we will hold the lock another thread waiting to get 
+	 * an available metadata won't be able to have this one. */
+	for (iter = priv->metadata_running; iter; iter = iter->next) {
+		const gchar *metadata_uri;
+		BraseroMetadata *metadata_iter;
+		BraseroMetadataFlag metadata_flags;
+
+		metadata_iter = iter->data;
+		metadata_uri = brasero_metadata_get_uri (metadata_iter);
+		metadata_flags = brasero_metadata_get_flags (metadata_iter);
+
+		if (((flags & metadata_flags) == flags)
+		&&  !strcmp (uri, metadata_uri)) {
+			/* Found one: release the IO lock to let other threads
+			 * do what they need to do then lock the metadata lock
+			 * Let the thread that got the lock first move it back
+			 * to waiting queue */
+			BRASERO_BURN_LOG ("Already ongoing search for %s", uri);
+
+			metadata = metadata_iter;
+
+			g_mutex_unlock (priv->lock);
+			brasero_metadata_lock (metadata);
+			result = brasero_metadata_set_info (metadata, meta_info);
+			brasero_metadata_unlock (metadata);
+
+			return result;
+		}
+	}
 	/* Seek in the buffer if we have already explored these metadata. Check 
 	 * the info last modified time in case a result should be updated. */
 	node = g_queue_find_custom (priv->meta_buffer,
@@ -673,11 +757,13 @@ brasero_io_get_metadata_info (BraseroIO *self,
 				/* If there isn't any snapshot retry */
 				if (cached->info->snapshot) {
 					brasero_metadata_info_copy (meta_info, cached->info);
+					g_mutex_unlock (priv->lock);
 					return TRUE;
 				}
 			}
 			else {
 				brasero_metadata_info_copy (meta_info, cached->info);
+				g_mutex_unlock (priv->lock);
 				return TRUE;
 			}
 		}
@@ -692,16 +778,32 @@ brasero_io_get_metadata_info (BraseroIO *self,
 	/* grab an available metadata (NOTE: there should always be at least one
 	 * since we run 2 threads at max and have two metadatas available) */
 	do {
-		g_mutex_lock (priv->lock);
-		if (priv->metadatas) {
-			metadata = priv->metadatas->data;
-			priv->metadatas = g_slist_remove (priv->metadatas, metadata);
-		}
-		g_mutex_unlock (priv->lock);
 
+		if (g_cancellable_is_cancelled (cancel)) {
+			g_mutex_unlock (priv->lock);
+			return FALSE;
+		}
+
+		for (iter = priv->metadatas; iter; iter = iter->next) {
+			BraseroMetadata *metadata_iter;
+
+			metadata_iter = iter->data;
+			if (brasero_metadata_try_lock (metadata_iter)) {
+				metadata = priv->metadatas->data;
+				priv->metadatas = g_slist_remove (priv->metadatas, metadata);
+				priv->metadata_running = g_slist_prepend (priv->metadata_running, metadata);
+				break;
+			}
+		}
+
+		g_mutex_unlock (priv->lock);
 		g_usleep (250);
+		g_mutex_lock (priv->lock);
+
 	} while (!metadata);
 
+	g_mutex_unlock (priv->lock);
+
 	result = brasero_metadata_get_info_wait (metadata,
 						 cancel,
 						 uri,
@@ -709,6 +811,13 @@ brasero_io_get_metadata_info (BraseroIO *self,
 						 NULL);
 	brasero_metadata_set_info (metadata, meta_info);
 
+	g_mutex_lock (priv->lock);
+
+	/* release the lock on the metadata in case another thread is waiting 
+	 * for this search result. That way that thread will be able to wake
+	 * up. */
+	brasero_metadata_unlock (metadata);
+
 	if (result) {
 		/* see if we should add it to the buffer */
 		if (meta_info->has_audio || meta_info->has_video) {
@@ -728,8 +837,9 @@ brasero_io_get_metadata_info (BraseroIO *self,
 		}
 	}
 
-	g_mutex_lock (priv->lock);
+	priv->metadata_running = g_slist_remove (priv->metadata_running, metadata);
 	priv->metadatas = g_slist_prepend (priv->metadatas, metadata);
+
 	g_mutex_unlock (priv->lock);
 
 	return result;
@@ -862,6 +972,7 @@ brasero_io_get_file_info_thread (BraseroAsyncTaskManager *manager,
 				  error,
 				  job->callback_data);
 
+	g_free (file_uri);
 	return BRASERO_ASYNC_TASK_FINISHED;
 }
 
@@ -1122,6 +1233,7 @@ typedef struct _BraseroIOCountData BraseroIOCountData;
 
 static void
 brasero_io_get_file_count_destroy (BraseroAsyncTaskManager *manager,
+				   gboolean cancelled,
 				   gpointer callback_data)
 {
 	BraseroIOCountData *data = callback_data;
@@ -1134,7 +1246,7 @@ brasero_io_get_file_count_destroy (BraseroAsyncTaskManager *manager,
 
 	brasero_io_job_progress_report_stop (BRASERO_IO (manager), callback_data);
 
-	brasero_io_job_free (callback_data);
+	brasero_io_job_free (BRASERO_IO (manager), cancelled, callback_data);
 }
 
 #ifdef BUILD_PLAYLIST
@@ -1273,6 +1385,11 @@ brasero_io_get_file_count_process_directory (BraseroIO *self,
 	while ((info = g_file_enumerator_next_file (enumerator, cancel, &error)) || error) {
 		GFile *child;
 
+		if (g_cancellable_is_cancelled (cancel)) {
+			g_object_unref (info);
+			break;
+		}
+
 		data->files_num ++;
 
 		if (error) {
@@ -1448,6 +1565,7 @@ typedef struct _BraseroIOContentsData BraseroIOContentsData;
 
 static void
 brasero_io_load_directory_destroy (BraseroAsyncTaskManager *manager,
+				   gboolean cancelled,
 				   gpointer callback_data)
 {
 	BraseroIOContentsData *data = callback_data;
@@ -1455,7 +1573,7 @@ brasero_io_load_directory_destroy (BraseroAsyncTaskManager *manager,
 	g_slist_foreach (data->children, (GFunc) g_object_unref, NULL);
 	g_slist_free (data->children);
 
-	brasero_io_job_free (BRASERO_IO_JOB (data));
+	brasero_io_job_free (BRASERO_IO (manager), cancelled, BRASERO_IO_JOB (data));
 }
 
 #ifdef BUILD_PLAYLIST
@@ -1698,17 +1816,6 @@ brasero_io_load_directory_thread (BraseroAsyncTaskManager *manager,
 		g_object_unref (child);
 	}
 
-	if (data->job.callback_data && data->job.callback_data->ref < 2) {
-		/* No result was returned so we need to return a dummy one to 
-		 * clean the callback_data in the main loop. */
-		brasero_io_return_result (BRASERO_IO (manager),
-					  data->job.base,
-					  NULL,
-					  NULL,
-					  NULL,
-					  data->job.callback_data);
-	}
-
 	g_file_enumerator_close (enumerator, NULL, NULL);
 	g_object_unref (enumerator);
 	g_object_unref (file);
@@ -1788,6 +1895,7 @@ brasero_io_xfer_pair_free (BraseroIOXferPair *pair)
 
 static void
 brasero_io_xfer_destroy (BraseroAsyncTaskManager *manager,
+			 gboolean cancelled,
 			 gpointer callback_data)
 {
 	BraseroIOXferData *data = callback_data;
@@ -1799,7 +1907,7 @@ brasero_io_xfer_destroy (BraseroAsyncTaskManager *manager,
 	g_mutex_free (data->current_lock);
 
 	/* no need to stop progress report as the following function will do it */
-	brasero_io_get_file_count_destroy (manager, callback_data);
+	brasero_io_get_file_count_destroy (manager, cancelled, callback_data);
 }
 
 static void
@@ -2304,8 +2412,7 @@ brasero_io_free_async_queue (BraseroAsyncTaskManager *manager,
 				    TRUE,
 				    job->callback_data);
 
-	brasero_io_job_free (job);
-
+	brasero_io_job_free (BRASERO_IO (manager), TRUE, job);
 	return TRUE;
 }
 
