@@ -54,6 +54,8 @@ struct _BraseroJacketEditPrivate
 	GtkWidget *underline;
 	GtkWidget *italic;
 	GtkWidget *bold;
+
+	GtkWidget *background;
 };
 
 #define BRASERO_JACKET_EDIT_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_JACKET_EDIT, BraseroJacketEditPrivate))
@@ -363,13 +365,15 @@ brasero_jacket_edit_font_changed_cb (BraseroJacketFont *button,
 	if (!priv->current_view)
 		return;
 
+	buffer = brasero_jacket_view_get_active_buffer (BRASERO_JACKET_VIEW (priv->current_view));
+	if (!buffer)
+		return;
+
 	font_name = brasero_jacket_font_get_name (button);
 	if (!font_name)
 		return;
 
 	desc = pango_font_description_from_string (font_name);
-
-	buffer = brasero_jacket_view_get_active_buffer (BRASERO_JACKET_VIEW (priv->current_view));
 	tag = gtk_text_buffer_create_tag (buffer, NULL,
 					  "font-desc", desc,
 					  NULL);
@@ -382,6 +386,17 @@ brasero_jacket_edit_font_changed_cb (BraseroJacketFont *button,
 		brasero_jacket_buffer_add_default_tag (BRASERO_JACKET_BUFFER (buffer), tag);
 }
 
+static void
+brasero_jacket_edit_configure_background_pressed_cb (GtkToolButton *button,
+						     BraseroJacketEdit *self)
+{
+	BraseroJacketEditPrivate *priv;
+
+	priv = BRASERO_JACKET_EDIT_PRIVATE (self);
+	if (priv->current_view)
+		brasero_jacket_view_configure_background (BRASERO_JACKET_VIEW (priv->current_view));
+}
+
 static void
 brasero_jacket_edit_update_button_state (BraseroJacketEdit *self)
 {
@@ -394,10 +409,14 @@ brasero_jacket_edit_update_button_state (BraseroJacketEdit *self)
 
 	priv = BRASERO_JACKET_EDIT_PRIVATE (self);
 
-	if (priv->current_view)
+	if (priv->current_view) {
 		buffer = brasero_jacket_view_get_active_buffer (BRASERO_JACKET_VIEW (priv->current_view));
-	else
+		gtk_widget_set_sensitive (priv->background, TRUE);
+	}
+	else {
 		buffer = NULL;
+		gtk_widget_set_sensitive (priv->background, FALSE);
+	}
 
 	if (!buffer)
 		return;
@@ -520,6 +539,7 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_widget_show (toolbar);
 	gtk_box_pack_start (GTK_BOX (object), toolbar, FALSE, TRUE, 0);
 
+	/* Items */
 	item = GTK_WIDGET (gtk_tool_button_new_from_stock (GTK_STOCK_PRINT));
 	gtk_widget_show (item);
 	g_signal_connect (item,
@@ -528,6 +548,24 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 			  object);
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
 
+	item = GTK_WIDGET (gtk_separator_tool_item_new ());
+	gtk_widget_show (item);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+
+	item = GTK_WIDGET (gtk_tool_button_new_from_stock (GTK_STOCK_SELECT_COLOR));
+	gtk_widget_show (item);
+	gtk_widget_set_sensitive (item, FALSE);
+	g_signal_connect (item,
+			  "clicked",
+			  G_CALLBACK (brasero_jacket_edit_configure_background_pressed_cb),
+			  object);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+	priv->background = item;
+
+	item = GTK_WIDGET (gtk_separator_tool_item_new ());
+	gtk_widget_show (item);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+
 	item = GTK_WIDGET (gtk_radio_tool_button_new_from_stock (NULL, GTK_STOCK_JUSTIFY_RIGHT));
 	gtk_widget_show (item);
 	g_signal_connect (item,
@@ -555,6 +593,10 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
 	priv->left = item;
 
+	item = GTK_WIDGET (gtk_separator_tool_item_new ());
+	gtk_widget_show (item);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+
 	item = GTK_WIDGET (gtk_toggle_tool_button_new_from_stock (GTK_STOCK_UNDERLINE));
 	gtk_widget_show (item);
 	g_signal_connect (item,
@@ -582,6 +624,10 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
 	priv->bold = item;
 
+	item = GTK_WIDGET (gtk_separator_tool_item_new ());
+	gtk_widget_show (item);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (item), 0);
+
 	priv->fonts = brasero_jacket_font_new ();
 	gtk_widget_show (priv->fonts);
 	g_signal_connect (priv->fonts,
@@ -622,7 +668,6 @@ brasero_jacket_edit_init (BraseroJacketEdit *object)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll), GTK_SHADOW_IN);
 	gtk_widget_show (scroll);
 	gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
 
