@@ -66,7 +66,6 @@ struct BraseroMetadataPrivate {
 
 	GstElement *snapshot;
 
-	GMainLoop *loop;
 	GError *error;
 	guint watch;
 
@@ -84,6 +83,9 @@ struct BraseroMetadataPrivate {
 	GMutex *mutex;
 	GCond *cond;
 
+	/* Used by threads */
+	GMutex *lock;
+
 	guint started:1;
 	guint moved_forward:1;
 	guint prev_level_mes:1;
@@ -174,6 +176,73 @@ brasero_metadata_info_free (BraseroMetadataInfo *info)
 	g_free (info);
 }
 
+static void
+brasero_metadata_destroy_pipeline (BraseroMetadata *self)
+{
+	GstState state;
+	GstStateChangeReturn change;
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	priv->started = 0;
+
+	if (!priv->pipeline)
+		return;
+
+	change = gst_element_set_state (GST_ELEMENT (priv->pipeline),
+					GST_STATE_NULL);
+
+	change = gst_element_get_state (priv->pipeline,
+					&state,
+					NULL,
+					GST_MSECOND);
+
+	/* better wait for the state change to be completed */
+	while (change == GST_STATE_CHANGE_ASYNC && state != GST_STATE_NULL) {
+		GstState pending;
+
+		change = gst_element_get_state (priv->pipeline,
+						&state,
+						&pending,
+						GST_MSECOND);
+		BRASERO_BURN_LOG ("Get state (current = %i pending = %i) returned %i",
+				  state, pending, change);
+	}
+
+	if (change == GST_STATE_CHANGE_FAILURE)
+		g_warning ("State change failure\n");
+
+	if (priv->audio) {
+		gst_bin_remove (GST_BIN (priv->pipeline), priv->audio);
+		priv->audio = NULL;
+	}
+
+	if (priv->video) {
+		gst_bin_remove (GST_BIN (priv->pipeline), priv->video);
+		priv->snapshot = NULL;
+		priv->video = NULL;
+	}
+
+	gst_object_unref (GST_OBJECT (priv->pipeline));
+	priv->pipeline = NULL;
+
+	if (priv->level) {
+		gst_object_unref (GST_OBJECT (priv->level));
+		priv->level = NULL;
+	}
+
+	if (priv->sink) {
+		gst_object_unref (GST_OBJECT (priv->sink));
+		priv->sink = NULL;
+	}
+
+	if (priv->convert) {
+		gst_object_unref (GST_OBJECT (priv->convert));
+		priv->convert = NULL;
+	}
+}
+
 static void
 brasero_metadata_stop (BraseroMetadata *self)
 {
@@ -181,8 +250,26 @@ brasero_metadata_stop (BraseroMetadata *self)
 
 	priv = BRASERO_METADATA_PRIVATE (self);
 
+	BRASERO_BURN_LOG ("Retrieval ended for %s %p",
+  			  priv->info ? priv->info->uri:"Unknown",
+			  self);
+
+	/* Destroy the pipeline as it has become unusable. */
 	if (priv->pipeline)
+		brasero_metadata_destroy_pipeline (self);
+/*	else if (priv->pipeline)
+	{	
+		GstState state = GST_STATE_NULL;
+
 		gst_element_set_state (priv->pipeline, GST_STATE_NULL);
+		gst_element_get_state (priv->pipeline, &state, NULL, GST_MSECOND);
+		while (state != GST_STATE_NULL)
+			gst_element_get_state (priv->pipeline,
+					       &state,
+					       NULL,
+					       GST_MSECOND);
+	}
+*/
 
 	if (priv->progress_id) {
 		g_source_remove (priv->progress_id);
@@ -220,12 +307,10 @@ brasero_metadata_stop (BraseroMetadata *self)
 
 	/* that's for sync_wait */
 	g_mutex_lock (priv->mutex);
-	g_cond_signal (priv->cond);
-	g_mutex_unlock (priv->mutex);
 
-	/* stop loop */
-	if (priv->loop && g_main_loop_is_running (priv->loop))
-		g_main_loop_quit (priv->loop);
+	/* use broadcast here as there could be more than one thread waiting */
+	g_cond_broadcast (priv->cond);
+	g_mutex_unlock (priv->mutex);
 }
 
 void
@@ -235,8 +320,11 @@ brasero_metadata_cancel (BraseroMetadata *self)
 
 	priv = BRASERO_METADATA_PRIVATE (self);
 
+	BRASERO_BURN_LOG ("Metadata retrieval cancelled for %s %p",
+			  priv->info ? priv->info->uri:"Unknown",
+			  self);
+	
 	brasero_metadata_stop (self);
-
 	if (priv->error) {
 		g_error_free (priv->error);
 		priv->error = NULL;
@@ -249,15 +337,14 @@ brasero_metadata_completed (BraseroMetadata *self)
 	BraseroMetadataPrivate *priv;
 
 	priv = BRASERO_METADATA_PRIVATE (self);
-	if ((!priv->loop || !g_main_loop_is_running (priv->loop)) && !priv->cond) {
-		/* we send a message only if we haven't got a loop (= async mode) */
-		g_object_ref (self);
-		g_signal_emit (G_OBJECT (self),
-			       brasero_metadata_signals [COMPLETED_SIGNAL],
-			       0,
-			       priv->error);
-		g_object_unref (self);
-	}
+
+	/* we send a message only if we haven't got a loop (= async mode) */
+	g_object_ref (self);
+	g_signal_emit (G_OBJECT (self),
+		       brasero_metadata_signals [COMPLETED_SIGNAL],
+		       0,
+		       priv->error);
+	g_object_unref (self);
 
 	brasero_metadata_stop (self);
 	return TRUE;
@@ -966,6 +1053,11 @@ brasero_metadata_create_audio_pipeline (BraseroMetadata *self)
 	/* set up the pipeline according to flags */
 	if (priv->flags & BRASERO_METADATA_FLAG_SILENCES) {
 		priv->prev_level_mes = 0;
+
+		/* Add a reference to these objects as we want to keep them
+		 * around after the bin they've been added to is destroyed
+		 * NOTE: now we destroy the pipeline every time which means
+		 * that it doesn't really matter. */
 		gst_object_ref (priv->convert);
 		gst_object_ref (priv->level);
 		gst_object_ref (priv->sink);
@@ -1156,7 +1248,6 @@ brasero_metadata_create_pipeline (BraseroMetadata *self)
 					   "Can't create audioconvert");
 		return FALSE;
 	}
-	gst_object_ref (priv->convert);
 
 	priv->level = gst_element_factory_make ("level", NULL);
 	if (!priv->level) {
@@ -1165,7 +1256,6 @@ brasero_metadata_create_pipeline (BraseroMetadata *self)
 					   "Can't create level");
 		return FALSE;
 	}
-	gst_object_ref (priv->level);
 	g_object_set (priv->level,
 		      "message", TRUE,
 		      "interval", (guint64) BRASERO_METADATA_SILENCE_INTERVAL,
@@ -1178,7 +1268,6 @@ brasero_metadata_create_pipeline (BraseroMetadata *self)
 					   "Can't create fake sink");
 		return FALSE;
 	}
-	gst_object_ref (priv->sink);
 
 	return TRUE;
 }
@@ -1192,7 +1281,12 @@ brasero_metadata_set_new_uri (BraseroMetadata *self,
 
 	priv = BRASERO_METADATA_PRIVATE (self);
 
-	BRASERO_BURN_LOG ("New retrieval for %s", uri);
+	BRASERO_BURN_LOG ("New retrieval for %s %p", uri, self);
+
+	if (priv->error) {
+		g_error_free (priv->error);
+		priv->error = NULL;
+	}
 
 	brasero_metadata_info_free (priv->info);
 	priv->info = NULL;
@@ -1279,6 +1373,7 @@ brasero_metadata_get_info_wait (BraseroMetadata *self,
 				BraseroMetadataFlag flags,
 				GError **error)
 {
+	GstStateChangeReturn state_change;
 	BraseroMetadataPrivate *priv;
 	gulong cancel_signal = 0;
 
@@ -1297,16 +1392,10 @@ brasero_metadata_get_info_wait (BraseroMetadata *self,
 
 	g_mutex_lock (priv->mutex);
 
-	cancel_signal = g_signal_connect (cancel,
-					  "cancelled",
-					  G_CALLBACK (brasero_metadata_cancelled_cb),
-					  self);
-
 	/* Now wait ... but check a last time otherwise we wouldn't get the
 	 * any notice of cancellation if it had been cancelled before we 
 	 * connected to the signal */
 	if (g_cancellable_is_cancelled (cancel)) {
-		g_signal_handler_disconnect (cancel, cancel_signal);
 		g_mutex_unlock (priv->mutex);
 
 		brasero_metadata_stop (self);
@@ -1316,60 +1405,29 @@ brasero_metadata_get_info_wait (BraseroMetadata *self,
 		return FALSE;
 	}
 
+	cancel_signal = g_signal_connect (cancel,
+					  "cancelled",
+					  G_CALLBACK (brasero_metadata_cancelled_cb),
+					  self);
+
 	priv->started = 1;
-	gst_element_set_state (GST_ELEMENT (priv->pipeline), BRASERO_METADATA_INITIAL_STATE);
-	g_cond_wait (priv->cond, priv->mutex);
-	g_mutex_unlock (priv->mutex);
+	state_change = gst_element_set_state (GST_ELEMENT (priv->pipeline),
+					      BRASERO_METADATA_INITIAL_STATE);
+
+	if (state_change != GST_STATE_CHANGE_FAILURE) {
+		/* no need to wait for a condition if it failed */
+		g_cond_wait (priv->cond, priv->mutex);
+		g_mutex_unlock (priv->mutex);
+	}
+	else {
+		g_mutex_unlock (priv->mutex);
+		brasero_metadata_stop (self);
+	}
 
 	g_signal_handler_disconnect (cancel, cancel_signal);
 
-	if (priv->error) {
-		if (error) {
-			g_propagate_error (error, priv->error);
-			priv->error = NULL;
-		} 
-		else {
-			BRASERO_BURN_LOG ("ERROR getting metadata : %s\n", priv->error->message);
-			g_error_free (priv->error);
-			priv->error = NULL;
-		}
-
-		return FALSE;
-	}
-
-	return (g_cancellable_is_cancelled (cancel) == FALSE);
-}
-
-gboolean
-brasero_metadata_get_info_sync (BraseroMetadata *self,
-				const gchar *uri,
-				BraseroMetadataFlag flags,
-				GError **error)
-{
-	BraseroMetadataPrivate *priv;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	priv->flags = flags;
-
-	if (!brasero_metadata_set_new_uri (self, uri)) {
-		g_propagate_error (error, priv->error);
-		priv->error = NULL;
-
-		brasero_metadata_info_free (priv->info);
-		priv->info = NULL;
-
-		return FALSE;
-	}
-
-	priv->started = 1;
-	gst_element_set_state (GST_ELEMENT (priv->pipeline), BRASERO_METADATA_INITIAL_STATE);
-
-	/* run the loop until it's finished */
-	priv->loop = g_main_loop_new (NULL, FALSE);
-	g_main_loop_run (priv->loop);
-	g_main_loop_unref (priv->loop);
-	priv->loop = NULL;
+	if (priv->pipeline)
+		gst_element_set_state (GST_ELEMENT (priv->pipeline), GST_STATE_NULL);
 
 	if (priv->error) {
 		if (error) {
@@ -1385,7 +1443,8 @@ brasero_metadata_get_info_sync (BraseroMetadata *self,
 		return FALSE;
 	}
 
-	return TRUE;
+	return (g_cancellable_is_cancelled (cancel) == FALSE) &&
+	       (state_change != GST_STATE_CHANGE_FAILURE);
 }
 
 gboolean
@@ -1394,6 +1453,7 @@ brasero_metadata_get_info_async (BraseroMetadata *self,
 				 BraseroMetadataFlag flags)
 {
 	BraseroMetadataPrivate *priv;
+	GstStateChangeReturn state_change;
 
 	priv = BRASERO_METADATA_PRIVATE (self);
 
@@ -1414,10 +1474,56 @@ brasero_metadata_get_info_async (BraseroMetadata *self,
 		return FALSE;
 	}
 
-	priv->started = 1;
-	gst_element_set_state (GST_ELEMENT (priv->pipeline), BRASERO_METADATA_INITIAL_STATE);
+	state_change = gst_element_set_state (GST_ELEMENT (priv->pipeline),
+					      BRASERO_METADATA_INITIAL_STATE);
 
-	return TRUE;
+	priv->started = (state_change != GST_STATE_CHANGE_FAILURE);
+	return priv->started;
+}
+
+void
+brasero_metadata_lock (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+	g_mutex_lock (priv->lock);
+}
+
+void
+brasero_metadata_unlock (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+	g_mutex_unlock (priv->lock);
+}
+
+gboolean
+brasero_metadata_try_lock (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+	return g_mutex_trylock (priv->lock);
+}
+
+const gchar *
+brasero_metadata_get_uri (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+	return priv->info?priv->info->uri:NULL;
+}
+
+BraseroMetadataFlag
+brasero_metadata_get_flags (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+	return priv->flags;
 }
 
 void
@@ -1500,79 +1606,7 @@ brasero_metadata_init (BraseroMetadata *obj)
 
 	priv->cond = g_cond_new ();
 	priv->mutex = g_mutex_new ();
-}
-
-static void
-brasero_metadata_destroy_pipeline (BraseroMetadata *self)
-{
-	GstStateChangeReturn change;
-	BraseroMetadataPrivate *priv;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	priv->started = 0;
-
-	if (!priv->pipeline)
-		return;
-
-	/* better to wait for the state change to be completed */
-	change = gst_element_set_state (GST_ELEMENT (priv->pipeline),
-					GST_STATE_READY);
-
-	while (change == GST_STATE_CHANGE_ASYNC) {
-		GstState state;
-		GstState pending;
-
-		change = gst_element_get_state (priv->pipeline,
-						&state,
-						&pending,
-						GST_MSECOND);
-	};
-	
-	change = gst_element_set_state (GST_ELEMENT (priv->pipeline),
-					GST_STATE_NULL);
-
-	while (change == GST_STATE_CHANGE_ASYNC) {
-		GstState state;
-		GstState pending;
-
-		change = gst_element_get_state (priv->pipeline,
-						&state,
-						&pending,
-						GST_MSECOND);
-	}
-
-	if (change == GST_STATE_CHANGE_FAILURE)
-		g_warning ("State change failure\n");
-
-	if (priv->audio) {
-		gst_bin_remove (GST_BIN (priv->pipeline), priv->audio);
-		priv->audio = NULL;
-	}
-
-	if (priv->video) {
-		gst_bin_remove (GST_BIN (priv->pipeline), priv->video);
-		priv->snapshot = NULL;
-		priv->video = NULL;
-	}
-
-	gst_object_unref (GST_OBJECT (priv->pipeline));
-	priv->pipeline = NULL;
-
-	if (priv->level) {
-		gst_object_unref (GST_OBJECT (priv->level));
-		priv->level = NULL;
-	}
-
-	if (priv->sink) {
-		gst_object_unref (GST_OBJECT (priv->sink));
-		priv->sink = NULL;
-	}
-
-	if (priv->convert) {
-		gst_object_unref (GST_OBJECT (priv->convert));
-		priv->convert = NULL;
-	}
+	priv->lock = g_mutex_new ();
 }
 
 static void
@@ -1604,14 +1638,6 @@ brasero_metadata_finalize (GObject *object)
 		priv->watch = 0;
 	}
 
-	if (priv->loop) {
-		if (g_main_loop_is_running (priv->loop))
-			g_main_loop_quit (priv->loop);
-
-		g_main_loop_unref (priv->loop);
-		priv->loop = NULL;
-	}
-
 	if (priv->info) {
 		brasero_metadata_info_free (priv->info);
 		priv->info = NULL;
@@ -1627,6 +1653,11 @@ brasero_metadata_finalize (GObject *object)
 		priv->cond = NULL;
 	}
 
+	if (priv->lock) {
+		g_mutex_free (priv->lock);
+		priv->lock = NULL;
+	}
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
