@@ -45,6 +45,7 @@
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtkscrolledwindow.h>
 
+#include "brasero-app.h"
 #include "brasero-utils.h"
 #include "brasero-project.h"
 #include "brasero-layout.h"
@@ -57,6 +58,7 @@
 #include "brasero-disc-copy-dialog.h"
 #include "brasero-io.h"
 #include "burn-caps.h"
+#include "burn-medium-monitor.h"
 
 #ifdef BUILD_SEARCH
 #include "brasero-search-beagle.h"
@@ -173,7 +175,9 @@ struct BraseroProjectManagerPrivate {
 
 	GtkWidget *project;
 	GtkWidget *layout;
-	GtkWidget *status;
+
+	gchar **selected;
+	guint preview_id;
 
 	guint status_ctx;
 
@@ -237,10 +241,18 @@ brasero_project_manager_set_statusbar (BraseroProjectManager *manager,
 				       gint files_num)
 {
 	gchar *status_string = NULL;
+	GtkWidget *toplevel;
+	GtkWidget *status;
 	gint valid_num;
 
-	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
-			   manager->priv->status_ctx);
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+	status = brasero_app_get_statusbar1 (BRASERO_APP (toplevel));
+
+	if (!manager->priv->status_ctx)
+		manager->priv->status_ctx = gtk_statusbar_get_context_id (GTK_STATUSBAR (status),
+									  "size_info");
+
+	gtk_statusbar_pop (GTK_STATUSBAR (status), manager->priv->status_ctx);
 
 	valid_num = files_num - invalid_num;
 	if (!invalid_num && valid_num) {
@@ -293,8 +305,7 @@ brasero_project_manager_set_statusbar (BraseroProjectManager *manager,
 	else
 		status_string = g_strdup (_("No file selected"));
 
-	gtk_statusbar_push (GTK_STATUSBAR (manager->priv->status),
-			    manager->priv->status_ctx,
+	gtk_statusbar_push (GTK_STATUSBAR (status), manager->priv->status_ctx,
 			    status_string);
 	g_free (status_string);
 }
@@ -337,29 +348,13 @@ brasero_project_manager_size_preview_progress (GObject *object,
 					       files_num);
 }
 
-void
-brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
-					       BraseroProjectManager *manager)
+static gboolean
+brasero_project_manager_selected_uris_preview (gpointer data)
 {
+	BraseroProjectManager *manager = BRASERO_PROJECT_MANAGER (data);
 	BraseroIOFlags flags;
-    	gchar **uris, **iter;
-    	GSList *list = NULL;
-
-	/* if we are in the middle of an unfinished size seek then
-	 * cancel it and re-initialize */
-	if (manager->priv->io)
-		brasero_io_cancel_by_base (manager->priv->io,
-					   manager->priv->size_preview);
-
-	uris = brasero_uri_container_get_selected_uris (container);
-	if (!uris) {
-		gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
-				   manager->priv->status_ctx);
-		gtk_statusbar_push (GTK_STATUSBAR (manager->priv->status),
-				    manager->priv->status_ctx,
-				    _("No file selected"));
-		return;
-	}
+	GSList *list = NULL;
+	gchar **iter;
 
 	if (!manager->priv->io)
 		manager->priv->io = brasero_io_get_default ();
@@ -370,7 +365,7 @@ brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
 								   NULL,
 								   brasero_project_manager_size_preview_progress);
     
-	for (iter = uris; iter && *iter; iter ++)
+	for (iter = manager->priv->selected; iter && *iter; iter ++)
 		list = g_slist_prepend (list, *iter);
 
 	flags = BRASERO_IO_INFO_RECURSIVE|BRASERO_IO_INFO_IDLE;
@@ -384,15 +379,126 @@ brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
 				   NULL);
 			       
 	g_slist_free (list);
-	g_strfreev (uris);
+	manager->priv->preview_id = 0;
+	return FALSE;
 }
+
 void
-brasero_project_manager_set_status (BraseroProjectManager *manager,
-				    GtkWidget *status)
+brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
+					       BraseroProjectManager *manager)
 {
-	manager->priv->status = status;
-	manager->priv->status_ctx = gtk_statusbar_get_context_id (GTK_STATUSBAR (status),
-								  "size_info");
+	gchar **uris;
+
+	/* Before cancelling everything, check that the size really changed
+	 * like in the case of double clicking or if the user selected one
+	 * file and double clicked on it afterwards.
+	 * NOTE: the following expects each URI to be unique. */
+	uris = brasero_uri_container_get_selected_uris (container);
+	if (uris) {
+		gchar **iter;
+		guint num = 0;
+		gboolean found = FALSE;
+
+		for (iter = manager->priv->selected; iter && *iter; iter ++) {
+			gchar **uri;
+
+			found = FALSE;
+			for (uri = uris; uri && *uri; uri ++) {
+				if (!strcmp (*uri, *iter)) {
+					found = TRUE;
+					break;
+				}
+			}
+
+			if (!found)
+				break;
+
+			num ++;
+		}
+
+		if (found) {
+			guint num_new = 0;
+
+			for (iter = uris; iter && *iter; iter ++)
+				num_new ++;
+
+			if (num_new == num) {
+				/* same uris no need to update anything. */
+				g_strfreev (uris);
+				return;
+			}
+		}
+	}
+
+	/* if we are in the middle of an unfinished size seek then
+	 * cancel it and re-initialize */
+	if (manager->priv->io)
+		brasero_io_cancel_by_base (manager->priv->io,
+					   manager->priv->size_preview);
+
+	if (manager->priv->selected) {
+		g_strfreev (manager->priv->selected);
+		manager->priv->selected = NULL;
+	}
+
+	if (manager->priv->preview_id) {
+		g_source_remove (manager->priv->preview_id);
+		manager->priv->preview_id = 0;
+	}
+
+	manager->priv->selected = uris;
+	if (!manager->priv->selected) {
+		GtkWidget *toplevel;
+ 		GtkWidget *status;
+ 
+ 		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+ 		status = brasero_app_get_statusbar1 (BRASERO_APP (toplevel));
+ 
+ 		if (!manager->priv->status_ctx)
+ 			manager->priv->status_ctx = gtk_statusbar_get_context_id (GTK_STATUSBAR (status),
+ 										  "size_info");
+ 
+ 		gtk_statusbar_pop (GTK_STATUSBAR (status), manager->priv->status_ctx);
+ 		gtk_statusbar_push (GTK_STATUSBAR (status),
+  				    manager->priv->status_ctx,
+  				    _("No file selected"));
+		return;
+	}
+
+	manager->priv->preview_id = g_timeout_add (500,
+						   brasero_project_manager_selected_uris_preview,
+						   manager);
+}
+
+void
+brasero_project_manager_sidepane_changed (BraseroLayout *layout,
+					  gboolean visible,
+					  BraseroProjectManager *manager)
+{
+	if (!visible) {
+		GtkWidget *toplevel;
+ 		GtkWidget *status;
+ 
+		/* If sidepane is disabled, remove any text about selection */
+		if (manager->priv->io)
+			brasero_io_cancel_by_base (manager->priv->io,
+						   manager->priv->size_preview);
+
+ 		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+ 		status = brasero_app_get_statusbar1 (BRASERO_APP (toplevel));
+
+ 		gtk_statusbar_pop (GTK_STATUSBAR (status), manager->priv->status_ctx);
+
+		if (manager->priv->selected) {
+			g_strfreev (manager->priv->selected);
+			manager->priv->selected = NULL;
+		}
+
+		if (manager->priv->preview_id) {
+			g_source_remove (manager->priv->preview_id);
+			manager->priv->preview_id = 0;
+		}
+	}
 }
 
 void
@@ -484,7 +590,8 @@ brasero_project_manager_burn_iso_dialog (BraseroProjectManager *manager,
 }
 
 static void
-brasero_project_manager_burn_disc (BraseroProjectManager *manager)
+brasero_project_manager_copy_disc (BraseroProjectManager *manager,
+				   const gchar *device)
 {
 	BraseroBurnSession *session;
 	GtkResponseType result;
@@ -499,6 +606,19 @@ brasero_project_manager_burn_disc (BraseroProjectManager *manager)
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_widget_show (dialog);
 
+	/* if a device is specified then get the corresponding medium */
+	if (device) {
+		BraseroDrive *drive;
+		BraseroMediumMonitor *monitor;
+
+		monitor = brasero_medium_monitor_get_default ();
+		drive = brasero_medium_monitor_get_drive (monitor, device);
+		g_object_unref (monitor);
+
+		brasero_disc_copy_dialog_set_drive (BRASERO_DISC_COPY_DIALOG (dialog), drive);
+		g_object_unref (drive);
+	}
+
 	result = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (result != GTK_RESPONSE_OK) {
 		gtk_widget_destroy (dialog);
@@ -528,10 +648,15 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 	&&  !brasero_project_confirm_switch (BRASERO_PROJECT (manager->priv->project)))
 		return;
 
-	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
-			   manager->priv->status_ctx);
-
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+
+	if (manager->priv->status_ctx) {
+		GtkWidget *status;
+
+		status = brasero_app_get_statusbar1 (BRASERO_APP (toplevel));
+		gtk_statusbar_pop (GTK_STATUSBAR (status), manager->priv->status_ctx);
+	}
+
 	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
 
 	manager->priv->type = type;
@@ -609,7 +734,7 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		if (toplevel)
 			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - Disc Copy"));
 
-		brasero_project_manager_burn_disc (manager);
+		brasero_project_manager_copy_disc (manager, uri);
 	}
 }
 
@@ -688,9 +813,10 @@ brasero_project_manager_video (BraseroProjectManager *manager, GSList *uris)
 }
 
 void
-brasero_project_manager_copy (BraseroProjectManager *manager)
+brasero_project_manager_copy (BraseroProjectManager *manager,
+			      const gchar *device)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_COPY, NULL, NULL, TRUE);
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_COPY, NULL, device, TRUE);
 }
 
 void
@@ -706,12 +832,11 @@ brasero_project_manager_open_project (BraseroProjectManager *manager,
 	BraseroProjectType type;
 	GtkAction *action;
 
-    	gtk_widget_show (manager->priv->layout);
+	gtk_widget_show (manager->priv->layout);
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
 	type = brasero_project_open_project (BRASERO_PROJECT (manager->priv->project), uri);
 
 	manager->priv->type = type;
-
     	if (type == BRASERO_PROJECT_TYPE_INVALID) {
 		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
 		return BRASERO_PROJECT_TYPE_INVALID;
@@ -761,7 +886,10 @@ brasero_project_manager_open_by_mime (BraseroProjectManager *manager,
 				      const gchar *uri,
 				      const gchar *mime)
 {
-	if (!strcmp (mime, "application/x-brasero"))
+	/* When our files/description of x-brasero mime type is not properly 
+	 * installed, it's returned as application/xml, so check that too. */
+	if (!strcmp (mime, "application/x-brasero")
+	||  !strcmp (mime, "application/xml"))
 		return brasero_project_manager_open_project (manager, uri);
 
 #ifdef BUILD_PLAYLIST
@@ -788,12 +916,11 @@ BraseroProjectType
 brasero_project_manager_open_uri (BraseroProjectManager *manager,
 				  const gchar *uri_arg)
 {
-  	GtkWidget *dialog;
-	GtkWidget *window;
 	gchar *uri;
 	GFile *file;
 	GFileInfo *info;
-	const gchar *mime;
+	GtkWidget *dialog;
+	GtkWidget *window;
 	BraseroProjectType type;
 
 	/* FIXME: make that asynchronous */
@@ -805,9 +932,10 @@ brasero_project_manager_open_uri (BraseroProjectManager *manager,
 				  NULL);
 
 	uri = g_file_get_uri (file);
-	mime = g_file_info_get_content_type (info);
-
 	if (g_file_query_exists (file, NULL)) {
+		const gchar *mime;
+
+		mime = g_file_info_get_content_type (info);
 	  	type = brasero_project_manager_open_by_mime (manager, uri, mime);
         } 
 	else {
@@ -968,6 +1096,11 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 	gtk_widget_show (obj->priv->layout);
 	gtk_notebook_append_page (GTK_NOTEBOOK (obj), obj->priv->layout, NULL);
 
+	g_signal_connect (obj->priv->layout,
+			  "sidepane",
+			  G_CALLBACK (brasero_project_manager_sidepane_changed),
+			  obj);
+
 	/* create the project for audio and data discs */
 	obj->priv->project = brasero_project_new ();
 
@@ -1070,6 +1203,16 @@ brasero_project_manager_finalize (GObject *object)
 		cobj->priv->io = NULL;
 	}
 
+	if (cobj->priv->preview_id) {
+		g_source_remove (cobj->priv->preview_id);
+		cobj->priv->preview_id = 0;
+	}
+
+	if (cobj->priv->selected) {
+		g_strfreev (cobj->priv->selected);
+		cobj->priv->selected = NULL;
+	}
+
 	g_free (cobj->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
