@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifndef BRASERO_PROJECT_SIZE_H
@@ -31,6 +33,7 @@
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkcontainer.h>
 
+#include "brasero-project-type-chooser.h"
 #include "burn-medium.h"
 
 G_BEGIN_DECLS
@@ -73,7 +76,7 @@ brasero_project_size_set_multisession (BraseroProjectSize *self,
 
 void
 brasero_project_size_set_context (BraseroProjectSize *self,
-				  gboolean is_audio);
+				  BraseroProjectType type);
 
 gboolean
 brasero_project_size_check_status (BraseroProjectSize *self,
