@@ -205,7 +205,7 @@ static GtkActionEntry entries [] = {
 	{"DeleteProject", GTK_STOCK_REMOVE, N_("_Remove Files"), NULL,
 	 N_("Remove the selected files from the project"), G_CALLBACK (brasero_project_remove_selected_uris_cb)},
 	{"DeleteAll", GTK_STOCK_CLEAR, N_("E_mpty Project"), NULL,
-	 N_("Delete all files from the project"), G_CALLBACK (brasero_project_empty_cb)},
+	 N_("Remove all files from the project"), G_CALLBACK (brasero_project_empty_cb)},
 	{"Burn", "media-optical-burn", N_("_Burn..."), NULL,
 	 N_("Burn the disc"), G_CALLBACK (brasero_project_burn_cb)},
 };
@@ -262,6 +262,8 @@ static GObjectClass *parent_class = NULL;
 
 #define BRASERO_PROJECT_VERSION "0.2"
 
+#define BRASERO_RESPONSE_ADD			1976
+
 GType
 brasero_project_get_type ()
 {
@@ -545,7 +547,7 @@ brasero_project_error_size_dialog (BraseroProject *project)
 	GtkWidget *message;
 
 	message = brasero_notify_message_add (BRASERO_NOTIFY (project->priv->message),
-					      _("Please, delete some files from the project."),
+					      _("Please, remove some files from the project."),
 					      _("The size of the project is too large for the disc even with the overburn option."),
 					      -1,
 					      BRASERO_NOTIFY_CONTEXT_SIZE);
@@ -571,7 +573,7 @@ brasero_project_overburn_dialog (BraseroProject *project)
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Project Size"));
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("The size of the project is too large for the disc and you must delete files otherwise."
+						  _("The size of the project is too large for the disc and you must remove files otherwise."
 						    "\nNOTE: This option might cause failure."));
 
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
@@ -941,13 +943,12 @@ brasero_project_check_default_burning_app (BraseroProject *project,
 					  GTK_BUTTONS_NONE,
 					  primary);
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Default Burning Application"));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
 						  _("This is a first time running dialog that won't be shown again. If you change your mind, you can change your choice later in the Removable Drives and Media Preferences whether or not you chose brasero."));
 
 	if (command)
 		/* NOTE for translators the %s is the old application name */
-		text = g_strdup_printf (_("_Keep Using \"%s\""), command);
+		text = g_strdup_printf (_("_Keep using \"%s\""), command);
 	else
 		text = g_strdup (GTK_STOCK_CANCEL);
 
@@ -1452,7 +1453,7 @@ brasero_project_file_chooser_response_cb (GtkWidget *chooser,
 	GSList *uris;
 	GSList *iter;
 
-	if (response != GTK_RESPONSE_OK) {
+	if (response != BRASERO_RESPONSE_ADD) {
 		gtk_widget_destroy (chooser);
 		return;
 	}
@@ -1506,7 +1507,6 @@ brasero_project_add_uris_cb (GtkAction *action, BraseroProject *project)
 							      GTK_WINDOW (toplevel),
 							      GTK_FILE_CHOOSER_ACTION_OPEN,
 							      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-							      GTK_STOCK_ADD, GTK_RESPONSE_OK,
 							      NULL);
 	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (project->priv->chooser), TRUE);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (project->priv->chooser), TRUE);
@@ -1514,6 +1514,16 @@ brasero_project_add_uris_cb (GtkAction *action, BraseroProject *project)
 	brasero_file_chooser_customize (project->priv->chooser, NULL);
 	gtk_widget_show (project->priv->chooser);
 
+	/* This is to work around a bug in GTK+ which doesn't want to add "Add"
+	 * button or anything that is not "Open" or "Cancel" buttons */
+	/* Just for the record, file chooser creation uses all GtkResponseType
+	 * that are already defined for internal use like GTK_RESPONSE_OK,
+	 * *_APPLY and so on (usually to open directories, not add them). So we
+	 * have to define on custom here. */
+	gtk_dialog_add_button (GTK_DIALOG (project->priv->chooser),
+			       GTK_STOCK_ADD,
+			       BRASERO_RESPONSE_ADD);
+
 	g_signal_connect (project->priv->chooser,
 			  "file-activated",
 			  G_CALLBACK (brasero_project_file_chooser_activated_cb),
@@ -1805,7 +1815,7 @@ _read_graft_point (xmlDocPtr project,
 			uri = xmlNodeListGetString (project,
 						    graft->xmlChildrenNode,
 						    1);
-			retval->uri = xmlURIUnescapeString ((char*) uri, 0, NULL);
+			retval->uri = g_uri_unescape_string ((char *)uri, NULL);
 			g_free (uri);
 			if (!retval->uri)
 				goto error;
@@ -1921,7 +1931,7 @@ _read_audio_track (xmlDocPtr project,
 				goto error;
 
 			song = g_new0 (BraseroDiscSong, 1);
-			song->uri = xmlURIUnescapeString ((gchar*) uri, 0, NULL);
+			song->uri = g_uri_unescape_string ((char *) uri, NULL);
 
 			/* to know if this info was set or not */
 			song->start = -1;
@@ -2138,7 +2148,6 @@ brasero_project_open_project (BraseroProject *project,
 	BraseroDiscTrack *track = NULL;
 	BraseroProjectType type;
 
-
 	if (!uri || *uri =='\0')
 		return BRASERO_PROJECT_TYPE_INVALID;
 
@@ -2380,7 +2389,7 @@ _save_audio_track_xml (xmlTextWriter *project,
 		gchar *end;
 
 		song = iter->data;
-		escaped = xmlURIEscapeStr ((xmlChar *) song->uri, NULL);
+		escaped = (unsigned char *) g_uri_escape_string (song->uri, NULL, FALSE);
 		success = xmlTextWriterWriteElement (project,
 						    (xmlChar *) "uri",
 						     escaped);
@@ -2450,7 +2459,7 @@ _save_data_track_xml (xmlTextWriter *project,
 		if (graft->uri) {
 			xmlChar *escaped;
 
-			escaped = xmlURIEscapeStr ((xmlChar *) graft->uri, NULL);
+			escaped = (unsigned char *) g_uri_escape_string (graft->uri, NULL, FALSE);
 			success = xmlTextWriterWriteElement (project, (xmlChar *) "uri", escaped);
 			g_free (escaped);
 			if (success < 0)
