@@ -7,12 +7,12 @@
  ****************************************************************************/
 
 /*
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
@@ -108,6 +108,7 @@ brasero_xfer (BraseroXferCtx *ctx,
 	/* download */
 	bzero (ctx, sizeof (BraseroXferCtx));
 
+	io = brasero_io_get_default ();
 	brasero_io_xfer (io, src_uri, dest_path, ctx->base, BRASERO_IO_INFO_NONE, ctx);
 
 	ctx->loop = g_main_loop_new (NULL, FALSE);
