@@ -2,17 +2,17 @@
  *            burn-debug.c
  *
  *  Sat Apr 14 10:53:08 2007
- *  Copyright  2007  algernon
- *  <algernon@localhost.localdomain>
+ *  Copyright  2007  Rouquier Philippe
+ *  <bonfire-app@wanadoo.fr>
  ****************************************************************************/
 
 /*
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
@@ -106,6 +106,8 @@ brasero_debug_burn_flags_to_string (gchar *buffer,
 		strcat (buffer, "no grace, ");
 	if (flags & BRASERO_BURN_FLAG_DAO)
 		strcat (buffer, "dao, ");
+	if (flags & BRASERO_BURN_FLAG_RAW)
+		strcat (buffer, "raw, ");
 	if (flags & BRASERO_BURN_FLAG_OVERBURN)
 		strcat (buffer, "overburn, ");
 	if (flags & BRASERO_BURN_FLAG_BURNPROOF)
@@ -268,10 +270,31 @@ brasero_debug_audio_format_to_string (gchar *buffer,
 		strcat (buffer, "RAW ");
 
 	if (format & BRASERO_AUDIO_FORMAT_UNDEFINED)
-		strcat (buffer, "UNDEFINED ");
+		strcat (buffer, "AUDIO UNDEFINED ");
 
 	if (format & BRASERO_AUDIO_FORMAT_4_CHANNEL)
 		strcat (buffer, "4 CHANNELS ");
+
+	if (format & BRASERO_AUDIO_FORMAT_MP2)
+		strcat (buffer, "MP2 ");
+
+	if (format & BRASERO_AUDIO_FORMAT_AC3)
+		strcat (buffer, "AC3 ");
+
+	if (format & BRASERO_AUDIO_FORMAT_44100)
+		strcat (buffer, "44100 ");
+
+	if (format & BRASERO_AUDIO_FORMAT_48000)
+		strcat (buffer, "48000 ");
+
+	if (format & BRASERO_VIDEO_FORMAT_UNDEFINED)
+		strcat (buffer, "VIDEO UNDEFINED ");
+
+	if (format & BRASERO_VIDEO_FORMAT_VCD)
+		strcat (buffer, "VCD ");
+
+	if (format & BRASERO_VIDEO_FORMAT_VCD)
+		strcat (buffer, "Video DVD ");
 }
 
 void
