@@ -52,14 +52,14 @@
 #include "burn-drive.h"
 
 const gchar *icons [] = { 	"iso-image-new",
-				"gnome-dev-cdrom",
-				"gnome-dev-disc-cdr",
-				"gnome-dev-disc-cdrw",
-				"gnome-dev-disc-dvdrom",
-				"gnome-dev-disc-dvdr",
-				"gnome-dev-disc-dvdrw",
-				"gnome-dev-disc-dvdr-plus",
-				"gnome-dev-disc-dvdram",
+				"media-optical-cd",
+				"media-optical-cd",
+				"media-optical-cd",
+				"media-optical-dvd",
+				"media-optical-dvd",
+				"media-optical-dvd",
+				"media-optical-dvd",
+				"media-optical-dvd",
 				NULL };
 const gchar *types [] = {	N_("file"),
 				N_("CDROM"),
@@ -79,6 +79,11 @@ const gchar *types [] = {	N_("file"),
 				N_("Rewritable Blu-ray disc"),
 				NULL };
 
+typedef enum {
+	BRASERO_MEDIUM_CAP_INVALID	= 0,
+	BRASERO_MEDIUM_CAP_TRUE		= 1,
+	BRASERO_MEDIUM_CAP_FALSE		= 2
+} BraseroMediumCapState;
 
 typedef struct _BraseroMediumPrivate BraseroMediumPrivate;
 struct _BraseroMediumPrivate
@@ -92,6 +97,8 @@ struct _BraseroMediumPrivate
 
 	gchar *udi;
 
+	gchar *id;
+
 	gint max_rd;
 	gint max_wrt;
 
@@ -101,9 +108,16 @@ struct _BraseroMediumPrivate
 	gint64 block_num;
 	gint64 block_size;
 
+	guint first_open_track;
 	guint64 next_wr_add;
+
 	BraseroMedia info;
 	BraseroDrive *drive;
+
+	/* Do we really need both? */
+	guint dummy_sao:2;
+	guint dummy_tao:2;
+	guint burnfree:2;
 };
 
 #define BRASERO_MEDIUM_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_MEDIUM, BraseroMediumPrivate))
@@ -159,6 +173,69 @@ brasero_medium_get_status (BraseroMedium *medium)
 	return priv->info;
 }
 
+BraseroBurnFlag
+brasero_medium_supported_flags (BraseroMedium *self,
+				BraseroBurnFlag flags)
+{
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	/* This is always FALSE */
+	if (priv->info & BRASERO_MEDIUM_PLUS)
+		flags &= ~BRASERO_BURN_FLAG_DUMMY;
+	/* Simulation is only possible according to write modes. This mode is
+	 * mostly used by cdrecord/wodim for CLONE images. */
+	else if (priv->info & BRASERO_MEDIUM_DVD) {
+		if (priv->dummy_sao != BRASERO_MEDIUM_CAP_TRUE)
+			flags &= ~BRASERO_BURN_FLAG_DUMMY;
+	}
+	else if (flags & BRASERO_BURN_FLAG_DAO) {
+		if (priv->dummy_sao != BRASERO_MEDIUM_CAP_TRUE)
+			flags &= ~BRASERO_BURN_FLAG_DUMMY;
+	}
+	else if (priv->dummy_tao != BRASERO_MEDIUM_CAP_TRUE)
+		flags &= ~BRASERO_BURN_FLAG_DUMMY;
+
+	if (!priv->burnfree)
+		flags &= ~BRASERO_BURN_FLAG_BURNPROOF;
+
+	return flags;
+}
+
+gboolean
+brasero_medium_support_flags (BraseroMedium *self,
+			      BraseroBurnFlag flags)
+{
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	if (flags & BRASERO_BURN_FLAG_DUMMY) {
+		/* This is always FALSE */
+		if (priv->info & BRASERO_MEDIUM_PLUS)
+			return FALSE;
+
+		if (priv->info & BRASERO_MEDIUM_DVD) {
+			if (priv->dummy_sao != BRASERO_MEDIUM_CAP_TRUE)
+				return FALSE;
+		}
+		else if (flags & BRASERO_BURN_FLAG_DAO) {
+			if (priv->dummy_sao != BRASERO_MEDIUM_CAP_TRUE)
+				return FALSE;
+		}
+		else if (priv->dummy_tao != BRASERO_MEDIUM_CAP_TRUE)
+			return FALSE;
+	}
+
+	if (flags & BRASERO_BURN_FLAG_BURNPROOF) {
+		if (!priv->burnfree)
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
 GSList *
 brasero_medium_get_tracks (BraseroMedium *medium)
 {
@@ -388,6 +465,9 @@ brasero_medium_get_write_speeds (BraseroMedium *medium)
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
+	if (!priv->wr_speeds)
+		return NULL;
+
 	while (priv->wr_speeds [max] != 0) max ++;
 
 	speeds = g_new0 (gint64, max + 1);
@@ -552,6 +632,201 @@ brasero_medium_get_capacity (BraseroMedium *medium,
 		brasero_medium_get_free_space (medium, size, blocks);
 }
 
+/**
+ * Test presence of simulate burning
+ */
+
+static BraseroBurnResult
+brasero_medium_test_simulate_CD_SAO (BraseroMedium *self,
+				     BraseroDeviceHandle *handle,
+				     BraseroScsiErrCode *code)
+{
+	BraseroScsiGetConfigHdr *hdr = NULL;
+	BraseroScsiCDTAODesc *tao_desc;
+	BraseroScsiFeatureDesc *desc;
+	BraseroMediumPrivate *priv;
+	BraseroScsiResult result;
+	int size;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	/* Try TAO and then SAO if it isn't persistent */
+	BRASERO_BURN_LOG ("Checking simulate (CD TAO)");
+	result = brasero_mmc2_get_configuration_feature (handle,
+							 BRASERO_SCSI_FEAT_WRT_TAO,
+							 &hdr,
+							 &size,
+							 code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("GET CONFIGURATION failed");
+		return BRASERO_BURN_ERR;
+	}
+
+	desc = hdr->desc;
+	if (!desc->current)
+		BRASERO_BURN_LOG ("Feature is not current");
+
+	tao_desc = (BraseroScsiCDTAODesc *) desc->data;
+	priv->dummy_tao = tao_desc->dummy ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	priv->burnfree = tao_desc->buf ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	g_free (hdr);
+	return BRASERO_BURN_OK;
+}
+
+static BraseroBurnResult
+brasero_medium_test_simulate_CD_TAO (BraseroMedium *self,
+				     BraseroDeviceHandle *handle,
+				     BraseroScsiErrCode *code)
+{
+	BraseroScsiGetConfigHdr *hdr = NULL;
+	BraseroScsiCDSAODesc *sao_desc;
+	BraseroScsiFeatureDesc *desc;
+	BraseroMediumPrivate *priv;
+	BraseroScsiResult result;
+	int size;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	BRASERO_BURN_LOG ("Checking simulate (CD SAO)");
+	result = brasero_mmc2_get_configuration_feature (handle,
+							 BRASERO_SCSI_FEAT_WRT_SAO_RAW,
+							 &hdr,
+							 &size,
+							 code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("GET CONFIGURATION failed");
+		return BRASERO_BURN_ERR;
+	}
+
+	desc = hdr->desc;
+	if (!desc->current)
+		BRASERO_BURN_LOG ("Feature is not current");
+
+	sao_desc = (BraseroScsiCDSAODesc *) desc->data;
+	priv->dummy_sao = sao_desc->dummy ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	priv->burnfree = sao_desc->buf ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	g_free (hdr);
+	return BRASERO_BURN_OK;
+}
+
+static BraseroBurnResult
+brasero_medium_test_simulate_DVDRW (BraseroMedium *self,
+				    BraseroDeviceHandle *handle,
+				    BraseroScsiErrCode *code)
+{
+	BraseroScsiDVDRWlessWrtDesc *less_wrt_desc;
+	BraseroScsiGetConfigHdr *hdr = NULL;
+	BraseroScsiFeatureDesc *desc;
+	BraseroMediumPrivate *priv;
+	BraseroScsiResult result;
+	int size;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	/* Only DVD-R(W) support simulation */
+	BRASERO_BURN_LOG ("Checking simulate (DVD-R/W)");
+	result = brasero_mmc2_get_configuration_feature (handle,
+							 BRASERO_SCSI_FEAT_WRT_DVD_LESS,
+							 &hdr,
+							 &size,
+							 code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("GET CONFIGURATION failed");
+		return BRASERO_BURN_ERR;
+	}
+
+	desc = hdr->desc;
+	if (!desc->current)
+		BRASERO_BURN_LOG ("Feature is not current");
+
+	less_wrt_desc = (BraseroScsiDVDRWlessWrtDesc *) desc->data;
+	priv->dummy_sao = less_wrt_desc->dummy ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	priv->dummy_tao = less_wrt_desc->dummy ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	priv->burnfree = less_wrt_desc->buf ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	g_free (hdr);
+	return BRASERO_BURN_OK;
+}
+
+/**
+ * This is a last resort when the initialization has failed.
+ */
+
+static void
+brasero_medium_test_simulate_2A (BraseroMedium *self,
+				 BraseroDeviceHandle *handle,
+				 BraseroScsiErrCode *code)
+{
+	BraseroScsiStatusPage *page_2A = NULL;
+	BraseroScsiModeData *data = NULL;
+	BraseroMediumPrivate *priv;
+	BraseroScsiResult result;
+	int size = 0;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	result = brasero_spc1_mode_sense_get_page (handle,
+						   BRASERO_SPC_PAGE_STATUS,
+						   &data,
+						   &size,
+						   code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("MODE SENSE failed");
+		return;
+	}
+
+	/* NOTE: this bit is only valid:
+	 * - for CDs when mode write is TAO or SAO
+	 * - for DVDs when mode write is incremental or SAO
+	 */
+
+	page_2A = (BraseroScsiStatusPage *) &data->page;
+	priv->dummy_sao = page_2A->dummy ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	priv->dummy_tao = page_2A->dummy ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	priv->burnfree = page_2A->buffer ? BRASERO_MEDIUM_CAP_TRUE:BRASERO_MEDIUM_CAP_FALSE;
+	g_free (data);
+}
+
+static void
+brasero_medium_init_caps (BraseroMedium *self,
+			  BraseroDeviceHandle *handle,
+			  BraseroScsiErrCode *code)
+{
+	BraseroMediumPrivate *priv;
+	BraseroScsiResult res;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	/* These special media don't support/need burnfree and simulation */
+	if (priv->info & BRASERO_MEDIUM_PLUS)
+		return;
+
+	if (priv->info & BRASERO_MEDIUM_CD) {
+		/* we have to do both */
+		res = brasero_medium_test_simulate_CD_SAO (self, handle, code);
+		if (res == BRASERO_SCSI_OK)
+			brasero_medium_test_simulate_CD_TAO (self, handle, code);
+	}
+	else
+		res = brasero_medium_test_simulate_DVDRW (self, handle, code);
+
+	BRASERO_BURN_LOG ("Tested simulation %d %d, burnfree %d",
+			  priv->dummy_tao,
+			  priv->dummy_sao,
+			  priv->burnfree);
+
+	if (res == BRASERO_SCSI_OK)
+		return;
+
+	/* it didn't work out as expected use fallback */
+	BRASERO_BURN_LOG ("Using fallback 2A page for testing simulation and burnfree");
+	brasero_medium_test_simulate_2A (self, handle, code);
+
+	BRASERO_BURN_LOG ("Re-tested simulation %d %d, burnfree %d",
+			  priv->dummy_tao,
+			  priv->dummy_sao,
+			  priv->burnfree);
+}
+
 /**
  * Function to retrieve the capacity of a media
  */
@@ -871,6 +1146,12 @@ brasero_medium_get_page_2A_max_speed (BraseroMedium *self,
 	priv->max_rd = BRASERO_GET_16 (page_2A->rd_max_speed);
 	priv->max_wrt = BRASERO_GET_16 (page_2A->wr_max_speed);
 
+	/* also add it to the speed descriptors */
+	priv->wr_speeds = g_new0 (gint, 2);
+	priv->wr_speeds [0] = BRASERO_GET_16 (page_2A->wr_max_speed);
+	priv->rd_speeds = g_new0 (gint, 2);
+	priv->rd_speeds [0] = BRASERO_GET_16 (page_2A->rd_max_speed);
+
 	g_free (data);
 	return BRASERO_BURN_OK;
 }
@@ -918,6 +1199,7 @@ brasero_medium_track_volume_size (BraseroMedium *self,
 
 		if (error)
 			g_error_free (error);
+
 		return BRASERO_BURN_ERR;
 	}
 
@@ -1109,9 +1391,10 @@ brasero_medium_track_get_info (BraseroMedium *self,
 }
 
 static BraseroBurnResult
-brasero_medium_track_get_nwa (BraseroMedium *self,
-			      BraseroDeviceHandle *handle,
-			      BraseroScsiErrCode *code)
+brasero_medium_track_set_leadout (BraseroMedium *self,
+				  BraseroDeviceHandle *handle,
+				  BraseroMediumTrack *leadout,
+				  BraseroScsiErrCode *code)
 {
 	BraseroScsiModeData *data = NULL;
 	BraseroScsiTrackInfo track_info;
@@ -1121,7 +1404,7 @@ brasero_medium_track_get_nwa (BraseroMedium *self,
 	gint track_num;
 	int size;
 
-	BRASERO_BURN_LOG ("Retrieving NWA");
+	BRASERO_BURN_LOG ("Retrieving NWA and leadout information");
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
@@ -1172,8 +1455,12 @@ brasero_medium_track_get_nwa (BraseroMedium *self,
 	/* The following includes DL */
 	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDR_PLUS)) 
 		track_num = 0xFF;
-	else
-		track_num = g_slist_length (priv->tracks);
+	else if (priv->first_open_track >= 0)
+		track_num = priv->first_open_track;
+	else {
+		BRASERO_BURN_LOG ("There aren't any open session set");
+		return BRASERO_BURN_ERR;
+	}
 
 	result = brasero_mmc1_read_track_info (handle,
 					       track_num,
@@ -1186,12 +1473,24 @@ brasero_medium_track_get_nwa (BraseroMedium *self,
 		return BRASERO_BURN_ERR;
 	}
 
-	if (track_info.next_wrt_address_valid) {
+	BRASERO_BURN_LOG ("Next Writable Address is %d", BRASERO_GET_32 (track_info.next_wrt_address));
+	if (track_info.next_wrt_address_valid)
 		priv->next_wr_add = BRASERO_GET_32 (track_info.next_wrt_address);
-		BRASERO_BURN_LOG ("Next Writable Address is %d", priv->next_wr_add);
-	}
 	else
-		BRASERO_BURN_LOG ("No Next Writable Address");
+		BRASERO_BURN_LOG ("Next Writable Address is not valid");
+
+	/* Set free space */
+	BRASERO_BURN_LOG ("Free blocks %d", BRASERO_GET_32 (track_info.free_blocks));
+	leadout->blocks_num = BRASERO_GET_32 (track_info.free_blocks);
+
+	if (!leadout->blocks_num) {
+		leadout->blocks_num = BRASERO_GET_32 (track_info.track_size);
+		BRASERO_BURN_LOG ("Using track size %d", leadout->blocks_num);
+	}
+
+	BRASERO_BURN_LOG ("Leadout: start = %llu size = %llu",
+			  leadout->start,
+			  leadout->blocks_num);
 
 	return BRASERO_BURN_OK;
 }
@@ -1276,16 +1575,23 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 	       sizeof (BraseroScsiTocDesc);
 
 	/* remove 1 for leadout */
-	multisession = (priv->info & BRASERO_MEDIUM_APPENDABLE) || (num -1) != 1;
+	multisession = !(priv->info & BRASERO_MEDIUM_BLANK) && num > 0;
 
+	/* NOTE: in the case of DVD- there are always only 3 sessions if they
+	 * are open: all first concatenated sessions, the last session, and the
+	 * leadout. */
+	
 	BRASERO_BURN_LOG ("%i track(s) found", num);
 
 	desc = toc->desc;
 	for (i = 0; i < num; i ++, desc ++) {
 		BraseroMediumTrack *track;
 
-		if (desc->track_num == BRASERO_SCSI_TRACK_LEADOUT_START)
+		if (desc->track_num == BRASERO_SCSI_TRACK_LEADOUT_START) {
+			BRASERO_BURN_LOG ("Leadout reached %d",
+					  BRASERO_GET_32 (desc->track_start));
 			break;
+		}
 
 		track = g_new0 (BraseroMediumTrack, 1);
 		priv->tracks = g_slist_prepend (priv->tracks, track);
@@ -1321,8 +1627,9 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 			 * which have only one track: the first. Since it's not
 			 * possible to know the amount of data that were really
 			 * written in this session, read the filesystem. */
-			BRASERO_BURN_LOG ("DVD+RW (DL) or DVD-RW (restricted overwrite) checking volume size");
+			BRASERO_BURN_LOG ("DVD+RW (DL) or DVD-RW (restricted overwrite) checking volume size (start = %i)", track->start);
 			track->session = 1;
+			track->start = 0;
 			result = brasero_medium_track_volume_size (self, 
 								   track,
 								   handle);
@@ -1364,23 +1671,19 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
 		brasero_medium_add_DVD_plus_RW_leadout (self);
 	else if (!(priv->info & BRASERO_MEDIUM_CLOSED)) {
-		BraseroMediumTrack *track;
+		BraseroMediumTrack *leadout;
 
 		/* we shouldn't request info on leadout if the disc is closed
 		 * (except for DVD+/- (restricted) RW (see above) */
-		track = g_new0 (BraseroMediumTrack, 1);
-		priv->tracks = g_slist_append (priv->tracks, track);
-		track->start = BRASERO_GET_32 (desc->track_start);
-		track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
+		leadout = g_new0 (BraseroMediumTrack, 1);
+		leadout->start = BRASERO_GET_32 (desc->track_start);
+		leadout->type = BRASERO_MEDIUM_TRACK_LEADOUT;
+		priv->tracks = g_slist_append (priv->tracks, leadout);
 
-		brasero_medium_track_get_info (self,
-					       FALSE,
-					       track,
-					       g_slist_length (priv->tracks),
-					       handle,
-					       code);
-
-		brasero_medium_track_get_nwa (self, handle, code);
+		brasero_medium_track_set_leadout (self,
+						  handle,
+						  leadout,
+						  code);
 	}
 
 	g_free (toc);
@@ -1388,6 +1691,34 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 	return BRASERO_BURN_OK;
 }
 
+static void
+brasero_medium_get_DVD_id (BraseroMedium *self,
+			   BraseroDeviceHandle *handle,
+			   BraseroScsiErrCode *code)
+{
+	gint size = 0;
+	BraseroScsiResult result;
+	BraseroMediumPrivate *priv;
+	BraseroScsiReadDiscStructureHdr *hdr = NULL;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	/* This should be only possible for DVD-R(W) and not with all drives */
+	result = brasero_mmc2_read_generic_structure (handle,
+						      BRASERO_SCSI_FORMAT_LESS_MEDIA_ID_DVD,
+						      &hdr,
+						      &size,
+						      code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("Retrieval of DVD id failed");
+		return;
+	}
+
+	BRASERO_BURN_LOG ("DVD id %d", BRASERO_GET_16 (hdr->data + 2));
+	priv->id = g_strdup_printf ("%d", BRASERO_GET_16 (hdr->data + 2));
+	g_free (hdr);
+}
+
 static BraseroBurnResult
 brasero_medium_get_contents (BraseroMedium *self,
 			     BraseroDeviceHandle *handle,
@@ -1413,9 +1744,19 @@ brasero_medium_get_contents (BraseroMedium *self,
 		return BRASERO_BURN_ERR;
 	}
 
+	if (info->disc_id_valid) {
+		/* Try to get the disc identification if possible (CDs only) */
+		BRASERO_BURN_LOG ("Disc id %i", BRASERO_GET_32 (info->disc_id));
+		priv->id = g_strdup_printf ("%d", BRASERO_GET_32 (info->disc_id));
+	}
+	else if (!(priv->info & BRASERO_MEDIUM_CD))
+		brasero_medium_get_DVD_id (self, handle, code);
+
 	if (info->erasable)
 		priv->info |= BRASERO_MEDIUM_REWRITABLE;
 
+	priv->first_open_track = -1;
+
 	if (info->status == BRASERO_SCSI_DISC_EMPTY) {
 		BraseroMediumTrack *track;
 
@@ -1424,6 +1765,8 @@ brasero_medium_get_contents (BraseroMedium *self,
 		priv->info |= BRASERO_MEDIUM_BLANK;
 		priv->block_size = 2048;
 
+		priv->first_open_track = BRASERO_FIRST_TRACK_IN_LAST_SESSION (info);
+
 		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
 		||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
 			brasero_medium_add_DVD_plus_RW_leadout (self);
@@ -1433,19 +1776,21 @@ brasero_medium_get_contents (BraseroMedium *self,
 			track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
 			priv->tracks = g_slist_prepend (priv->tracks, track);
 			
-			brasero_medium_track_get_info (self,
-						       FALSE,
-						       track,
-						       1,
-						       handle,
-						       code);
+			brasero_medium_track_set_leadout (self,
+							  handle,
+							  track,
+							  code);
 		}
+
 		goto end;
 	}
 
 	if (info->status == BRASERO_SCSI_DISC_INCOMPLETE) {
 		priv->info |= BRASERO_MEDIUM_APPENDABLE;
 		BRASERO_BURN_LOG ("Appendable media");
+
+		priv->first_open_track = BRASERO_FIRST_TRACK_IN_LAST_SESSION (info);
+		BRASERO_BURN_LOG ("First track in last open session %i", priv->first_open_track);
 	}
 	else if (info->status == BRASERO_SCSI_DISC_FINALIZED) {
 		priv->info |= BRASERO_MEDIUM_CLOSED;
@@ -2230,6 +2575,8 @@ brasero_medium_init_real (BraseroMedium *object,
 	if (BRASERO_MEDIUM_IS (priv->info, (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_ROM)))
 		brasero_medium_get_css_feature (object, handle, &code);
 
+	brasero_medium_init_caps (object, handle, &code);
+
 	BRASERO_BURN_LOG_DISC_TYPE (priv->info, "media is ");
 
 	if (!priv->wr_speeds)
@@ -2332,6 +2679,8 @@ brasero_medium_reload_info (BraseroMedium *self)
 {
 	BraseroMediumPrivate *priv;
 
+	g_return_if_fail (BRASERO_IS_MEDIUM (self));
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
 	priv->max_rd = 0;
@@ -2348,6 +2697,11 @@ brasero_medium_reload_info (BraseroMedium *self)
 		priv->retry_id = 0;
 	}
 
+	if (priv->id) {
+		g_free (priv->id);
+		priv->id = NULL;
+	}
+
 	g_free (priv->rd_speeds);
 	priv->rd_speeds = NULL;
 
@@ -2401,6 +2755,11 @@ brasero_medium_finalize (GObject *object)
 		priv->retry_id = 0;
 	}
 
+	if (priv->id) {
+		g_free (priv->id);
+		priv->id = NULL;
+	}
+
 	g_free (priv->rd_speeds);
 	priv->rd_speeds = NULL;
 
@@ -2598,6 +2957,15 @@ brasero_medium_get_udi (BraseroMedium *self)
 	return priv->udi;
 }
 
+const gchar *
+brasero_medium_get_id (BraseroMedium *self)
+{
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+	return priv->id;
+}
+
 GType
 brasero_medium_get_type (void)
 {
