@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -628,8 +626,10 @@ brasero_task_ctx_set_progress (BraseroTaskCtx *self,
 		return BRASERO_BURN_OK;
 	}
 
-	if (priv->timer)
-		elapsed = g_timer_elapsed (priv->timer, NULL);
+	if (!priv->timer)
+		return BRASERO_BURN_OK;
+
+	elapsed = g_timer_elapsed (priv->timer, NULL);
 
 	if ((elapsed - priv->last_elapsed) > 0.5) {
 		priv->last_progress = priv->progress;
