@@ -145,6 +145,12 @@ struct _BraseroTrackType {
 };
 typedef struct _BraseroTrackType BraseroTrackType;
 
+#define BRASERO_TRACK_TYPE_HAS_VIDEO(type_MACRO)				\
+	(((type_MACRO)->type == BRASERO_TRACK_TYPE_AUDIO) &&			\
+	 ((type_MACRO)->subtype.audio_format & (BRASERO_VIDEO_FORMAT_UNDEFINED|	\
+						BRASERO_VIDEO_FORMAT_VCD|	\
+						BRASERO_VIDEO_FORMAT_VIDEO_DVD)))
+
 gboolean
 brasero_track_type_equal (const BraseroTrackType *type_A,
 			  const BraseroTrackType *type_B);
@@ -317,6 +323,13 @@ brasero_track_tag_lookup (BraseroTrack *track,
 			  const gchar *tag,
 			  GValue **value);
 
+/*
+ * Commonly used Tags
+ */
+
+#define BRASERO_TRACK_MEDIUM_ADDRESS_START_TAG	"track::medium::address::start"
+#define BRASERO_TRACK_MEDIUM_ADDRESS_END_TAG	"track::medium::address::end"
+
 G_END_DECLS
 
 #endif /* _BURN_TRACK_H */
