@@ -100,7 +100,7 @@ brasero_volume_get_primary_from_file (BraseroVolSrc *vol,
 	BraseroVolDesc *desc;
 
 	/* skip the first 16 blocks */
-	if (!BRASERO_VOL_SRC_SEEK (vol, SYSTEM_AREA_SECTORS, SEEK_CUR, error))
+	if (BRASERO_VOL_SRC_SEEK (vol, SYSTEM_AREA_SECTORS, SEEK_CUR, error) == -1)
 		return FALSE;
 
 	if (!BRASERO_VOL_SRC_READ (vol, primary_vol, 1, error))
@@ -119,20 +119,13 @@ brasero_volume_get_primary_from_file (BraseroVolSrc *vol,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("there isn't a valid volume descriptor"));
+		BRASERO_BURN_LOG ("Wrong volume descriptor, got %.5s", desc->id);
 		return FALSE;
 	}
 
 	return TRUE;
 }
 
-gboolean
-brasero_volume_is_valid_fd (BraseroVolSrc *vol, GError **error)
-{
-	gchar buffer [ISO9660_BLOCK_SIZE];
-
-	return brasero_volume_get_primary_from_file (vol, buffer, error);
-}
-
 gboolean
 brasero_volume_get_size (BraseroVolSrc *vol,
 			 gint64 block,
@@ -142,7 +135,7 @@ brasero_volume_get_size (BraseroVolSrc *vol,
 	gboolean result;
 	gchar buffer [ISO9660_BLOCK_SIZE];
 
-	if (!BRASERO_VOL_SRC_SEEK (vol, block, SEEK_SET, error))
+	if (block && BRASERO_VOL_SRC_SEEK (vol, block, SEEK_SET, error) == -1)
 		return FALSE;
 
 	result = brasero_volume_get_primary_from_file (vol, buffer, error);
@@ -165,7 +158,7 @@ brasero_volume_get_files (BraseroVolSrc *vol,
 {
 	gchar buffer [ISO9660_BLOCK_SIZE];
 
-	if (!BRASERO_VOL_SRC_SEEK (vol, block, SEEK_SET, error))
+	if (BRASERO_VOL_SRC_SEEK (vol, block, SEEK_SET, error) == -1)
 		return FALSE;
 
 	if (!brasero_volume_get_primary_from_file (vol, buffer, error))
@@ -193,7 +186,7 @@ brasero_volume_get_file (BraseroVolSrc *vol,
 {
 	gchar buffer [ISO9660_BLOCK_SIZE];
 
-	if (!BRASERO_VOL_SRC_SEEK (vol, volume_start_block, SEEK_SET, error))
+	if (BRASERO_VOL_SRC_SEEK (vol, volume_start_block, SEEK_SET, error) == -1)
 		return NULL;
 
 	if (!brasero_volume_get_primary_from_file (vol, buffer, error))
