@@ -59,6 +59,7 @@ struct _BraseroBurnPrivate {
 	BraseroBurnSession *session;
 
 	GMainLoop *sleep_loop;
+	guint timeout_id;
 
 	guint tasks_done;
 	guint task_nb;
@@ -69,6 +70,9 @@ struct _BraseroBurnPrivate {
 
 	gint appcookie;
 
+	guint64 session_start;
+	guint64 session_end;
+
 	guint src_locked:1;
 	guint dest_locked:1;
 
@@ -125,7 +129,7 @@ static guint brasero_burn_signals [LAST_SIGNAL] = { 0 };
 #define BRASERO_BURN_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_BURN, BraseroBurnPrivate))
 
 #define MAX_EJECT_WAIT_TIME	20000
-#define MAX_MOUNT_ATTEMPTS	10
+#define MAX_MOUNT_ATTEMPTS	20
 #define MOUNT_TIMEOUT		500
 
 #define MAX_LOAD_ATTEMPTS	20
@@ -200,6 +204,7 @@ brasero_burn_wakeup (BraseroBurn *burn)
 	if (priv->sleep_loop)
 		g_main_loop_quit (priv->sleep_loop);
 
+	priv->timeout_id = 0;
 	return FALSE;
 }
 
@@ -207,16 +212,24 @@ static BraseroBurnResult
 brasero_burn_sleep (BraseroBurn *burn, gint msec)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
+	GMainLoop *loop;
 
 	priv->sleep_loop = g_main_loop_new (NULL, FALSE);
-	g_timeout_add (msec,
-		       (GSourceFunc) brasero_burn_wakeup,
-		       burn);
+	priv->timeout_id = g_timeout_add (msec,
+					  (GSourceFunc) brasero_burn_wakeup,
+					  burn);
 
-	g_main_loop_run (priv->sleep_loop);
+	/* Keep a reference to the loop in case we are cancelled to destroy it */
+	loop = priv->sleep_loop;
+	g_main_loop_run (loop);
 
+	if (priv->timeout_id) {
+		g_source_remove (priv->timeout_id);
+		priv->timeout_id = 0;
+	}
+
+	g_main_loop_unref (loop);
 	if (priv->sleep_loop) {
-		g_main_loop_unref (priv->sleep_loop);
 		priv->sleep_loop = NULL;
 		return BRASERO_BURN_OK;
 	}
@@ -709,6 +722,7 @@ brasero_burn_is_loaded_dest_media_supported (BraseroBurn *burn,
 			*must_blank = FALSE;
 		else
 			*must_blank = TRUE;
+
 		return BRASERO_BURN_ERROR_NONE;
 	}
 
@@ -740,7 +754,9 @@ brasero_burn_is_loaded_dest_media_supported (BraseroBurn *burn,
 }
 
 static BraseroBurnResult
-brasero_burn_lock_dest_media (BraseroBurn *burn, GError **error)
+brasero_burn_lock_dest_media (BraseroBurn *burn,
+			      BraseroBurnError *ret_error,
+			      GError **error)
 {
 	gchar *failure;
 	BraseroMedia media;
@@ -766,8 +782,6 @@ brasero_burn_lock_dest_media (BraseroBurn *burn, GError **error)
 
 	result = BRASERO_BURN_OK;
 
-again:
-
 	medium = brasero_drive_get_medium (priv->dest);
 	if (!medium) {
 		result = BRASERO_BURN_NEED_RELOAD;
@@ -796,7 +810,7 @@ again:
 	BRASERO_BURN_LOG_WITH_FULL_TYPE (BRASERO_TRACK_TYPE_DISC,
 					 media,
 					 BRASERO_PLUGIN_IO_NONE,
-					 "Waiting for dest drive");
+					 "Media inserted is");
 
 	if (priv->dest_locked) {
 		/* NOTE: after a blanking, for nautilus_burn the CD/DVD is still
@@ -828,9 +842,8 @@ again:
 	berror = brasero_burn_is_loaded_dest_media_supported (burn,
 							      media,
 							      &must_blank);
-
 	if (berror != BRASERO_BURN_ERROR_NONE) {
-		BRASERO_BURN_LOG ("the media is not supported");
+		BRASERO_BURN_LOG ("Inserted media is not supported");
 		result = BRASERO_BURN_NEED_RELOAD;
 		goto end;
 	}
@@ -888,8 +901,6 @@ again:
 				goto end;
 		}
 
-		/* FIXME: if NO_TMP_FILE is not set then the warning won't get
-		 * emitted */
 		if (input.type == BRASERO_TRACK_TYPE_DISC
 		&& (input.subtype.media & (BRASERO_MEDIUM_HAS_AUDIO|BRASERO_MEDIUM_HAS_DATA)) == BRASERO_MEDIUM_HAS_AUDIO) {
 			result = brasero_burn_emit_signal (burn, WARN_REWRITABLE_SIGNAL, BRASERO_BURN_CANCEL);
@@ -912,25 +923,14 @@ again:
 
 end:
 
-	if (result == BRASERO_BURN_NEED_RELOAD) {
-		BraseroMedia required_media;
-
-		required_media = brasero_burn_caps_get_required_media_type (priv->caps,
-									    priv->session);
-
-		result = brasero_burn_ask_for_dest_media (burn,
-							  berror,
-							  required_media,
-							  error);
-		if (result == BRASERO_BURN_OK)
-			goto again;
-	}
-
-	if (result != BRASERO_BURN_OK) {
+	if (result != BRASERO_BURN_OK && priv->dest_locked) {
 		priv->dest_locked = 0;
 		brasero_drive_unlock (priv->dest);
 	}
 
+	if (result == BRASERO_BURN_ERROR_RELOAD_MEDIA && ret_error)
+		*ret_error = berror;
+
 	return result;
 }
 
@@ -947,7 +947,13 @@ again:
 
 	/* eject and ask the user to reload a disc */
 	required_media = brasero_burn_caps_get_required_media_type (priv->caps, priv->session);
-	required_media &= (BRASERO_MEDIUM_WRITABLE|BRASERO_MEDIUM_CD|BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL);
+	required_media &= (BRASERO_MEDIUM_WRITABLE|
+			   BRASERO_MEDIUM_CD|
+			   BRASERO_MEDIUM_DVD|
+			   BRASERO_MEDIUM_DVD_DL);
+
+	if (required_media == BRASERO_MEDIUM_NONE)
+		required_media = BRASERO_MEDIUM_WRITABLE;
 
 	result = brasero_burn_ask_for_dest_media (burn,
 						  error_code,
@@ -956,7 +962,9 @@ again:
 	if (result != BRASERO_BURN_OK)
 		return result;
 
-	result = brasero_burn_lock_dest_media (burn, error);
+	result = brasero_burn_lock_dest_media (burn,
+					       &error_code,
+					       error);
 	if (result == BRASERO_BURN_NEED_RELOAD)
 		goto again;
 
@@ -1179,7 +1187,40 @@ brasero_burn_action_changed (BraseroTask *task,
 			     BraseroBurnAction action,
 			     BraseroBurn *burn)
 {
+	BraseroBurnPrivate *priv;
+	BraseroMedia media;
+
 	brasero_burn_action_changed_real (burn, action);
+
+	if (action != BRASERO_BURN_ACTION_START_RECORDING)
+		return;
+
+	priv = BRASERO_BURN_PRIVATE (burn);
+
+	media = brasero_burn_session_get_dest_media (priv->session);
+	if ((BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
+	||   BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED))) {
+		BraseroBurnFlag flags;
+		BraseroMedium *medium;
+		BraseroDrive *drive;
+		gint64 len = 0;
+
+		drive = brasero_burn_session_get_burner (priv->session);
+		medium = brasero_drive_get_medium (drive);
+		flags = brasero_burn_session_get_flags (priv->session);
+
+		/* we need to save some parameters for later checksuming */
+		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
+							  &len,
+							  NULL);
+
+		if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
+			priv->session_start = brasero_medium_get_next_writable_address (medium);
+		else
+			priv->session_start = 0;
+
+		priv->session_end = priv->session_start + len;
+	}
 }
 
 void
@@ -1213,15 +1254,21 @@ brasero_burn_status (BraseroBurn *burn,
 	g_return_val_if_fail (BRASERO_BURN (burn), BRASERO_BURN_ERR);
 	
 	priv = BRASERO_BURN_PRIVATE (burn);
-	if (!priv->task || !brasero_task_is_running (priv->task))
+
+	if (!priv->task)
+		return BRASERO_BURN_NOT_READY;
+
+	if (isosize)
+		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
+							  NULL,
+							  isosize);
+
+	if (!brasero_task_is_running (priv->task))
 		return BRASERO_BURN_NOT_READY;
 
 	if (rate)
 		brasero_task_ctx_get_rate (BRASERO_TASK_CTX (priv->task), rate);
 
-	if (isosize)
-		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task), NULL, isosize);
-
 	if (written)
 		brasero_task_ctx_get_written (BRASERO_TASK_CTX (priv->task), written);
 
@@ -1783,17 +1830,19 @@ static BraseroBurnResult
 brasero_burn_check_session_consistency (BraseroBurn *burn,
 					GError **error)
 {
+	BraseroMedia media;
 	BraseroBurnFlag flag;
 	BraseroTrackType type;
 	BraseroBurnFlag flags;
 	BraseroBurnFlag retval;
+	BraseroBurnResult result;
 	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
 	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 
 	BRASERO_BURN_DEBUG (burn, "Checking session consistency");
 
-	/* make sure there is a session, a burner */
+	/* make sure there is a track in the session. */
 	brasero_burn_session_get_input_type (priv->session, &type);
 	if (type.type == BRASERO_TRACK_TYPE_NONE
 	|| !brasero_burn_session_get_tracks (priv->session)) {
@@ -1805,7 +1854,7 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 		return BRASERO_BURN_ERR;
 	}
 
-	/* make sure there is a drive set as burner */
+	/* make sure there is a drive set as burner. */
 	if (!brasero_burn_session_is_dest_file (priv->session)) {
 		BraseroDrive *burner;
 
@@ -1825,79 +1874,69 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 			brasero_burn_session_set_num_copies (priv->session, 1);
 	}
 
+	media = brasero_burn_session_get_dest_media (priv->session);
+
 	/* save then wipe out flags from session to check them one by one */
 	flags = brasero_burn_session_get_flags (priv->session);
 	brasero_burn_session_remove_flag (priv->session, flags);
 
-	brasero_burn_caps_get_flags (priv->caps,
-				     priv->session,
-				     &supported,
-				     &compulsory);
+	result = brasero_burn_caps_get_flags (priv->caps,
+					      priv->session,
+					      &supported,
+					      &compulsory);
+	if (result != BRASERO_BURN_OK)
+		return result;
 
 	for (flag = 1; flag < BRASERO_BURN_FLAG_LAST; flag <<= 1) {
-		/* check each flag before re-adding it */
-		if ((flags & flag) && (supported & flag)) {
+		/* see if this flag was originally set */
+		if (!(flags & flag))
+			continue;
+
+		/* Check each flag before re-adding it. Emit warnings to user
+		 * to know if he wants to carry on for some flags when they are
+		 * not supported; namely DUMMY. Other flags trigger an error.
+		 * No need for BURNPROOF since that usually means it is just the
+		 * media type that doesn't need it. */
+		if (supported & flag) {
 			brasero_burn_session_add_flag (priv->session, flag);
 			brasero_burn_caps_get_flags (priv->caps,
 						     priv->session,
 						     &supported,
 						     &compulsory);
 		}
-		else if (flags & flag)
-			BRASERO_BURN_LOG_FLAGS (flag, "Flag set but not supported");
+		else {
+			BRASERO_BURN_LOG_FLAGS (flag, "Flag set but not supported:");
+
+			if (flag & BRASERO_BURN_FLAG_DUMMY) {
+				/* This is simply a warning that it's not possible */
+
+			}
+			else if (flag & BRASERO_BURN_FLAG_MERGE) {
+				/* we pay attention to one flag in particular
+				 * (MERGE) if it was set then it must be
+				 * supported. Otherwise error out. */
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("merging data is impossible with this disc"));
+				return BRASERO_BURN_ERR;
+			}
+			else if ((flag & BRASERO_BURN_FLAG_BURNPROOF)
+			     && !(BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
+			     ||   BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED))) {
+				 /* Warn the user BURNPROOF won't be possible */
+
+			}
+		}
 	}
 
 	retval = brasero_burn_session_get_flags (priv->session);
-
-	if ((flags & BRASERO_BURN_FLAG_MERGE)
-	&& !(retval & BRASERO_BURN_FLAG_MERGE)) {
-		/* we pay attention to one flag in particular (MERGE) if it was
-		 * set then it must be supported. Otherwise error out. */
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("merging data is impossible with this disc"));
-		return BRASERO_BURN_ERR;
-	}
-
 	if (retval != flags)
-		BRASERO_BURN_LOG_FLAGS (flags, "Some flags were not supported. Corrected to ");
+		BRASERO_BURN_LOG_FLAGS (retval, "Some flags were not supported. Corrected to");
 
 	if (retval != (retval | compulsory)) {
-		BRASERO_BURN_DEBUG (burn,
-				    "Some compulsory flags were forgotten. Corrected to ",
-				   (retval & compulsory),
-				    compulsory);
-
-		brasero_burn_session_add_flag (priv->session, compulsory);
-	}
-
-	/* we check flags consistency 
-	 * NOTE: should we return an error if they are not consistent? */
-	if ((type.type != BRASERO_TRACK_TYPE_AUDIO
-	&&   type.type != BRASERO_TRACK_TYPE_DATA
-	&&   type.type != BRASERO_TRACK_TYPE_DISC)
-	||   brasero_burn_session_is_dest_file (priv->session)) {
-		if (retval & BRASERO_BURN_FLAG_MERGE) {
-			BRASERO_BURN_DEBUG (burn, "Inconsistent flag: you can't use flag merge");
-			retval &= ~BRASERO_BURN_FLAG_MERGE;
-		}
-			
-		if (retval & BRASERO_BURN_FLAG_APPEND) {
-			BRASERO_BURN_DEBUG (burn, "Inconsistent flags: you can't use flag append");
-			retval &= ~BRASERO_BURN_FLAG_APPEND;
-		}
-
-		if (retval & BRASERO_BURN_FLAG_NO_TMP_FILES) {
-			BRASERO_BURN_DEBUG (burn, "Inconsistent flag: you can't use flag on_the_fly");
-			retval &= ~BRASERO_BURN_FLAG_NO_TMP_FILES;
-		}
-	}
-
-	if (brasero_burn_session_is_dest_file (priv->session)
-	&& (retval & BRASERO_BURN_FLAG_DONT_CLEAN_OUTPUT) == 0) {
-		BRASERO_BURN_DEBUG (burn, "Forgotten flag: you must use flag dont_clean_output");
-		retval |= BRASERO_BURN_FLAG_DONT_CLEAN_OUTPUT;
+		retval |= compulsory;
+		BRASERO_BURN_LOG_FLAGS (retval, "Some compulsory flags were forgotten. Corrected to");
 	}
 
 	brasero_burn_session_set_flags (priv->session, retval);
@@ -2029,22 +2068,6 @@ brasero_burn_record_session (BraseroBurn *burn,
 	if (type == BRASERO_CHECKSUM_NONE)
 		return BRASERO_BURN_OK;
 
-	/* unlock dest drive that's necessary if we want to check burnt medias
-	 * it seems that the kernel caches its contents and can't/don't update
-	 * its caches after a blanking/recording. */
-	/* NOTE: that work if the disc had not been mounted before. That's the 
-	 * mount that triggers the caches. So maybe if the disc was blank (and
-	 * therefore couldn't have been previously mounted) we could skip that
-	 * unlock/eject step. A better way would be to have a system call to 
-	 * force a re-load. */
-
-/*	result = brasero_burn_eject_dest_media (burn, error);
-	if (result != BRASERO_BURN_OK)
-		return result;
-
-	priv->dest = NULL;
-*/
-
 	if (type == BRASERO_CHECKSUM_MD5
 	||  type == BRASERO_CHECKSUM_SHA1
 	||  type == BRASERO_CHECKSUM_SHA256) {
@@ -2086,13 +2109,8 @@ brasero_burn_record_session (BraseroBurn *burn,
 	 * anymore. BraseroBurnSession refs it. */
 	brasero_track_unref (track);
 
-	/* reload media */
-/*	result = brasero_burn_lock_checksum_media (burn, error);
-	if (result != BRASERO_BURN_OK)
-		return result;
-*/
-	/* this may be necessary for the drive to settle down */
-	/* and possibly be mounted by gnome-volume-manager (just temporarily) */
+	/* this may be necessary for the drive to settle down
+	 * and possibly be mounted by gnome-volume-manager (just temporarily) */
 	result = brasero_burn_sleep (burn, 5000);
 	if (result != BRASERO_BURN_OK)
 		return result;
@@ -2103,17 +2121,47 @@ brasero_burn_record_session (BraseroBurn *burn,
 	if (type == BRASERO_CHECKSUM_MD5
 	||  type == BRASERO_CHECKSUM_SHA1
 	||  type == BRASERO_CHECKSUM_SHA256) {
-		guint track_num;
+		BraseroMedia media;
 		BraseroDrive *drive;
 		BraseroMedium *medium;
 
-		/* get the last track number */
+		/* get the last written track address in case of DVD+RW/DVD-RW
+		 * restricted overwrite since there is no such thing as track
+		 * number for these drives. */
 		drive = brasero_burn_session_get_burner (priv->session);
 		medium = brasero_drive_get_medium (drive);
-		track_num = brasero_medium_get_track_num (medium);
+		media = brasero_medium_get_status (medium);
 
-		BRASERO_BURN_LOG ("Last written track num == %i", track_num);
-		brasero_track_set_drive_track (track, track_num);
+		if (!BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
+		&&  !BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
+			guint track_num;
+
+			track_num = brasero_medium_get_track_num (medium);
+
+			BRASERO_BURN_LOG ("Last written track num == %i", track_num);
+			brasero_track_set_drive_track (track, track_num);
+		}
+		else {
+			GValue *value;
+
+			value = g_new0 (GValue, 1);
+			g_value_init (value, G_TYPE_UINT64);
+
+			BRASERO_BURN_LOG ("Start of last written track address == %lli", priv->session_start);
+			g_value_set_uint64 (value, priv->session_start);
+			brasero_track_tag_add (track,
+					       BRASERO_TRACK_MEDIUM_ADDRESS_START_TAG,
+					       value);
+
+			value = g_new0 (GValue, 1);
+			g_value_init (value, G_TYPE_UINT64);
+
+			BRASERO_BURN_LOG ("End of last written track address == %lli", priv->session_end);
+			g_value_set_uint64 (value, priv->session_end);
+			brasero_track_tag_add (track,
+					       BRASERO_TRACK_MEDIUM_ADDRESS_END_TAG,
+					       value);
+		}
 	}
 
 	result = brasero_burn_check_real (burn, track, error);
@@ -2175,21 +2223,23 @@ brasero_burn_check (BraseroBurn *self,
 
 	brasero_burn_powermanagement (self, FALSE);
 
-	/* no need to check the result of the comparison, it's set in session */
-	priv->session = NULL;
-	g_object_unref (session);
-
 	if (result == BRASERO_BURN_OK)
 		result = brasero_burn_unlock_medias (self, error);
 	else
 		brasero_burn_unlock_medias (self, NULL);
 
+	/* no need to check the result of the comparison, it's set in session */
+
+	/* NOTE: unref session only AFTER drives are unlocked */
+	priv->session = NULL;
+	g_object_unref (session);
+
 	return result;
 }
 
 static BraseroBurnResult
-brasero_burn_same_src_dest (BraseroBurn *self,
-			    GError **error)
+brasero_burn_same_src_dest_image (BraseroBurn *self,
+				  GError **error)
 {
 	gchar *toc = NULL;
 	gchar *image = NULL;
@@ -2279,6 +2329,78 @@ end:
 	return result;
 }
 
+static BraseroBurnResult
+brasero_burn_same_src_dest_reload_medium (BraseroBurn *burn,
+					  GError **error)
+{
+	BraseroBurnError berror;
+	BraseroBurnPrivate *priv;
+	BraseroBurnResult result;
+	BraseroMedia required_media;
+	BraseroBurnFlag session_flags;
+
+	priv = BRASERO_BURN_PRIVATE (burn);
+	/* Now there is the problem of flags... This really is a special
+	 * case. we need to try to adjust the flags to the media type
+	 * just after a new one is detected. For example there could be
+	 * BURNPROOF/DUMMY whereas they are not supported for DVD+RW. So
+	 * we need to be lenient. */
+
+	/* Eject and ask the user to reload a disc */
+	required_media = brasero_burn_caps_get_required_media_type (priv->caps,
+								    priv->session);
+	required_media &= (BRASERO_MEDIUM_WRITABLE|
+			   BRASERO_MEDIUM_CD|
+			   BRASERO_MEDIUM_DVD|
+			   BRASERO_MEDIUM_DVD_DL);
+
+	/* There is sometimes no way to determine which type of media is
+	 * required since some flags (that will be adjusted afterwards)
+	 * prevent to reach some media type like BURNPROOF and DVD+RW. */
+	if (required_media == BRASERO_MEDIUM_NONE)
+		required_media = BRASERO_MEDIUM_WRITABLE;
+
+	/* save the flags in case we modify them */
+	session_flags = brasero_burn_session_get_flags (priv->session);
+	berror = BRASERO_BURN_WARNING_INSERT_AFTER_COPY;
+
+again:
+
+	result = brasero_burn_ask_for_dest_media (burn,
+						  berror,
+						  required_media,
+						  error);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	/* update the flags now before locking it since in lock function
+	 * we check the adequacy of the medium inserted. */
+	result = brasero_burn_check_session_consistency (burn, error);
+	if (result == BRASERO_BURN_CANCEL)
+		return result;
+
+	if (result != BRASERO_BURN_OK) {
+		/* Tell the user his/her disc is not supported and reload */
+		berror = BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
+		brasero_burn_session_set_flags (priv->session, session_flags);
+		goto again;
+	}
+
+	/* One thing could make us fail now that flags and media type are
+	 * supported: the size. */
+	result = brasero_burn_lock_dest_media (burn, &berror, error);
+	if (result == BRASERO_BURN_CANCEL)
+		return result;
+
+	if (result != BRASERO_BURN_OK) {
+		/* Tell the user his/her disc is not supported and reload */
+		brasero_burn_session_set_flags (priv->session, session_flags);
+		goto again;
+	}
+
+	return BRASERO_BURN_OK;
+}
+
 BraseroBurnResult 
 brasero_burn_record (BraseroBurn *burn,
 		     BraseroBurnSession *session,
@@ -2302,22 +2424,39 @@ brasero_burn_record (BraseroBurn *burn,
 
 	if (brasero_burn_session_same_src_dest_drive (session)) {
 		/* This is a special case */
-		result = brasero_burn_same_src_dest (burn, error);
+		result = brasero_burn_same_src_dest_image (burn, error);
 		if (result != BRASERO_BURN_OK)
 			goto end;
 
-		/* lock the dest drive do it this way for the message */
-		result = brasero_burn_reload_dest_media (burn,
-							 BRASERO_BURN_WARNING_INSERT_AFTER_COPY,
-							 error);
+		result = brasero_burn_same_src_dest_reload_medium (burn, error);
 		if (result != BRASERO_BURN_OK)
 			goto end;
 	}
 	else if (!brasero_burn_session_is_dest_file (session)) {
+		BraseroBurnError berror;
+
 		/* do some drive locking quite early to make sure we have a
 		 * media in the drive so that we'll have all the necessary
 		 * information */
-		result = brasero_burn_lock_dest_media (burn, error);
+		result = brasero_burn_lock_dest_media (burn, &berror, error);
+		while (result == BRASERO_BURN_NEED_RELOAD) {
+			BraseroMedia required_media;
+
+			required_media = brasero_burn_caps_get_required_media_type (priv->caps,
+										    priv->session);
+			if (required_media == BRASERO_MEDIUM_NONE)
+				required_media = BRASERO_MEDIUM_WRITABLE;
+
+			result = brasero_burn_ask_for_dest_media (burn,
+								  berror,
+								  required_media,
+								  error);
+			if (result == BRASERO_BURN_OK)
+				goto end;
+
+			result = brasero_burn_lock_dest_media (burn, &berror, error);
+		}
+
 		if (result != BRASERO_BURN_OK)
 			goto end;
 	}
@@ -2328,7 +2467,8 @@ brasero_burn_record (BraseroBurn *burn,
 			goto end;
 	}
 
-	/* burn the session a first time whatever the number of copies required except if dummy session */
+	/* burn the session a first time whatever the number of copies required 
+	 * except if dummy session */
 	result = brasero_burn_record_session (burn, TRUE, error);
 	if (result == BRASERO_BURN_OK) {
 		gint num_copies;
@@ -2508,6 +2648,11 @@ brasero_burn_cancel (BraseroBurn *burn, gboolean protect)
 
 	priv = BRASERO_BURN_PRIVATE (burn);
 
+	if (priv->timeout_id) {
+		g_source_remove (priv->timeout_id);
+		priv->timeout_id = 0;
+	}
+
 	if (priv->sleep_loop) {
 		g_main_loop_quit (priv->sleep_loop);
 		priv->sleep_loop = NULL;
@@ -2524,6 +2669,11 @@ brasero_burn_finalize (GObject *object)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (object);
 
+	if (priv->timeout_id) {
+		g_source_remove (priv->timeout_id);
+		priv->timeout_id = 0;
+	}
+
 	if (priv->sleep_loop) {
 		g_main_loop_quit (priv->sleep_loop);
 		priv->sleep_loop = NULL;
