@@ -152,11 +152,24 @@ brasero_cdrdao_read_stderr_record (BraseroCdrdao *cdrdao, const gchar *line)
 		brasero_job_set_dangerous (BRASERO_JOB (cdrdao), TRUE);
 	}
 	else {
-		gchar *cuepath, *name;
-		BraseroTrack *track;
+		gchar *name = NULL;
+		gchar *cuepath = NULL;
+		BraseroTrack *track = NULL;
+		BraseroJobAction action;
+
+		/* Try to catch error could not find cue file */
+
+		/* Track could be NULL here if we're simply blanking a medium */
+		brasero_job_get_action (BRASERO_JOB (cdrdao), &action);
+		if (action == BRASERO_JOB_ACTION_ERASE)
+			return TRUE;
 
 		brasero_job_get_current_track (BRASERO_JOB (cdrdao), &track);
+		if (!track)
+			return FALSE;
+
 		cuepath = brasero_track_get_toc_source (track, FALSE);
+
 		if (!cuepath)
 			return FALSE;
 
@@ -595,9 +608,12 @@ brasero_cdrdao_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (input);
 
 	/* cdrdao is used to burn images so it can't APPEND and the disc must
-	 * have been blanked before (it can't overwrite) */
+	 * have been blanked before (it can't overwrite)
+	 * NOTE: BRASERO_MEDIUM_FILE is needed here because of restriction API
+	 * when we output an image. */
 	brasero_plugin_set_flags (plugin,
-				  media_w,
+				  media_w|
+				  BRASERO_MEDIUM_FILE,
 				  BRASERO_BURN_FLAG_DAO|
 				  BRASERO_BURN_FLAG_BURNPROOF|
 				  BRASERO_BURN_FLAG_OVERBURN|
