@@ -239,8 +239,9 @@ brasero_genisoimage_set_argv_image (BraseroGenisoimage *genisoimage,
 		g_ptr_array_add (argv, g_strdup ("-iso-level"));
 		g_ptr_array_add (argv, g_strdup ("3"));
 
-		/* NOTE the following is specific to genisoimage */
-		/* g_ptr_array_add (argv, g_strdup ("-allow-limited-size")); */
+		/* NOTE: the following is specific to genisoimage
+		 * It allows to burn files over 4 GiB */
+		g_ptr_array_add (argv, g_strdup ("-allow-limited-size"));
 	}
 
 	if (type.subtype.fs_type & BRASERO_IMAGE_FS_UDF)
@@ -517,7 +518,6 @@ brasero_genisoimage_export_caps (BraseroPlugin *plugin, gchar **error)
 	}
 	g_free (prog_name);
 
-	/* NOTE: we don't include DVDRW+ DVDRW- restricted in here */
 	brasero_plugin_set_flags (plugin,
 				  BRASERO_MEDIUM_CDR|
 				  BRASERO_MEDIUM_CDRW|
@@ -525,7 +525,6 @@ brasero_genisoimage_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_MEDIUM_DVDRW|
 				  BRASERO_MEDIUM_DVD_DL|
 				  BRASERO_MEDIUM_DVDR_PLUS|
-				  BRASERO_MEDIUM_RESTRICTED|
 				  BRASERO_MEDIUM_APPENDABLE|
 				  BRASERO_MEDIUM_HAS_AUDIO|
 				  BRASERO_MEDIUM_HAS_DATA,
@@ -533,6 +532,17 @@ brasero_genisoimage_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_BURN_FLAG_MERGE,
 				  BRASERO_BURN_FLAG_NONE);
 
+	brasero_plugin_set_flags (plugin,
+				  BRASERO_MEDIUM_DVD_DL|
+				  BRASERO_MEDIUM_DVDRW_PLUS|
+				  BRASERO_MEDIUM_RESTRICTED|
+				  BRASERO_MEDIUM_APPENDABLE|
+				  BRASERO_MEDIUM_CLOSED|
+				  BRASERO_MEDIUM_HAS_DATA,
+				  BRASERO_BURN_FLAG_APPEND|
+				  BRASERO_BURN_FLAG_MERGE,
+				  BRASERO_BURN_FLAG_NONE);
+
 	input = brasero_caps_data_new (BRASERO_IMAGE_FS_ISO|
 				       BRASERO_IMAGE_FS_UDF|
 				       BRASERO_IMAGE_ISO_FS_LEVEL_3|
