@@ -43,6 +43,7 @@
 
 #include <gconf/gconf-client.h>
 
+#include "scsi-device.h"
 #include "burn-plugin.h"
 #include "burn-job.h"
 #include "burn-checksum-files.h"
@@ -290,18 +291,15 @@ brasero_checksum_file_process_former_line (BraseroChecksumFiles *self,
 	guint i;
 	gchar *path;
 	GSList *grafts;
-	gchar *checksum;
 	guint written_bytes;
 	BraseroChecksumFilesPrivate *priv;
 
 	priv = BRASERO_CHECKSUM_FILES_PRIVATE (self);
 
-	/* first read the checksum string */
+	/* first skip the checksum string */
 	i = 0;
 	while (!isspace (line [i])) i ++;
 
-	checksum = g_strndup (line, i);
-
 	/* skip white spaces */
 	while (isspace (line [i])) i ++;
 
@@ -315,15 +313,21 @@ brasero_checksum_file_process_former_line (BraseroChecksumFiles *self,
 		/* NOTE: graft->path + 1 is because in the checksum files on the 
 		 * disc there is not first "/" so if we want to compare ... */
 		graft = grafts->data;
-		if (!strcmp (graft->path + 1, path))
+		if (!strcmp (graft->path + 1, path)) {
+			g_free (path);
 			return BRASERO_BURN_OK;
+		}
 
 		len = strlen (graft->path + 1);
 		if (!strncmp (graft->path + 1, path, len)
-		&&   path [len] == G_DIR_SEPARATOR)
+		&&   path [len] == G_DIR_SEPARATOR) {
+			g_free (path);
 			return BRASERO_BURN_OK;
+		}
 	}
 
+	g_free (path);
+
 	/* write the whole line in the new file */
 	written_bytes = fwrite (line, 1, strlen (line), priv->file);
 	if (written_bytes != strlen (line)) {
@@ -351,6 +355,7 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 {
 	BraseroBurnFlag flags = BRASERO_BURN_FLAG_NONE;
 	BraseroChecksumFilesPrivate *priv;
+	BraseroDeviceHandle *dev_handle;
 	BraseroVolFileHandle *handle;
 	BraseroBurnResult result;
 	BraseroVolFile *file;
@@ -374,9 +379,11 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 		return result;
 
 	/* try every file and make sure they are of the same type */
-	/* FIXME: if not we could make a new checksum ... */
 	brasero_job_get_device (BRASERO_JOB (self), &device);
-	vol = brasero_volume_source_open_file (device, error);
+	dev_handle = brasero_device_handle_open (device, NULL);
+	g_free (device);
+
+	vol = brasero_volume_source_open_device_handle (dev_handle, error);
 	file = brasero_volume_get_file (vol,
 					"/"BRASERO_MD5_FILE,
 					start_block,
@@ -393,13 +400,11 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 							start_block,
 							NULL);
 			if (!file) {
-				g_free (device);
 				brasero_volume_source_close (vol);
 				BRASERO_JOB_LOG (self, "no checksum file found");
 				return BRASERO_BURN_OK;
 			}
 			else if (priv->checksum_type != BRASERO_CHECKSUM_SHA256_FILE) {
-				g_free (device);
 				brasero_volume_source_close (vol);
 				BRASERO_JOB_LOG (self, "checksum type mismatch (%i against %i)",
 						 priv->checksum_type,
@@ -412,12 +417,10 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 					 priv->checksum_type,
 					 BRASERO_CHECKSUM_SHA1_FILE);
 			brasero_volume_source_close (vol);
-			g_free (device);
 			return BRASERO_BURN_OK;
 		}
 	}
 	else if (priv->checksum_type != BRASERO_CHECKSUM_MD5_FILE) {
-		g_free (device);
 		brasero_volume_source_close (vol);
 		BRASERO_JOB_LOG (self, "checksum type mismatch (%i against %i)",
 				 priv->checksum_type,
@@ -425,17 +428,14 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 		return BRASERO_BURN_OK;
 	}
 
-	BRASERO_JOB_LOG (self, "Found file %s on %s", file, device);
-	brasero_volume_source_open_device_path (device, error);
-	if (!vol)
-		return BRASERO_BURN_ERR;
-
+	BRASERO_JOB_LOG (self, "Found file %s", file);
 	handle = brasero_volume_file_open (vol, file);
 	brasero_volume_source_close (vol);
-	g_free (device);
 
 	if (!handle) {
 		BRASERO_JOB_LOG (self, "Failed to open file");
+		brasero_device_handle_close (dev_handle);
+		brasero_volume_file_free (file);
 		return BRASERO_BURN_ERR;
 	}
 
@@ -448,12 +448,19 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 	while (result == BRASERO_BURN_RETRY) {
 		if (priv->cancel) {
 			brasero_volume_file_close (handle);
+			brasero_volume_file_free (file);
+			brasero_device_handle_close (dev_handle);
 			return BRASERO_BURN_CANCEL;
 		}
 
-		result = brasero_checksum_file_process_former_line (self, track, buffer, error);
+		result = brasero_checksum_file_process_former_line (self,
+								    track,
+								    buffer,
+								    error);
 		if (result != BRASERO_BURN_OK) {
 			brasero_volume_file_close (handle);
+			brasero_volume_file_free (file);
+			brasero_device_handle_close (dev_handle);
 			return result;
 		}
 
@@ -462,6 +469,8 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 
 	result = brasero_checksum_file_process_former_line (self, track, buffer, error);
 	brasero_volume_file_close (handle);
+	brasero_volume_file_free (file);
+	brasero_device_handle_close (dev_handle);
 
 	return result;
 }
@@ -686,8 +695,8 @@ brasero_checksum_files_check_files (BraseroChecksumFiles *self,
 	if (!root)
 		return BRASERO_BURN_ERR;
 
-	memcpy (filename, root, sizeof (filename));
 	root_len = strlen (root);
+	memcpy (filename, root, root_len);
 	filename [root_len ++ ] = '/';
 
 	name = brasero_track_get_checksum (track);
@@ -716,6 +725,11 @@ brasero_checksum_files_check_files (BraseroChecksumFiles *self,
 	}
 
 	file_num = 0;
+	brasero_job_set_current_action (BRASERO_JOB (self),
+				        BRASERO_BURN_ACTION_CHECKSUM,
+					_("Checking file integrity"),
+					TRUE);
+	brasero_job_start_progress (BRASERO_JOB (self), FALSE);
 
 	/* Get the checksum type */
 	switch (checksum_type) {
@@ -937,14 +951,18 @@ brasero_checksum_files_end (gpointer data)
 			graft->path = g_strdup ("/"BRASERO_MD5_FILE);
 			break;
 		}
+
+		BRASERO_JOB_LOG (self,
+				 "Adding graft for checksum file %s %s",
+				 graft->path,
+				 graft->uri);
+
 		new_grafts = g_slist_prepend (new_grafts, graft);
+		excluded = brasero_track_get_data_excluded_source (track, TRUE);
 
 		track = brasero_track_new (BRASERO_TRACK_TYPE_DATA);
 		brasero_track_add_data_fs (track, type.subtype.fs_type);
-
-		excluded = brasero_track_get_data_excluded_source (track, TRUE);
 		brasero_track_set_data_source (track, new_grafts, excluded);
-
 		brasero_track_set_checksum (track,
 					    priv->checksum_type,
 					    graft->uri);
