@@ -51,8 +51,8 @@
 BRASERO_PLUGIN_BOILERPLATE (BraseroGrowisofs, brasero_growisofs, BRASERO_TYPE_PROCESS, BraseroProcess);
 
 struct BraseroGrowisofsPrivate {
-	gint64 sectors_num;
 	guint use_utf8:1;
+	guint use_genisoimage:1;
 };
 typedef struct BraseroGrowisofsPrivate BraseroGrowisofsPrivate;
 
@@ -238,6 +238,15 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 	gchar *emptydir = NULL;
 	gchar *videodir = NULL;
 
+	priv = BRASERO_GROWISOFS_PRIVATE (growisofs);
+
+	if (priv->use_genisoimage) {
+		BRASERO_JOB_LOG (growisofs, "Using genisoimage");
+	}
+	else {
+		BRASERO_JOB_LOG (growisofs, "Using mkisofs");
+	}
+
 	g_ptr_array_add (argv, g_strdup ("-r"));
 
 	brasero_job_get_current_track (BRASERO_JOB (growisofs), &track);
@@ -252,6 +261,13 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 		 * like MacOSX and freebsd.*/
 		g_ptr_array_add (argv, g_strdup ("-iso-level"));
 		g_ptr_array_add (argv, g_strdup ("3"));
+
+		/* NOTE: the following is specific to genisoimage
+		 * It allows to burn files over 4 GiB.
+		 * The only problem here is which are we using? mkisofs or
+		 * genisoimage? That's what we determined first. */
+		if (priv->use_genisoimage)
+			g_ptr_array_add (argv, g_strdup ("-allow-limited-size"));
 	}
 
 	if (input.subtype.fs_type & BRASERO_IMAGE_FS_UDF)
@@ -267,7 +283,6 @@ brasero_growisofs_set_mkisofs_argv (BraseroGrowisofs *growisofs,
 			return result;
 	}
 
-	priv = BRASERO_GROWISOFS_PRIVATE (growisofs);
 	if (priv->use_utf8) {
 		g_ptr_array_add (argv, g_strdup ("-input-charset"));
 		g_ptr_array_add (argv, g_strdup ("utf8"));
@@ -627,22 +642,53 @@ static void
 brasero_growisofs_init (BraseroGrowisofs *obj)
 {
 	BraseroGrowisofsPrivate *priv;
-	gchar *standard_error;
-	gboolean res;
+	gchar *standard_error = NULL;
 	gchar *prog_name;
+	gboolean res;
 
 	priv = BRASERO_GROWISOFS_PRIVATE (obj);
 
-	/* this code comes from ncb_mkisofs_supports_utf8 */
-	
-	prog_name = g_find_program_in_path ("genisoimage");
-        if (prog_name && g_file_test (prog_name, G_FILE_TEST_IS_EXECUTABLE)) {
-	  	res = g_spawn_command_line_sync ("genisoimage -input-charset utf8", NULL, &standard_error, NULL, NULL);
-	} else {
-                res = g_spawn_command_line_sync ("mkisofs -input-charset utf8", NULL, &standard_error, NULL, NULL);
-	}
+	/* this code (remotely) comes from ncb_mkisofs_supports_utf8 */
+	/* Added a way to detect whether we'll use mkisofs or genisoimage */
 
-       	if (res && !g_strrstr (standard_error, "Unknown charset"))
+	prog_name = g_find_program_in_path ("mkisofs");
+        if (prog_name && g_file_test (prog_name, G_FILE_TEST_IS_EXECUTABLE)) {
+		gchar *standard_output = NULL;
+
+		res = g_spawn_command_line_sync ("mkisofs -version",
+						 &standard_output,
+						 NULL,
+						 NULL,
+						 NULL);
+		if (res) {
+			/* Really make sure it is mkisofs and not a symlink */
+			if (standard_output && strstr (standard_output, "genisoimage"))
+				priv->use_genisoimage = TRUE;
+
+			if (standard_output)
+				g_free (standard_output);
+		}
+		else
+			priv->use_genisoimage = TRUE;
+	}
+	else
+		priv->use_genisoimage = TRUE;
+
+	/* Don't use BRASERO_JOB_LOG () here!! */
+	if (priv->use_genisoimage)
+		res = g_spawn_command_line_sync ("genisoimage -input-charset utf8",
+						 NULL,
+						 &standard_error,
+						 NULL,
+						 NULL);
+	else
+	  	res = g_spawn_command_line_sync ("mkisofs -input-charset utf8",
+						 NULL,
+						 &standard_error,
+						 NULL,
+						 NULL);
+
+	if (res && !g_strrstr (standard_error, "Unknown charset"))
 		priv->use_utf8 = TRUE;
 	else
 		priv->use_utf8 = FALSE;
@@ -742,7 +788,7 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (output);
 
 	/* growisofs has the possibility to record to closed DVD+RW/-restricted
-	 * and to append some more data to them which makes it unique */
+	 * and to append some more data to them which makes them unique */
 	output = brasero_caps_disc_new (BRASERO_MEDIUM_DVD|
 					BRASERO_MEDIUM_DVD_DL|
 					BRASERO_MEDIUM_PLUS|
@@ -754,183 +800,20 @@ brasero_growisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (output);
 	g_slist_free (input);
 
-	/* For DVD-W and DVD-RW sequential
-	 * NOTE: DAO et MULTI are exclusive. */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVD|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_SEQUENTIAL|
-				  BRASERO_MEDIUM_JUMP|
-				  BRASERO_MEDIUM_WRITABLE|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_UNFORMATTED|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVD|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_SEQUENTIAL|
-				  BRASERO_MEDIUM_JUMP|
-				  BRASERO_MEDIUM_WRITABLE|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_UNFORMATTED|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVD|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_SEQUENTIAL|
-				  BRASERO_MEDIUM_JUMP|
-				  BRASERO_MEDIUM_WRITABLE|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_APPEND|
-				  BRASERO_BURN_FLAG_MERGE,
-				  BRASERO_BURN_FLAG_NONE);
+	/* For DVD-W and DVD-RW sequential */
+	BRASERO_PLUGIN_ADD_STANDARD_DVDR_FLAGS (plugin);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_FLAGS (plugin);
 
 	/* see NOTE for DVD-RW restricted overwrite below */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVD|
-				  BRASERO_MEDIUM_RESTRICTED|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_UNFORMATTED|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE,
-				  BRASERO_BURN_FLAG_MULTI);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_RESTRICTED_FLAGS (plugin);
 
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVD|
-				  BRASERO_MEDIUM_RESTRICTED|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_CLOSED|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE,
-				  BRASERO_BURN_FLAG_MULTI);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVD|
-				  BRASERO_MEDIUM_RESTRICTED|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_CLOSED|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_MERGE,
-				  BRASERO_BURN_FLAG_MULTI);
-
-	/* DVD+ R/RW don't support dummy mode */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDR_PLUS|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDR_PLUS|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDR_PLUS|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_APPEND|
-				  BRASERO_BURN_FLAG_MERGE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	/* NOTE: growisofs doesn't support APPEND flag with DVD+RW. DVD+RW and 
-	 * other overwrite media have only one session and therefore you can't
-	 * add another session after (APPEND). Now growisofs is still able to 
-	 * merge more data nevertheless. */
+	/* DVD+ W */
+	BRASERO_PLUGIN_ADD_STANDARD_DVDR_PLUS_FLAGS (plugin);
 
 	/* for DVD+RW */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDRW_PLUS|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_UNFORMATTED|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_MULTI);
+	BRASERO_PLUGIN_ADD_STANDARD_DVDRW_PLUS_FLAGS (plugin);
 
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDRW_PLUS|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_CLOSED|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE,
-				  BRASERO_BURN_FLAG_MULTI);
-
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_DVDRW_PLUS|
-				  BRASERO_MEDIUM_DVD_DL|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_CLOSED|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_MERGE,
-				  BRASERO_BURN_FLAG_MULTI);
-
-	/* blank caps for +/restricted RW*/
+	/* blank caps for +/restricted RW */
 	output = brasero_caps_disc_new (BRASERO_MEDIUM_DVD|
 					BRASERO_MEDIUM_DVD_DL|
 					BRASERO_MEDIUM_PLUS|
